--                    -*- Mode: Ada -*-
-- Filename        : opengl-models-forms.adb
-- Description     : Package for the definition of form-models. 
-- Author          : H. Altmann, A. Gaisbauer, J. Schrer
-- Created On      : 09. Feb. 1998
-- Last Modified By: J. Schrer
-- Last Modified On: 09.04.1998
-- Update Count    : 2
-- Status          : 

with Win32_Addons.Gl;
with Opengl.Float_32.Elementary_Functions;
with Opengl.Float_32.Arrays.Vector_Operations;
with Opengl.Float_32.Arrays.Types;

package body Opengl.Models.Forms is

  use Opengl.Float_32.Elementary_Functions;
      -- to make sin, cos, etc directly visible.
  use Opengl.Float_32.Arrays.Vector_Operations;
      -- to make vector-operators directly visible.
  use type Opengl.Float_32.Arrays.Types.Vector;
  use type Gl.Glfloat;
  use type Gl.Gldouble;
  use type Glu.Pgluquadricobj;
  use type Types.Points_2d;
  use type Gl.Glboolean;
  ----------------------------------------------------------------------------       
  procedure Render_2d_Line(From, To : in Types.Vector_2d) is
  begin
    Gl.Glbegin(Gl.Gl_Lines);
    Gl.Glvertex2f(From(X), From(Y));
    Gl.Glvertex2f(To(X),   To(Y));
    Gl.Glend;
  end Render_2d_Line;
  ----------------------------------------------------------------------------
  procedure Render_3d_Line(From, To : in Types.Vector_3d) is
  begin
    Gl.Glbegin(Gl.Gl_Lines);
    Gl.Glvertex3f(From(X), From(Y), From(Z));
    Gl.Glvertex3f(To(X),   To(Y),   To(Z));
    Gl.Glend;
  end Render_3d_Line;
  ----------------------------------------------------------------------------
  procedure Render    (The_Points : in out Types.Points_2d) is
  begin
    for I in The_Points'range loop
    	Gl.Glvertex2f(X => The_Points(I)(X), Y => The_Points(I)(Y));
    end loop;
  end Render;
  ----------------------------------------------------------------------------
  procedure Render(The_Square_Width : in Gl.Glfloat;
    							 Angle            : in Gl.Glfloat := 0.0) is
    Half_Width : constant Gl.Glfloat := The_Square_Width / 2.0;
  begin
    Gl.Glpushmatrix;
    Gl.Glrotatef(Angle => Angle, X => 0.0, Y => 0.0, Z => 1.0);
    for Count in 1 .. Render_Count loop
	    Gl.Glrectf(X1 => - Half_Width,
	               Y1 =>   Half_Width,
	               X2 =>   Half_Width,
	               Y2 => - Half_Width);
    end loop;
    Gl.Glpopmatrix;
  end Render;
  ---------------------------------------------------------------------------                     
  procedure Render(The_Rectangle : in Rectangle;
    							 Angle         : in Gl.Glfloat := 0.0) is
  begin
    Gl.Glpushmatrix;
    Gl.Glrotatef(Angle => Angle, X => 0.0, Y => 0.0, Z => 1.0);
    for Count in 1 .. Render_Count loop
	    Gl.Glrectf(X1 => The_Rectangle.Lower_Left (X),
	               Y1 => The_Rectangle.Upper_Right(Y),
	               X2 => The_Rectangle.Upper_Right(X),
	               Y2 => The_Rectangle.Lower_Left (Y));
    end loop;
    Gl.Glpopmatrix;
  end Render;
  ---------------------------------------------------------------------------
  function Display_List_Of(Rectangle : in Forms.Rectangle;
                           Color     : in Types.Color_Type) return Gl.Gluint is
                                 
    The : constant Gl.Gluint := Display_Lists.Next_List_Id;                              
  begin
    Gl.Glnewlist(The, Mode => Gl.Gl_Compile);    
    Set_Color(Color, Force => True);
    Render_Count := 2;
    Render(Rectangle);
    Render_Count := 1;
    Gl.Glendlist;
    return The;
  end Display_List_Of;
  --------------------------------------------------------------------------                        
  function Display_List_Model_Of(Rectangle : in Forms.Rectangle;
                                 Color     : in Types.Color_Type) 
           return Display_List_Model_2d_Z is
             
    The : Display_List_Model_2d_Z;                              
  begin
    The.Display_List := Display_List_Of(Rectangle, Color);
    return The;
  end Display_List_Model_Of;
  --------------------------------------------------------------------------                        
  function rectangle_and_frame
          (Position         : in Types.Vector_3d  := (X => 0.0, Y => 0.0, z => 0.0);
           attitude         : in gl.glfloat       := 0.0;
           Width            : in gl.glfloat       := screen.width;
           Height           : in Gl.Glfloat       := screen.height;
           Frame_Line_Width : in Gl.Glfloat       := 1.0;
           Background_Color : in Types.Color_Type := Colors.Black;
           Frame_Color      : in Types.Color_Type := Colors.White) 
    return rectangle_and_frame_2d is    
  begin
    return rectangle_and_frame_2d'
          (ada.finalization.controlled with
           size_scaling     => 1.0,
           color            => Background_Color,
           position         => position,
           attitude         => attitude,
           Width            => width,
           Height           => height,
           Frame_Line_Width => Frame_Line_Width,
           Frame_Color      => frame_color);                               
  end Rectangle_And_Frame;    
  --------------------------------------------------------------------------                        
  procedure render(the_model : in out rectangle_and_frame_2d) is
  
    Half_Width  : constant gl.glfloat := the_model.Width  / 2.0;         
    Half_Height : constant gl.glfloat := the_model.Height / 2.0;         
    Rectangle   : Forms.Rectangle := 
                 (Lower_Left  => the_model.Position(x .. y) + 
                                (X => - Half_Width,           
                                 Y => - Half_Height),
                  Upper_Right => the_model.Position(x .. y) + 
                                (X =>   Half_Width,
                                 Y =>   Half_Height));
  begin
    -- Background
    Set_Color(the_model.Color, Force => True);
    Render_Count := 2;
    Render(Rectangle);
    Render_Count := 1;
    
    if the_model.Frame_Line_Width > 0.0 then
      -- Lower frame line.
      Set_Color(the_model.Frame_Color, Force => True);
      Rectangle.Lower_Left (Y) := the_model.Position(Y) - Half_Height - 
                                  the_model.Frame_Line_Width; 
      Rectangle.Upper_Right(Y) := the_model.Position(Y) - Half_Height; 
      Render(Rectangle);
      -- Upper frame line.        
      Rectangle.Lower_Left (Y) := the_model.Position(Y) + Half_Height; 
      Rectangle.Upper_Right(Y) := the_model.Position(Y) + Half_Height + 
                                  the_model.Frame_Line_Width;         
      Render(Rectangle);
      -- Left frame line
      Rectangle := 
        (Lower_Left  => the_model.Position(x .. y) + 
                       (X => - Half_Width  - the_model.Frame_Line_Width, 
                        Y => - Half_Height - the_model.Frame_Line_Width),
         Upper_Right => the_model.Position(x .. y) + 
                       (X => - Half_Width, 
                        Y =>   Half_Height + the_model.Frame_Line_Width));
      Render(Rectangle);
      -- Right frame line
      Rectangle.Lower_Left (X) := the_model.Position(X) + Half_Width; 
      Rectangle.Upper_Right(X) := the_model.Position(X) + Half_Width + 
                                  the_model.Frame_Line_Width;         
      Render(Rectangle);
    end if;  
  end render;
  --------------------------------------------------------------------------                        
  function Rectangle_And_Frame
          (Position         : in Types.Vector_2d  := (X => 0.0, Y => 0.0);
           Width            : in gl.glfloat       := screen.width;
           Height           : in Gl.Glfloat       := screen.height;
           Frame_Line_Width : in Gl.Glfloat       := 1.0;
           Background_Color : in Types.Color_Type := Colors.Black;
           Frame_Color      : in Types.Color_Type := Colors.White) 
    return Gl.Gluint is

    obj : rectangle_and_frame_2d := rectangle_and_frame
          (Position         => (X => position(x), Y => position(y), z => 0.0),
           attitude         => 0.0,
           Width            => width,
           Height           => height,
           Frame_Line_Width => Frame_Line_Width,
           Background_Color => Background_Color,
           Frame_Color      => Frame_Color);
           
    The : constant Gl.Gluint := Display_Lists.Next_List_Id;                              
  begin
    Gl.Glnewlist(The, Mode => Gl.Gl_Compile);    
    render(obj);
    Gl.Glendlist;      
    return The;
  end Rectangle_And_Frame;    
  --------------------------------------------------------------------------              
  procedure Initialize(The_Model : in out Rectangle_2d_Z) is
  begin
    The_Model.Points := Rectangle'
      (Lower_Left  => - 0.5 * Types.Vector_2d(The_Model.Dimension),
       Upper_Right =>   0.5 * Types.Vector_2d(The_Model.Dimension));
  end Initialize;
  ---------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Rectangle_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ---------------------------------------------------------------------------
  procedure Render(The_Model : in out Square_2d_Z) is
    Half_Width : constant Gl.Glfloat := The_Model.Width / 2.0;
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Gl.Glrectf(X1 => - Half_Width,
	               Y1 =>   Half_Width,
	               X2 =>   Half_Width,
	               Y2 => - Half_Width);
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Render    (The_Model : in out Rectangle_2d_Z) is
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Gl.Glrectf(X1 => The_Model.Points.Lower_Left (X),
	               Y1 => The_Model.Points.Upper_Right(Y),
	               X2 => The_Model.Points.Upper_Right(X),
	               Y2 => The_Model.Points.Lower_Left (Y));
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Striped_Rectangle_2d_Z) is
  begin
    The_Model.Hidden_Data := 
      (Points           => Rectangle'(Lower_Left  => (0.0, 0.0),
                                      Upper_Right => The_Model.Size_Scaling * 
                                                     (1.0, The_Model.Height)),
       Stripe_Width     => The_Model.Stripe_Width   * The_Model.Size_Scaling,
       Stripe_Spacing   => The_Model.Stripe_Spacing * The_Model.Size_Scaling,
       First_Stripe     => The_Model.First_Stripe   * The_Model.Size_Scaling,
       X_Offset         => The_Model.Height / Tan(The_Model.Stripe_Angle) *
                           The_Model.Size_Scaling,
       Number           => 2 + Natural(1.0 / (The_Model.Stripe_Width + 
                                              The_Model.Stripe_Spacing)),                                              
       Left_Clip_Plane  => Clip_Planes.Clip_Plane
                          (Axis    => X,
                           Visible => Clip_Planes.Positive_Direction,
                           Cutoff  => 0.0),
       Right_Clip_Plane => Clip_Planes.Clip_Plane
                          (Axis    => X,
                           Visible => Clip_Planes.Negative_Direction,
                           Cutoff  => The_Model.Size_Scaling));
  end Initialize;
  ---------------------------------------------------------------------------        
  procedure Adjust    (The_Model : in out Striped_Rectangle_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ---------------------------------------------------------------------------
  procedure Render    (The_Model : in out Striped_Rectangle_2d_Z) is
  begin
    Initialize_Render(The_Model);
    Clip_Planes.Enable(The_Model.Hidden_Data.Left_Clip_Plane);
    Clip_Planes.Enable(The_Model.Hidden_Data.Right_Clip_Plane);
    
    Gl.Glrectf(X1 => The_Model.Hidden_Data.Points.Lower_Left (X),
	             Y1 => The_Model.Hidden_Data.Points.Upper_Right(Y),
	             X2 => The_Model.Hidden_Data.Points.Upper_Right(X),
	             Y2 => The_Model.Hidden_Data.Points.Lower_Left (Y));
    Set_Color(The_Model.Stripe_Color);
    Translate_X(The_Model.Hidden_Data.First_Stripe);
    
    for I in 1 .. The_Model.Hidden_Data.Number loop
      Gl.Glbegin(Gl.Gl_Quads);
        Gl.Glvertex2f(X => 0.0, Y => 0.0);
	      Gl.Glvertex2f(X => The_Model.Hidden_Data.X_Offset,                
                      Y => The_Model.Hidden_Data.Points.Upper_Right(Y));
	      Gl.Glvertex2f(X => The_Model.Hidden_Data.X_Offset + 
                           The_Model.Hidden_Data.Stripe_Width, 
                      Y => The_Model.Hidden_Data.Points.Upper_Right(Y));
	      Gl.Glvertex2f(X => The_Model.Hidden_Data.Stripe_Width, Y => 0.0);
      Gl.Glend;
      if I < The_Model.Hidden_Data.Number then
        Translate_X(The_Model.Hidden_Data.Stripe_Width + 
                    The_Model.Hidden_Data.Stripe_Spacing);
      end if;
    end loop;
    
    Clip_Planes.Disable(2);
    Finalize_Render(The_Model);    
  end Render;
  ---------------------------------------------------------------------------
  procedure Render    (The_Triangle : in out Triangle) is
  begin
    Gl.Glbegin(Gl.Gl_Triangles);
    for Count in 1 .. Render_Count loop
    	Gl.Glvertex2f(X => The_Triangle.Points(1)(X), 
                    Y => The_Triangle.Points(1)(Y));
      Gl.Glvertex2f(X => The_Triangle.Points(2)(X), 
                    Y => The_Triangle.Points(2)(Y));
      Gl.Glvertex2f(X => The_Triangle.Points(3)(X), 
                    Y => The_Triangle.Points(3)(Y));
    end loop;
    Gl.Glend;
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Triangle_2d_Z) is
  begin
    null;
    --    The_Model.Points := The_Model.Size_Scaling * The_Model.Points;
    -- JS, 18.3.98: added to avoid accumulating calculations in initialize
    --    The_Model.Size_Scaling := 1.0;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Triangle_2d_Z) is
  begin
  	Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Triangle_2d_Z) is
  begin
    Initialize_Render(The_Model);
    Gl.Glbegin(Gl.Gl_Triangles);
    for Count in 1 .. Render_Count loop
    	Gl.Glvertex2f(X => The_Model.Points(1)(X), Y => The_Model.Points(1)(Y));
      Gl.Glvertex2f(X => The_Model.Points(2)(X), Y => The_Model.Points(2)(Y));
      Gl.Glvertex2f(X => The_Model.Points(3)(X), Y => The_Model.Points(3)(Y));
    end loop;
    Gl.Glend;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Equal_Sided_Triangle_2d_Z) is
    Top   : Positive := 3;
    Lower : Positive := 2;
  begin
		-- exchange Top with lower, front is counter-clock
    if The_Model.Base_Radius > The_Model.Peak_Radius then
      Top   := 2;
      Lower := 3;
    end if;
    The_Model.Points(1)     := The_Model.Size_Scaling *
                               (X => The_Model.Base_Radius,
                                Y => abs(The_Model.Peak_Radius - 
                                         The_Model.Base_Radius) / Sqrt(3.0));
  	The_Model.Points(Lower) := (X =>   The_Model.Points(1)(X),
                                Y => - The_Model.Points(1)(Y));
  	The_Model.Points(Top)   := (X => The_Model.Size_Scaling *
                                     The_Model.Peak_Radius,
                                Y => 0.0);
  end Initialize;
  ---------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Equal_Sided_Triangle_2d_Z) is
  begin
  	Initialize(The_Model);
  end Adjust;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Symmetric_Triangle_2d_Z) is
    Top   : Positive := 3;
    Lower : Positive := 2;
  begin
		-- exchange Top with Right, front is counter-clock
    if The_Model.Base_Radius > The_Model.Peak_Radius then
      Top   := 2;
      Lower := 3;
    end if;
    The_Model.Points(1)     := The_Model.Size_Scaling *
                               (X => The_Model.Base_Radius,
                                Y => The_Model.Base_Width / 2.0);
  	The_Model.Points(Lower) := (X =>   The_Model.Points(1)(X),
                                Y => - The_Model.Points(1)(Y));
  	The_Model.Points(Top)   := (X => The_Model.Size_Scaling *
                                     The_Model.Peak_Radius,
                                Y => 0.0);
  end Initialize;
  ---------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Symmetric_Triangle_2d_Z) is
  begin
  	Initialize(The_Model);
  end Adjust;
  ---------------------------------------------------------------------------
  procedure Render    (The_Polygon : in out Polygon_Type) is
  begin
    Gl.Glbegin(Gl.Gl_Polygon);
  	for I in The_Polygon.Points'range loop
      Gl.Glvertex2f(X => The_Polygon.Points(I)(X),
                    Y => The_Polygon.Points(I)(Y));
    end loop;
    Gl.Glend;
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Polygon_2d_Z) is
  begin
    The_Model.Points := The_Model.Size_Scaling * The_Model.Points;
    -- JS, 18.3.98: added to avoid accumulating calculations in initialize
    The_Model.Size_Scaling := 1.0;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Polygon_2d_Z) is
  begin
  	Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Polygon_2d_Z) is
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
    	Gl.Glbegin(Gl.Gl_Polygon);
    		for I in The_Model.Points'range loop
    			Gl.Glvertex2f(X => The_Model.Points(I)(X),
                        Y => The_Model.Points(I)(Y));
      	end loop;
    	Gl.Glend;
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Prepare(The_Model : in out Striped_Polygon_2d_Z) is

    type Rgb_Color is array (0..2) of Gl.Glubyte;
    -----------------------------------------------------------------    
    function To_Rgb (Color : in Types.Color_Type) return Rgb_Color is
    begin
      return (0 => Gl.Glubyte(Color.Red   * 255.0),
         	    1 => Gl.Glubyte(Color.Green * 255.0),
              2 => Gl.Glubyte(Color.Blue  * 255.0));
    end To_Rgb;
    ----------------------------------------------------------
    Image1d : array(0 .. 7) of Rgb_Color :=
      	       (0 .. 3 => To_Rgb(The_Model.Color),
		  	        others => To_Rgb(The_Model.Stripe_Color));
    Spacing : Gl.Glfloat;
  begin
    if Win32_Addons.Gl.Glistexture(The_Model.Texture_Id) /= Gl.Gl_True then
	    Gl.Glpixelstorei(Gl.Gl_Unpack_Alignment, 1);
    	Win32_Addons.Gl.Glgentextures(1, The_Model.Texture_Id'Unchecked_Access);
    	Win32_Addons.Gl.Glbindtexture(Gl.Gl_Texture_1d, The_Model.Texture_Id);
    	Gl.Glhint(Gl.Gl_Perspective_Correction_Hint,Gl.Gl_Nicest);
	    Gl.Gltexparameteri(Target => Gl.Gl_Texture_1d,
                         Pname  => Gl.Gl_Texture_Mag_Filter,
                         Param  => Gl.Gl_Nearest);
	    Gl.Gltexparameteri(Target => Gl.Gl_Texture_1d,
                         Pname  => Gl.Gl_Texture_Min_Filter,
                         Param  => Gl.Gl_Linear);
      Gl.Gltexparameteri(Target => Gl.Gl_Texture_1d,
                         Pname  => Gl.Gl_Texture_Wrap_S,
                         Param  => Gl.Gl_Repeat);
	    Gl.Gltexparameterf(Target => Gl.Gl_Texture_1d,
                         Pname  => Win32_Addons.Gl.Gl_Texture_Priority,
                         Param  => 1.0);
	    Gl.Glteximage1d(Target 		 => Gl.Gl_Texture_1d,
	              	    Level  		 => 0,
			                Components => Rgb_Color'Length,--3,
			                Width  		 => Image1d'Length,--8,
			                Border 		 => 0,
			                Format 		 => Gl.Gl_Rgb,
			                C_Type 		 => Gl.Gl_Unsigned_Byte,
			                Pixels 		 => Image1d'Address);
    	-- stripe per Unit    
	    Spacing := The_Model.Stripe_Number / The_Model.Size_Scaling;
	    The_Model.Gen_Param := (0 => -Sin(The_Model.Stripe_Angle,360.0) * Spacing,
					                    1 =>  Cos(The_Model.Stripe_Angle,360.0) * Spacing,
	            		            others => 0.0);
	    -- stripe width
	    -- The_Model.Gen_Param := (0 => (0.5 / The_Model.Size_Scaling) / The_Model.Spacing, 
	    --            	   			   1 => (0.5 / The_Model.Size_Scaling) / The_Model.Spacing,
	    --                         others => 0.0);    
 	    Initialize(Polygon_2d_Z(The_Model));
    end if;
  end Prepare;
  ----------------------------------------------------------------------------;
--  procedure Adjust    (The_Model : in out Polygon_Textured_2d_Z) is
--  begin
--  	Initialize(The_Model);    
--  end Adjust;
  ----------------------------------------------------------------------------;
  procedure Render    (The_Model : in out Striped_Polygon_2d_Z) is
    Anti_Aliasing : constant Gl.Glboolean :=
                    Gl.Glisenabled(Gl.Gl_Polygon_Smooth);
  begin
    if Anti_Aliasing = Gl.Gl_True then
  	  Gl.Gldisable(Gl.Gl_Polygon_Smooth);
    end if;
    -- automatic Texture coordinate generation
    Gl.Glhint(Gl.Gl_Perspective_Correction_Hint, Gl.Gl_Nicest);
    Gl.Gltexenvi (Target => Gl.Gl_Texture_Env,
                  Pname  => Gl.Gl_Texture_Env_Mode,
                  Param  => Gl.Gl_Decal);
    Gl.Gltexgeni (Coord  => Gl.Gl_S,
                  Pname  => Gl.Gl_Texture_Gen_Mode,
                  Param  => Gl.Gl_Object_Linear);
    Gl.Gltexgenfv(Coord  => Gl.Gl_S,
                  Pname  => Gl.Gl_Object_Plane,
                  Params => The_Model.Gen_Param(0)'Unchecked_Access);
    Gl.Glenable(Gl.Gl_Texture_1d);
    Gl.Glenable(Gl.Gl_Texture_Gen_S);
    Win32_Addons.Gl.Glbindtexture(Gl.Gl_Texture_1d, The_Model.Texture_Id);
    -- workaround Gloria XL - antialiasing textured polygons
    -- draw polygon outline, then textured polygon without antialiasing
    Gl.Gllinewidth(1.0);
    Gl.Glpolygonmode(Gl.Gl_Front_And_Back, Gl.Gl_Line);
    Render(Polygon_2d_Z(The_Model));
    Gl.Glpolygonmode(Gl.Gl_Front_And_Back, Gl.Gl_Fill);
    Render(Polygon_2d_Z(The_Model));
    Gl.Gldisable(Gl.Gl_Texture_Gen_S);
    Gl.Gldisable(Gl.Gl_Texture_1d);
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Glenable(Gl.Gl_Polygon_Smooth);
    end if;
  end Render;
  ---------------------------------------------------------------------------
	procedure Render(The_Model : in out Line_List_2d_Z) is
    Half_Square_Width : constant Gl.Glfloat := The_Model.Line_Width / 6.0;
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    case The_Model.Kind is
        when Gl_Points     => 
          Gl.Glpointsize(The_Model.Line_Width);
	        Gl.Glbegin(Gl.Gl_Points);
        when Gl_Line_Strip => 
          Gl.Gllinewidth(The_Model.Line_Width);
	        Gl.Glbegin(Gl.Gl_Line_Strip);
        when Gl_Line_Loop  => 
          for I in The_Model.Points'range loop
            Gl.Glpushmatrix;
            Gl.Gltranslatef(X => The_Model.Points(I)(X),
                            Y => The_Model.Points(I)(Y),
                            Z => 0.0);      
            Gl.Glrectf(X1 => - Half_Square_Width,
    	                 Y1 =>   Half_Square_Width,
    	                 X2 =>   Half_Square_Width,
    	                 Y2 => - Half_Square_Width);
            Gl.Glpopmatrix;
          end loop;           
          Gl.Gllinewidth(The_Model.Line_Width);
          Gl.Glbegin(Gl.Gl_Line_Loop);        
        when Gl_Lines      =>
          Gl.Gllinewidth(The_Model.Line_Width);
	        Gl.Glbegin(Gl.Gl_Lines);
	    end case;
	    for I in The_Model.Points'range loop
	      Gl.Glvertex2f(X => The_Model.Points(I)(X),
	                    Y => The_Model.Points(I)(Y));
      end loop;
	    Gl.Glend;      
	  end loop;
    Finalize_Render(The_Model);
  end Render;
  ----------------------------------------------------------------------------
  procedure Initialize(The_Model: in out Line_List_2d_Z) is
  begin
    if not The_Model.Initialized and then The_Model.Size_Scaling /= 1.0 then      
      The_Model.Points      := The_Model.Points * The_Model.Size_Scaling;
      The_Model.Position    := The_Model.Position * The_Model.Size_Scaling;
      The_Model.Line_Width  := The_Model.Line_Width * The_Model.Size_Scaling;
      The_Model.Initialized := True;
    end if;  
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust(The_Model: in out Line_List_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Circle_2d_Z) is

    Points_4 : constant Positive := The_Model.Point_Number / 4;
    D_Phi    : constant Gl.Glfloat := 90.0 / Gl.Glfloat(Points_4);
    Phi      : Gl.Glfloat;
  begin
    The_Model.Points := new Types.Points_2d(1 .. Points_4);
    for I in The_Model.Points'range loop
      Phi := Gl.Glfloat(I) * D_Phi;
      The_Model.Points(I) := The_Model.Size_Scaling *
                             (X => Cos(Phi, Cycle => 360.0),
                              Y => Sin(Phi, Cycle => 360.0));
    end loop;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Render(The_Model : in out Circle_2d_Z) is

    subtype Sign_Type is Gl.Glfloat range -1.0 .. 1.0;
    ----------------------------------------------------------
    procedure Render_Quarter(X_Sign, Y_Sign : in Sign_Type) is
    begin
      for I in The_Model.Points'range loop
        Gl.Glvertex2f(X => X_Sign * The_Model.Points(I)(X),
                      Y => Y_Sign * The_Model.Points(I)(Y));
      end loop;
    end Render_Quarter;
    -------------------
  begin
    Initialize_Render(The_Model);

    Gl.Glpolygonmode(Gl.Gl_Front_And_Back, Gl.Gl_Line);
    Gl.Glbegin(Gl.Gl_Polygon);

    Render_Quarter(X_Sign =>   1.0, Y_Sign =>   1.0);
    Render_Quarter(X_Sign => - 1.0, Y_Sign =>   1.0);
    Render_Quarter(X_Sign => - 1.0, Y_Sign => - 1.0);
    Render_Quarter(X_Sign =>   1.0, Y_Sign => - 1.0);

    Gl.Glend;
    Gl.Glpolygonmode(Gl.Gl_Front_And_Back, Gl.Gl_Fill);
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Disk_2d_Z) is
  begin
    if The_Model.Glu_Quadric_Object = null then
      The_Model.Glu_Quadric_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Disk_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Finalize  (The_Model : in out Disk_2d_Z) is
  begin
    null;
--    Glu.Gludeletequadric(The_Model.Glu_Quadric_Object);
  end Finalize;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Disk_2d_Z) is
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Glu.Gludisk(Qobj        => The_Model.Glu_Quadric_Object,
	                Innerradius => Gl.Gldouble(The_Model.Size_Scaling *
	                                           The_Model.Inner_Radius),
	                Outerradius => Gl.Gldouble(The_Model.Size_Scaling *
	                                           The_Model.Outer_Radius),
	                Slices      => Gl.Glint(The_Model.Slices),
	                Loops       => Gl.Glint(The_Model.Loops));
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Partial_Disk_2d_Z) is
  begin
    if The_Model.Glu_Quadric_Object = null then
      The_Model.Glu_Quadric_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Partial_Disk_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Finalize  (The_Model : in out Partial_Disk_2d_Z) is
  begin
    null;
--    Glu.Gludeletequadric(The_Model.Glu_Quadric_Object);
  end Finalize;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Partial_Disk_2d_Z) is
    Anti_Aliasing : constant Gl.Glboolean :=
                    Gl.Glisenabled(Gl.Gl_Polygon_Smooth);
  begin
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Gldisable(Gl.Gl_Polygon_Smooth);
    end if;
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Glu.Glupartialdisk(Qobj        => The_Model.Glu_Quadric_Object,
				                 Innerradius => Gl.Gldouble(The_Model.Size_Scaling *
	                                                  The_Model.Inner_Radius),
				                 Outerradius => Gl.Gldouble(The_Model.Size_Scaling *
	                                                  The_Model.Outer_Radius),
				                 Slices      => Gl.Glint(The_Model.Slices),
				                 Loops       => Gl.Glint(The_Model.Loops),
				                 Startangle  => Gl.Gldouble(The_Model.Start_Angle),
				                 Sweepangle  => Gl.Gldouble(The_Model.Sweep_Angle));
	  end loop;
    Finalize_Render(The_Model);
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Glenable(Gl.Gl_Polygon_Smooth);
    end if;
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Arc_2d_Z) is
  begin
    if The_Model.Glu_Quadric_Object = null then
      The_Model.Glu_Quadric_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Arc_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Finalize  (The_Model : in out Arc_2d_Z) is
  begin
    null;
--    Glu.Gludeletequadric(The_Model.Glu_Quadric_Object);
  end Finalize;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Arc_2d_Z) is
    Anti_Aliasing : constant Gl.Glboolean :=
                    Gl.Glisenabled(Gl.Gl_Polygon_Smooth);
    Width         : constant Gl.Glfloat :=
                    (The_Model.Outer_Radius - The_Model.Inner_Radius) * 0.90;
  begin
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Gldisable(Gl.Gl_Polygon_Smooth);
    end if;
    Initialize_Render(The_Model);
    -- workaround Gloria XL - antialiasing polygons
    -- draw polygon without antialiasing, then polygon with antialiasing
    Glu.Glupartialdisk(Qobj        => The_Model.Glu_Quadric_Object,
								       Innerradius => Gl.Gldouble((The_Model.Outer_Radius - Width) *
                                                 	 The_Model.Size_Scaling),
								       Outerradius => Gl.Gldouble((The_Model.Inner_Radius + Width) *
                                           		     The_Model.Size_Scaling),
									     Slices      => Gl.Glint(The_Model.Slices),
								       Loops       => Gl.Glint(The_Model.Loops),
								       Startangle  => Gl.Gldouble(90.0 - The_Model.Start_Angle),
								       Sweepangle  => Gl.Gldouble(The_Model.Start_Angle -
                                                  The_Model.End_Angle));

    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Glenable(Gl.Gl_Polygon_Smooth);
    end if;
    Glu.Glupartialdisk(Qobj        => The_Model.Glu_Quadric_Object,
								       Innerradius => Gl.Gldouble(The_Model.Size_Scaling *
								                                  The_Model.Inner_Radius),
								       Outerradius => Gl.Gldouble(The_Model.Size_Scaling *
								                                  The_Model.Outer_Radius),
									     Slices      => Gl.Glint(The_Model.Slices),
								       Loops       => Gl.Glint(The_Model.Loops),
								       Startangle  => Gl.Gldouble(90.0 - The_Model.Start_Angle),
								       Sweepangle  => Gl.Gldouble(The_Model.Start_Angle -
                                                  The_Model.End_Angle));
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Render    (The_Model : in out Dashed_Arc_2d_Z) is
    Anti_Aliasing : constant Gl.Glboolean :=
                    Gl.Glisenabled(Gl.Gl_Polygon_Smooth);
    Dash_Number   : constant Natural := 
                    Natural(abs((The_Model.Start_Angle - The_Model.End_Angle) /
                                 The_Model.Dash_Angle)) / 2;
    Inner_Radius  : constant Gl.Gldouble := Gl.Gldouble
                    (The_Model.Size_Scaling * The_Model.Inner_Radius);
	  Outer_Radius   : constant Gl.Gldouble := Gl.Gldouble
                    (The_Model.Size_Scaling * The_Model.Outer_Radius);
	  Start_Angle   : Gl.Gldouble := Gl.Gldouble(90.0 - The_Model.Start_Angle);
    Angle_Inc     : constant Gl.Gldouble := Gl.Gldouble
                    (2.0 * The_Model.Dash_Angle);                
  begin
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Gldisable(Gl.Gl_Polygon_Smooth);
    end if;
    Initialize_Render(The_Model);
    for I in 1 .. Dash_Number loop
      Glu.Glupartialdisk(Qobj        => The_Model.Glu_Quadric_Object,
  								       Innerradius => Inner_Radius,
  								       Outerradius => Outer_Radius,
  									     Slices      => Gl.Glint(The_Model.Slices),
  								       Loops       => Gl.Glint(The_Model.Loops),
  								       Startangle  => Start_Angle,
  								       Sweepangle  => Gl.Gldouble(The_Model.Dash_Angle));    
      Start_Angle := Start_Angle - Angle_Inc;                   
    end loop;                   
    Finalize_Render(The_Model);
    if Anti_Aliasing = Gl.Gl_True then
	    Gl.Glenable(Gl.Gl_Polygon_Smooth);
    end if;
  end Render;
  ---------------------------------------------------------------------------      
  procedure Initialize(The_Model : in out Cylinder_3d) is
  begin
    if The_Model.Glu_Quadric_Object = null then
      The_Model.Glu_Quadric_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Cylinder_3d) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Finalize  (The_Model : in out Cylinder_3d) is
  begin
    null;
--    Glu.Gludeletequadric(The_Model.Glu_Quadric_Object);
  end Finalize;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Cylinder_3d) is
  begin
    Initialize_Render(The_Model);
    Glu.Glucylinder (Qobj       => The_Model.Glu_Quadric_Object,
							       Baseradius => Gl.Gldouble(The_Model.Base_Radius),
							       Topradius  => Gl.Gldouble(The_Model.Top_Radius),
							       Height     => Gl.Gldouble(The_Model.Height),
							       Slices     => Gl.Glint(The_Model.Slices),
							       Stacks     => Gl.Glint(The_Model.Stacks));
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Sphere_3d) is
  begin
    if The_Model.Glu_Quadric_Object = null then
      The_Model.Glu_Quadric_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Sphere_3d) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Finalize  (The_Model : in out Sphere_3d) is
  begin
    null;
--    Glu.Gludeletequadric(The_Model.Glu_Quadric_Object);
  end Finalize;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Sphere_3d) is
  begin
    Initialize_Render(The_Model);
    Glu.Glusphere (Qobj   => The_Model.Glu_Quadric_Object,
    				       Radius => Gl.Gldouble(The_Model.Radius),
					         Slices => Gl.Glint(The_Model.Slices),
					         Stacks => Gl.Glint(The_Model.Stacks));
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Radial_Arrow_2d_Z) is

    The_Triangle  : Symmetric_Triangle_2d_Z :=
                   (Ada.Finalization.Controlled with
                    Size_Scaling => The_Model.Size_Scaling,
                    Color        => The_Model.Color,
                    Position     => (0.0, 0.0, 0.0),
                    Attitude     => The_Model.Attitude - 90.0,
                    Points       => (others => (X => 0.0, Y => 0.0)),
                    Base_Radius  => The_Model.Triangle_Base_Radius,
                    Peak_Radius  => The_Model.Triangle_Peak_Radius,
                    Base_Width   => The_Model.Triangle_Base_Width);
    The_Rectangle : Rectangle_2d_Z :=
                   (Ada.Finalization.Controlled with
                    Size_Scaling => The_Model.Size_Scaling,
                    Color        => The_Model.Color,
                    Position     => (X => The_Model.Size_Scaling *
                                         (The_Model.Inner_Radius +
                                          The_Model.Triangle_Base_Radius) / 2.0,
                                     Y => 0.0,
                                     Z => 0.0),
                    Attitude     => The_Model.Attitude,
                    Dimension    => Types.Rectangle
                                   (The_Model.Size_Scaling * Types.Vector_2d'
                                    (X => The_Model.Triangle_Base_Radius -
                                          The_Model.Inner_Radius,
                                     Y => The_Model.Rectangle_Width)),
                    Points       => Default_Rectangle);
  begin
    Initialize(The_Triangle);
    Initialize(The_Rectangle);
    for I in The_Triangle.Points'range loop
      The_Triangle.Points(I) :=
        (X => The_Triangle.Points(I)(Y), Y => The_Triangle.Points(I)(X));
    end loop;
--    for I in The_Triangle.Points'range loop
--      The_Model.Parts.Triangle_Part.Points(I) := 
--        (X => The_Triangle.Points(I)(Y), Y => The_Triangle.Points(I)(X));
--    end loop;    
--    The_Model.Parts.Rectangle_Part       := 
--      (Lower_Left  => The_Rectangle.Points.Lower_Left  + 
--                      (The_Rectangle.Position(X), 0.0),
--       Upper_Right => The_Rectangle.Points.Upper_Right + 
--                      (The_Rectangle.Position(X), 0.0));
    The_Model.Parts.Display_List := Display_Lists.Next_List_Id;
    Gl.Glnewlist(List => The_Model.Parts.Display_List,
                 Mode => Gl.Gl_Compile);
    Motion_Sequence := Rotate_First;
    Render(The_Rectangle);
    Motion_Sequence := Translate_First;
    Render(The_Triangle);
    Gl.Glendlist;
  end Initialize;
  ----------------------------------------------------------------------------                     
  procedure Adjust    (The_Model : in out Radial_Arrow_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Radial_Arrow_2d_Z) is
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Gl.Glcalllist(The_Model.Parts.Display_List);
--    Render(The_Model.Parts.Triangle_Part);
--    Render(The_Model.Parts.Rectangle_Part);    
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ---------------------------------------------------------------------------
  procedure Initialize(The_Model : in out Round_Arrow_2d_Z) is

    Arc_End_Angle : constant Gl.Glfloat :=
      The_Model.Arc_Start_Angle + The_Model.Arc_Delta_Angle;
    Arc_Radius    : constant Gl.Glfloat :=
      The_Model.Arc_Radius * The_Model.Size_Scaling;
  begin
    The_Model.Parts.Triangle_Part := new Symmetric_Triangle_2d_Z'
	     (Ada.Finalization.Controlled with
	      Size_Scaling => The_Model.Size_Scaling,
	      Color        => The_Model.Color,
	      Position     => (X => Cos(Arc_End_Angle, Cycle => 360.0) * Arc_Radius,
                         Y => Sin(Arc_End_Angle, Cycle => 360.0) * Arc_Radius,
                         Z => 0.0),
	      Attitude     => Arc_End_Angle - 90.0,
	      Points       => (others => (X => 0.0, Y => 0.0)),
	      Base_Radius  => 0.0,
	      Peak_Radius  => The_Model.Triangle_Height,
	      Base_Width   => The_Model.Triangle_Base_Width);

    Initialize(The_Model.Parts.Triangle_Part.all);

    if The_Model.Arc_Delta_Angle < 0.0 then
      The_Model.Parts.Triangle_Part.Attitude :=
        The_Model.Parts.Triangle_Part.Attitude - 180.0;
    end if;

    The_Model.Parts.Arc_Inner_Radius := The_Model.Size_Scaling *
      (The_Model.Arc_Radius - The_Model.Arc_Width / 2.0);
    The_Model.Parts.Arc_Outer_Radius := The_Model.Size_Scaling *
      (The_Model.Arc_Radius + The_Model.Arc_Width / 2.0);
    The_Model.Parts.Arc_Slices       :=
      Positive(abs The_Model.Arc_Delta_Angle / 10.0) + 1;

    if The_Model.Parts.Arc_Object = null then
      The_Model.Parts.Arc_Object := Glu.Glunewquadric;
    end if;
  end Initialize;
  ----------------------------------------------------------------------------
  procedure Adjust    (The_Model : in out Round_Arrow_2d_Z) is
  begin
    Initialize(The_Model);
  end Adjust;
  ----------------------------------------------------------------------------
  procedure Render    (The_Model : in out Round_Arrow_2d_Z) is
  begin
    Initialize_Render(The_Model);
    for Count in 1 .. Render_Count loop
	    Render(The_Model.Parts.Triangle_Part.all);
	    Glu.Glupartialdisk(Qobj        => The_Model.Parts.Arc_Object,
									       Innerradius => Gl.Gldouble(The_Model.Parts.Arc_Inner_Radius),
									       Outerradius => Gl.Gldouble(The_Model.Parts.Arc_Outer_Radius),
									       Slices      => Gl.Glint(The_Model.Parts.Arc_Slices),
									       Loops       => 1,
									       Startangle  => Gl.Gldouble(90.0 - The_Model.Arc_Start_Angle),
									       Sweepangle  => Gl.Gldouble(- The_Model.Arc_Delta_Angle));
    end loop;
    Finalize_Render(The_Model);
  end Render;
  ----------------------------------------------------------------------------
end Opengl.Models.Forms;
