--                    -*- Mode: Ada -*-
-- Filename        : opengl-win_stuff.adb
-- Description     : see spec.
-- Author          : J. Schrer
-- Created On      : 10. Feb. 1998
-- Last Modified By: J. Schrer
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Ada.Exceptions;
with Ada.Strings.Fixed;
with Win32.Winbase;
with Win32.Winuser;
with Win32.Wingdi;
with Opengl.Messages;

package body Opengl.Win_Stuff is
  
  use type Win32.Dword;
  use type Win32.Bool;
  ----------------------------------------------------------------------------
  function Font_Weight_Bold return Font_Weight_Type is 
  begin
    return Win32.Wingdi.Fw_Bold;
  end Font_Weight_Bold;
  ----------------------------------------------------------------------------    
  function Font_Weight_Normal return Font_Weight_Type is 
  begin
    return Win32.Wingdi.Fw_Normal;
  end Font_Weight_Normal;
  ----------------------------------------------------------------------------        
  function Font_Weight_Thin return Font_Weight_Type is 
  begin
    return Win32.Wingdi.Fw_Thin;
  end Font_Weight_Thin;
  ----------------------------------------------------------------------------        
  function Default_Font_Name return String is
  begin
    return "Arial";
--    return "Helvetica";
--    return "Times Roman";
--    return "Courier New"
  end Default_Font_Name;    
  ----------------------------------------------------------------------------
  function Screen_Width_In_Mm  return Screen_Size_In_Mm is
  begin
--    return Screen_Size_In_Mm(Win32.Winuser.Getsystemmetrics(Win32.Winuser.Sm_Cxfullscreen)); 
    return 360.0;
  end Screen_Width_In_Mm;
  ----------------------------------------------------------------------------
  function Screen_Height_In_Mm return Screen_Size_In_Mm is
  begin
--    return Screen_Size_In_Mm(Win32.Winuser.Getsystemmetrics(Win32.Winuser.Sm_Cyfullscreen)); 
    return 275.0;
  end Screen_Height_In_Mm;
  ----------------------------------------------------------------------------  
  function Screen_Pixel_Width  return Natural is
  begin
    return Natural(Win32.Winuser.Getsystemmetrics(Win32.Winuser.Sm_Cxscreen));
  end Screen_Pixel_Width;
  ----------------------------------------------------------------------------
  function Screen_Pixel_Height return Natural is
  begin
    return Natural(Win32.Winuser.Getsystemmetrics(Win32.Winuser.Sm_Cyscreen));
  end Screen_Pixel_Height;
  ----------------------------------------------------------------------------
  function Max_Clip_Planes return Natural is
    Number : aliased Gl.Glint := 0;
  begin
    Gl.Glgetintegerv(Gl.Gl_Max_Clip_Planes, Number'Unchecked_Access);          
    return Natural(Number);
  end Max_Clip_Planes;
  ----------------------------------------------------------------------------  
  function Os_Info return String is  
    Version_No : constant Win32.Dword := Win32.Winbase.Getversion;
    Is_Winnt   : constant Boolean := (Version_No and 
                                      Win32.Dword(16#80000000#)) = 0;
    Version    : constant String := "Version " &
      	         Win32.Byte'Image(Win32.Windef.Lobyte
                   (Win32.Windef.Loword(Version_No))) & "." & 
                   Ada.Strings.Fixed.Trim(Win32.Byte'Image
                     (Win32.Windef.Hibyte(Win32.Windef.Loword(Version_No))),
                      Side => Ada.Strings.Both);
  begin
    if Is_Winnt then
  		return "Windows NT " & Version;
    else
      return "Windows " & Version;
    end if;
  end Os_Info;
  ----------------------------------------------------------------------------  
  function Area_Of(The_Window : in Win32.Windef.Hwnd) return Win32.Windef.Rect is
    Area : aliased Win32.Windef.Rect;
  begin
--    if Win32.Winuser.Getwindowrect
    if Win32.Winuser.Getclientrect
           (Hwnd   => The_Window,
            Lprect => Area'Unchecked_Access) = Win32.True then
              
      return Area;
    else
      Ada.Exceptions.Raise_Exception(Opengl_Error'Identity, Messages.Error_Message
        (Subprogram => "OpenGL.Win_Stuff.Area_Of",
         Message    => "Win32.Winuser.Getclientrect failed"));
--         Message    => "Win32.Winuser.Getwindowrect failed"));
         
      return Invalid_Area;
    end if;                                       
  end Area_Of;  
  ----------------------------------------------------------------------------  
end Opengl.Win_Stuff;

