--                              -*- Mode: Ada -*- 
-- Filename        : math-generics-arrays-types-io.adb
-- Description     : Declaration of output procedures for array types
-- Author          : J. Schrer
-- Created On      : Thu May 22 09:58:02 1997
-- Last Modified By: 
-- Last Modified On: Thu May 22 10:20:15 1997
-- Update Count    : 5
-- Status          :

with Math.Output;

package body Math.Generics.Arrays.Io is
  --------------------------------------------------------------------------
  procedure Put(File : in Ada.Text_Io.File_Type;
                Item : in Types.Vector;
                Name : in String := "") is

    Margin : Ada.Text_Io.Positive_Count;
  begin
    if Name = "" then
      Ada.Text_Io.Put(File, "(");
      Margin := Ada.Text_Io.Col(File);
      Math.Output.Set(First_Col => Margin);
      for I in Item'range loop
        Ada.Text_Io.Put(File, Index'Image(I) & " => ");
        Put(File, Item(I));
        if I < Item'Last then
          Ada.Text_Io.Put_Line(File, ",");
          Ada.Text_Io.Set_Col(File, Margin);
        else
          Ada.Text_Io.Put(File, ")");
        end if;
      end loop;
    else
      Ada.Text_Io.Put_Line(File,"-- Vector " & Name & " -------------------");
      for I in Item'range loop
        Ada.Text_Io.Put(File, Name & "(" & Index'Image(I) & ") := ");
        if I = Item'First then
          Margin := Ada.Text_Io.Col(File);
          Math.Output.Set(First_Col => Margin);
        end if;
        Put(File, Item(I));
        Ada.Text_Io.Put_Line(File, ";");
      end loop;
    end if;
    Math.Output.Set;
  end Put;
  --------------------------------------------------------------------------
  procedure Put(File : in Ada.Text_Io.File_Type;
                Item : in Types.Matrix;
                Name : in String := "") is

    Row_Margin, Col_Margin : Ada.Text_Io.Positive_Count;
  begin
    if Name = "" then
      Ada.Text_Io.Put(File, "(");
      Row_Margin := Ada.Text_Io.Col(File);
      for I in Item'range(1) loop
        Ada.Text_Io.Put(File, Index'Image(I) & " => (");
        Col_Margin := Ada.Text_Io.Col(File);
        if I = Item'First(1) then
          Math.Output.Set(First_Col => Col_Margin);
        end if;
        for J in Item'range(2) loop
          Ada.Text_Io.Put(File, Index'Image(J) & " => ");
          Put(File, Item(I,J));
          if J < Item'Last(2) then
            Ada.Text_Io.Put_Line(File, ",");
            Ada.Text_Io.Set_Col(File, Col_Margin);
          else
            Ada.Text_Io.Put(File, ")");
          end if;
        end loop;
        if I < Item'Last(1) then
          Ada.Text_Io.Put_Line(File, ",");
          Ada.Text_Io.Set_Col(File, Row_Margin);
        else
          Ada.Text_Io.Put(File, ")");
        end if;
      end loop;
    else
      Ada.Text_Io.Put_Line(File,"-- Matrix " & Name & " -------------------");
      for I in Item'range(1) loop
        for J in Item'range(2) loop
          Ada.Text_Io.Put(File, Name & "(" & Index'Image(I) & "," &
                                             Index'Image(J) & ") := ");
          if I = Item'First(1) and then J = Item'First(2) then
            Col_Margin := Ada.Text_Io.Col(File);
            Math.Output.Set(First_Col => Col_Margin);
          end if;
          Put(File, Item(I,J));
          Ada.Text_Io.Put_Line(File, ";");
        end loop;
      end loop;
    end if;
    Math.Output.Set;
  end Put;
  --------------------------------------------------------------------------
  procedure Put(File : in Ada.Text_Io.File_Type;
                Item : in Types.Diagonal_Matrix;
                Name : in String := "") is

    Margin : Ada.Text_Io.Positive_Count;
  begin
    if Name = "" then
      Ada.Text_Io.Put(File, "(");
      Margin := Ada.Text_Io.Col(File);
      Math.Output.Set(First_Col => Margin);
      for I in Item'range loop
        Ada.Text_Io.Put(File, Index'Image(I) & " => ");
        Put(File, Item(I));
        if I < Item'Last then
          Ada.Text_Io.Put_Line(File, ",");
          Ada.Text_Io.Set_Col(File, Margin);
        else
          Ada.Text_Io.Put(File, ")");
        end if;
      end loop;
    else
      Ada.Text_Io.Put_Line(File,"-- Diagonal_Matrix " & Name & " ----------");
      for I in Item'range loop
        Ada.Text_Io.Put(File, Name & "(" & Index'Image(I) & ") := ");
        if I = Item'First then
          Margin := Ada.Text_Io.Col(File);
          Math.Output.Set(First_Col => Margin);
        end if;
        Put(File, Item(I));
        Ada.Text_Io.Put_Line(File, ";");
      end loop;
    end if;
    Math.Output.Set;
  end Put;
  --------------------------------------------------------------------------
  procedure Put(File : in Ada.Text_Io.File_Type;
                Item : in Types.Symmetric_Matrix;
                Name : in String := "") is

    Row_Margin, Col_Margin : Ada.Text_Io.Positive_Count;
    use type Types.Vector_Access;
  begin
    if Name = "" then
      Ada.Text_Io.Put(File, "(");
      Row_Margin := Ada.Text_Io.Col(File);
      for I in Item'range loop
        if Item(I) /= null then
          Ada.Text_Io.Put(File, Index'Image(I) & " => (");
          Col_Margin := Ada.Text_Io.Col(File);
          Math.Output.Set(First_Col => Col_Margin);
          declare
            Row : Types.Vector_Access renames Item(I);
          begin
            for J in Row'range loop
              Ada.Text_Io.Put(File, Index'Image(J) & " => ");
              Put(File, Row(J));
              if J < Row'Last then
                Ada.Text_Io.Put_Line(File, ",");
                Ada.Text_Io.Set_Col(File, Col_Margin);
              else
                Ada.Text_Io.Put(File, ")");
              end if;
            end loop;
          end;
          if I < Item'Last then
            Ada.Text_Io.Put_Line(File, ",");
            Ada.Text_Io.Set_Col(File, Col_Margin);
          else
            Ada.Text_Io.Put(File, ")");
          end if;
        else
          raise Types.Symmetric_Matrix_Allocation_Error;
        end if;
      end loop;
    else
      Ada.Text_Io.Put_Line(File,"-- Symmetric_Matrix " & Name & " ---------");
      for I in Item'range loop
        if Item(I) /= null then
          declare
            Image : constant String := Name & "(" & Index'Image(I) & ")(";
            Row   : Types.Vector_Access renames Item(I);
          begin
            for J in Row'range loop
              Ada.Text_Io.Put(File, Image & Index'Image(J) & ") := ");
              if I = Item'First and then J = Row'First then
                Col_Margin := Ada.Text_Io.Col(File);
                Math.Output.Set(First_Col => Col_Margin);
              end if;
              Put(File, Row(J));
              Ada.Text_Io.Put_Line(File, ";");
            end loop;
          end;
        else
          raise Types.Symmetric_Matrix_Allocation_Error;
        end if;
      end loop;
    end if;
    Math.Output.Set;
  end Put;
  --------------------------------------------------------------------------
end Math.Generics.Arrays.Io;

