--                              -*- Mode: Ada -*-
-- Filename        : math-generics-arrays-types.adb
-- Description     : Declaration of array types
-- Author          : J. Schrer
-- Created On      : Wed May 21 19:07:46 1997
-- Last Modified By:
-- Last Modified On: Thu May 22 10:09:37 1997
-- Update Count    : 5
-- Status          :

with Ada.Unchecked_Deallocation;

package body Math.Generics.Arrays.Types is
  ---------------------------------------------------------------------------
  function  Allocate_Symmetric_Matrix(Order : in Index)
            return Symmetric_Matrix is

    X : Symmetric_Matrix(1 .. Order);
  begin
    Allocate(X);
    return X;
  end Allocate_Symmetric_Matrix;
  ---------------------------------------------------------------------------
  procedure Allocate (The_Array : in out Symmetric_Matrix) is
  begin
    for I in The_Array'range loop
      Allocate(The_Array(I), First => I, Last => The_Array'Last);
    end loop;
  end Allocate;
  --------------------------------------------------------------------------
  procedure Allocate (The_Array : in out Vector_Access;
                      Last      : in     Index;
                      First     : in     Index := 1) is
  begin
    if The_Array = null then
      The_Array := new Vector(First .. Last);
    elsif The_Array'Last /= Last or The_Array'First /= First then
      Deallocate(The_Array);
      The_Array := new Vector(First .. Last);
    end if;
    The_Array.all := (others => Zero);
  end Allocate;
  --------------------------------------------------------------------------
  procedure Allocate (The_Array      : in out Matrix_Access;
                      Last1,  Last2  : in     Index;
                      First1, First2 : in     Index := 1) is
  begin
    if The_Array = null then
      The_Array := new Matrix(First1 .. Last1, First2 .. Last2);
    elsif The_Array'Last(1)  /= Last1  or The_Array'Last(2)  /= Last2 or
          The_Array'First(1) /= First1 or The_Array'First(2) /= First2 then
      Deallocate(The_Array);
      The_Array := new Matrix(First1 .. Last1, First2 .. Last2);
    end if;
    The_Array.all := (others => (others => Zero));
  end Allocate;
  --------------------------------------------------------------------------
  procedure Allocate (The_Array : in out Diagonal_Matrix_Access;
                      Last      : in     Index;
                      First     : in     Index := 1) is
  begin
    if The_Array = null then
      The_Array := new Diagonal_Matrix(First .. Last);
    elsif The_Array'Last /= Last or The_Array'First /= First then
      Deallocate (The_Array);
      The_Array := new Diagonal_Matrix(First .. Last);
    end if;
    The_Array.all := (others => Zero);
  end Allocate;
  --------------------------------------------------------------------------
  procedure Allocate (The_Array : in out Symmetric_Matrix_Access;
                      Last      : in     Index;
                      First     : in     Index := 1) is
  begin
    if The_Array = null then
      The_Array := new Symmetric_Matrix(First .. Last);
    elsif The_Array'Last /= Last or The_Array'First /= First then
      Deallocate(The_Array);
      The_Array := new Symmetric_Matrix(First .. Last);
    end if;
    Allocate(The_Array.all);
  end Allocate;
  --------------------------------------------------------------------------
  procedure Deallocate (The_Array : in out Vector_Access) is

    procedure Deallocate_Vector is new Ada.Unchecked_Deallocation
             (Object => Vector, Name => Vector_Access);
  begin
    Deallocate_Vector(The_Array);
  end Deallocate;
  --------------------------------------------------------------------------
  procedure Deallocate (The_Array : in out Matrix_Access) is

    procedure Deallocate_Matrix is new Ada.Unchecked_Deallocation
             (Object => Matrix, Name => Matrix_Access);
  begin
    Deallocate_Matrix(The_Array);
  end Deallocate;
  --------------------------------------------------------------------------
  procedure Deallocate (The_Array : in out Diagonal_Matrix_Access) is

    procedure Deallocate_Matrix is new Ada.Unchecked_Deallocation
             (Object => Diagonal_Matrix, Name => Diagonal_Matrix_Access);
  begin
    Deallocate_Matrix(The_Array);
  end Deallocate;
  --------------------------------------------------------------------------
  procedure Deallocate (The_Array : in out Symmetric_Matrix) is
  begin
    for I in The_Array'range loop
      Deallocate(The_Array(I));
    end loop;
  end Deallocate;
  --------------------------------------------------------------------------
  procedure Deallocate (The_Array : in out Symmetric_Matrix_Access) is

    procedure Deallocate_Matrix is new Ada.Unchecked_Deallocation
             (Object => Symmetric_Matrix, Name => Symmetric_Matrix_Access);
  begin
    if The_Array /= null then
      Deallocate(The_Array.all);
      Deallocate_Matrix(The_Array);
    end if;
  end Deallocate;
  --------------------------------------------------------------------------
end Math.Generics.Arrays.Types;