--                              -*- Mode: Ada -*-
-- Filename        : math-generics-interpolation.adb
-- Description     : Package for generic interpolation types and algorithms
-- Author          : J. Schrer
-- Created On      : 22. April 1998
-- Last Modified By: 
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

--with Ada.Text_Io, Debug; use Ada.Text_Io, Debug;

package body Math.Generics.Interpolation is
  ----------------------------------------------------------------------------
  procedure Initialize(The : in out Equidistant_Range) is
  begin
    The.Last := The.First + Integer((The.Max - The.Min) / The.Increment); 
--Open;
--Put_Line(Log_File,"Interpolation.Initialize");
--Put_Line(Log_File,
--  "first =" & Integer'Image(The.First) &
--  ", last =" & Integer'Image(The.Last) &
--  ", min =" & Float_Type'Image(The.Min) &
--  ", max =" & Float_Type'Image(The.Max) &
--  ", inc =" & Float_Type'Image(The.Increment));
--Close;
  end Initialize;
  ----------------------------------------------------------------------------
  function Search(The_Value : in Float_Type;
                  The_Range : in Equidistant_Range) return Search_Result is
                    
    Result : Search_Result;                
  begin
--Open;
--Put_Line(Log_File,"Interpolation.search(Equidistant_Range)");
--Put_Line(Log_File,"the_value =" & Float_Type'Image(The_Value));
--Put_Line(Log_File,
--  "first =" & Integer'Image(The_Range.First) &
--  ", last =" & Integer'Image(The_Range.Last) &
--  ", min =" & Float_Type'Image(The_Range.Min) &
--  ", max =" & Float_Type'Image(The_Range.Max) &
--  ", inc =" & Float_Type'Image(The_Range.Increment));
	  if    The_Value < The_Range.Min then
	    Result := (In_Range => False, Index => The_Range.First, Factor => 0.0);                                                  
	  elsif The_Value > The_Range.Max then
	    Result := (In_Range => False, Index => The_Range.Last,  Factor => 0.0);                                                  
    else -- I in the_range'First .. the_range'Last - 1
      Result.In_Range := True;
	    Result.Factor   := (The_Value - The_Range.Min) / The_Range.Increment;
      Result.Index    := Integer(Float_Type'Floor(Result.Factor));
      Result.Factor   := Result.Factor - Float_Type(Result.Index);
      Result.Index    := The_Range.First + Result.Index;
    end if;
--Put_Line(Log_File,
--  "in_range " & Boolean'Image(Result.In_Range) &
--  ", index " & Integer'Image(Result.Index) &
--  ", factor " & Float_Type'Image(Result.Factor));    
--Close;
    return Result;
  end Search;
  ----------------------------------------------------------------------------                                 
  function Is_In (The_Value : in Float_Type;
                  The_Range : in Equidistant_Range) return Boolean is
  begin
    return The_Value in The_Range.Min .. The_Range.Max;
  end Is_In;
  ----------------------------------------------------------------------------
  function Search(The_Value : in Float_Type;
                  The_Range : in Non_Equidistant_Range) return Search_Result is
                    
    Values : Array_1d renames The_Range.Values;     
    I      : Integer := Values'First;                    
    J      : Integer := Values'Last;                    
    K      : Integer;               
    Result : Search_Result;
  begin
--Open;
--Put_Line(Log_File,"Interpolation.search(Non_Equidistant_Range)");
--Put_Line(Log_File,"the_value =" & Float_Type'Image(The_Value));
--Put_Line(Log_File,
--  "first =" & Integer'Image(The_Range.First) &
--  ", last =" & Integer'Image(The_Range.Last));
--for I in Values'range loop
--  Put_Line(Log_File,"V(" & Integer'Image(I) & ") =" & Float_Type'Image(Values(I)));
--end loop;  
	  if    The_Value < Values(Values'First) then
--	    return (In_Range => False, Index => I, Factor => 0.0);                                                  
	    Result := (In_Range => False, Index => I, Factor => 0.0);                                                  
	  elsif The_Value > Values(Values'Last) then
--	    return (In_Range => False, Index => J, Factor => 0.0);                                                  
	    Result := (In_Range => False, Index => J, Factor => 0.0);                                                  
    else -- I in the_range'First .. the_range'LAst - 1
      loop
        K := (I + J) / 2;
        if The_Value >= Values(K) then
          I := K + 1;
          exit when The_Value < Values(I);
        else
          J := K - 1;
        end if;
      end loop;
--      return (In_Range => True,
      Result := (In_Range => True,
              Index    => K,	            
	            Factor   => (The_Value - Values(K)) / (Values(I) - Values(K)));                                                  
    end if;                          
--Put_Line(Log_File,
--  "in_range " & Boolean'Image(Result.In_Range) &
--  ", index " & Integer'Image(Result.Index) &
--  ", factor " & Float_Type'Image(Result.Factor));    
--Close;
    return Result;
  end Search;
  ---------------------------------------------------------------------------- 
  function Is_In (The_Value : in Float_Type;
                  The_Range : in Non_Equidistant_Range) return Boolean is
  begin
    return The_Value in The_Range.Values(The_Range.First) .. 
                        The_Range.Values(The_Range.Last);
  end Is_In;
  ----------------------------------------------------------------------------
  function Interpolated_Value(The_Table : in Array_1d;
                              Value_1   : in Search_Result) return Float_Type is

    I1 : Integer renames Value_1.Index;                                                                
  begin
--Open;
--Put_Line(Log_File,"Interpolation.Interpolated_Value(1d)");
--Put_Line(Log_File,
--  "in_range " & Boolean'Image(Value_1.In_Range) &
--  ", index " & Integer'Image(Value_1.Index) &
--  ", factor " & Float_Type'Image(Value_1.Factor));    
--Put_Line(Log_File,
--  "the_table'first =" & Integer'Image(The_Table'First) &
--  ", the_table'last =" & Integer'Image(The_Table'Last));
--Close;
    if Value_1.In_Range then
      return The_Table(I1) + (The_Table(I1 + 1) - The_Table(I1)) * Value_1.Factor;
    else
      return The_Table(I1);
    end if;  
  end Interpolated_Value;
  ---------------------------------------------------------------------------- 
  function Interpolated_Value(The_Table : in Array_2d;
                              Value_1   : in Search_Result;
                              Value_2   : in Search_Result) return Float_Type is
                                
    I1           : Integer renames Value_1.Index;                                                                
    I2           : Integer renames Value_2.Index;                                                                
    I1_1,  I2_1  : Integer;
    Lower, Upper : Float_Type;
  begin
--Open;
--Put_Line(Log_File,"Interpolation.Interpolated_Value(2d)");
--Put_Line(Log_File,
--  "in_range " & Boolean'Image(Value_1.In_Range) &
--  ", index " & Integer'Image(Value_1.Index) &
--  ", factor " & Float_Type'Image(Value_1.Factor));    
--Put_Line(Log_File,
--  "in_range " & Boolean'Image(Value_2.In_Range) &
--  ", index " & Integer'Image(Value_2.Index) &
--  ", factor " & Float_Type'Image(Value_2.Factor));    
--Put_Line(Log_File,
--  "the_table.range(1) =" & Integer'Image(The_Table'First(1)) & 
--  " .." & Integer'Image(The_Table'Last(1)) &
--  ", the_table.range(2) =" & Integer'Image(The_Table'First(2)) & 
--  " .." & Integer'Image(The_Table'Last(2)));
--Close;
    if Value_1.In_Range then
      I1_1 := I1 + 1;
    else
      I1_1 := I1;
    end if;      
    if Value_2.In_Range then
      I2_1 := I2 + 1;
    else
      I2_1 := I2;
    end if;     

    Lower := The_Table(I1, I2)   + 
            (The_Table(I1, I2_1)   - The_Table(I1,   I2)) * Value_2.Factor;
    Upper := The_Table(I1_1, I2) + 
            (The_Table(I1_1, I2_1) - The_Table(I1_1, I2)) * Value_2.Factor;

    return Lower + (Upper - Lower) * Value_1.Factor;
  end Interpolated_Value;
  ----------------------------------------------------------------------------
  function Interpolated_Value(The_Table : in Array_3d;
                              Value_1   : in Search_Result;
                              Value_2   : in Search_Result;
                              Value_3   : in Search_Result) return Float_Type is
                                
    I1                       : Integer renames Value_1.Index;                                                                
    I2                       : Integer renames Value_2.Index;                                                                
    I3                       : Integer renames Value_3.Index;                                                                
    I1_1, I2_1, I3_1         : Integer;
    Left_Lower,  Left_Upper,
    Right_Lower, Right_Upper,
    Lower,       Upper       : Float_Type;
  begin
    if Value_1.In_Range then
      I1_1 := I1 + 1;
    else
      I1_1 := I1;
    end if;      
    if Value_2.In_Range then
      I2_1 := I2 + 1;
    else
      I2_1 := I2;
    end if;     
    if Value_3.In_Range then
      I3_1 := I3 + 1;
    else
      I3_1 := I3;
    end if;     
    
    Left_Lower  := The_Table(I1,   I2,   I3)    + 
                  (The_Table(I1_1, I2,   I3)    - 
                   The_Table(I1,   I2,   I3))   * Value_1.Factor;
    Left_Upper  := The_Table(I1,   I2_1, I3)    + 
                  (The_Table(I1_1, I2_1, I3)    - 
                   The_Table(I1,   I2_1, I3))   * Value_1.Factor;
    Right_Lower := The_Table(I1,   I2,   I3_1)  + 
                  (The_Table(I1_1, I2,   I3_1)  - 
                   The_Table(I1,   I2,   I3_1)) * Value_1.Factor;
    Right_Upper := The_Table(I1,   I2_1, I3_1)  + 
                  (The_Table(I1_1, I2_1, I3_1)  - 
                   The_Table(I1,   I2_1, I3_1)) * Value_1.Factor;
                  
    Lower := Left_Lower  + (Left_Upper  - Left_Lower)  * Value_2.Factor;              
    Upper := Right_Lower + (Right_Upper - Right_Lower) * Value_2.Factor;              
    
    return Lower + (Upper - Lower) * Value_3.Factor;    
  end Interpolated_Value;
  ----------------------------------------------------------------------------
  function Interpolated_Value(The_Table : in Array_2d_Irregular;
                              Value_1   : in Search_Result;
                              Value_2   : in Search_Result) return Float_Type is
                                
    New_Value_2  : Search_Result := Value_2;
    I1           : Integer renames Value_1.Index;
    I2           : Integer renames New_Value_2.Index;                                                                
    I1_1,  I2_1  : Integer;
    Lower, Upper : Float_Type;
    -------------------------------------------------
    procedure Check(Index_1, Index_2 : in Integer) is
    begin
	    if    Index_2 > The_Table(Index_1)'Last  then
	      New_Value_2 := (In_Range => False, 
                        Index    => Integer'Min(The_Table(Index_1)'Last, 
                                                New_Value_2.Index),  
                        Factor   => 0.0);
        I2_1 := I2;                        
	    elsif Index_2 < The_Table(Index_1)'First then
	      New_Value_2 := (In_Range => False, 
                        Index    => Integer'Max(The_Table(Index_1)'First,
                                                New_Value_2.Index),
                        Factor   => 0.0);
        I2_1 := I2;                        
	    end if;
    end Check;
    ----------    
  begin
--Open;
--Put_Line(Log_File,"Interpolation.Interpolated_Value(Array_2d_Irregular)");
--Put_Line(Log_File,
--  "value_1.in_range " & Boolean'Image(Value_1.In_Range) &
--  ", value_1.index " & Integer'Image(Value_1.Index) &
--  ", value_1.factor " & Float_Type'Image(Value_1.Factor));    
--Put_Line(Log_File,
--  "value_2.in_range " & Boolean'Image(Value_2.In_Range) &
--  ", value_2.index " & Integer'Image(Value_2.Index) &
--  ", value_2.factor " & Float_Type'Image(Value_2.Factor));    
--for I in The_Table'range loop  
--  Put_Line(Log_File,
--    "the_table(" & Integer'Image(I) & ").range =" &
--    Integer'Image(The_Table(I)'First) & " .." &
--    Integer'Image(The_Table(I)'Last));
--end loop;
    if Value_2.In_Range then
      I2_1 := I2 + 1;
      Check(I1, I2_1);
      Check(I1, I2);
    else
      I2_1 := I2;
      Check(I1, I2);      
    end if;     
    if Value_1.In_Range then
      I1_1 := I1 + 1;
      Check(I1_1, I2);
      Check(I1_1, I2_1);
    else
      I1_1 := I1;
    end if;      
--Put_Line(Log_File,
--  "new_value_2.in_range " & Boolean'Image(New_Value_2.In_Range) &
--  ", new_value_2.index " & Integer'Image(New_Value_2.Index) &
--  ", new_value_2.factor " & Float_Type'Image(New_Value_2.Factor));    
--Close;
                 
    Lower := The_Table(I1)(I2)   + 
            (The_Table(I1)(I2_1)   - The_Table(I1)(I2))   * Value_2.Factor;
    Upper := The_Table(I1_1)(I2) + 
            (The_Table(I1_1)(I2_1) - The_Table(I1_1)(I2)) * Value_2.Factor;

    return Lower + (Upper - Lower) * Value_1.Factor;
  end Interpolated_Value;
  ----------------------------------------------------------------------------
end Math.Generics.Interpolation;
