--                              -*- Mode: Ada -*-
-- Filename        : Win32_Addons-gl.ads
-- Description     : Declare types and subprograms not in 
--                   original bindings in win32.gl.
-- Author          : ag
-- Created On      : 04. Feb. 1998
-- Last Modified By: J. Schrer
-- Last Modified On: 17. Apr. 1998
-- Update Count    : 2
-- Status          :

with Win32.Gl;

package Win32_Addons.Gl is
   
  type Ac_Glboolean is access constant Win32.Gl.Glboolean;
  -- polygon_offset */
	Gl_Polygon_Offset_Factor   				: constant := 16#8038#;
	Gl_Polygon_Offset_Units           : constant := 16#2A00#;
	Gl_Polygon_Offset_Point           : constant := 16#2A01#;
	Gl_Polygon_Offset_Line            : constant := 16#2A02#;
	Gl_Polygon_Offset_Fill            : constant := 16#8037#;
  
  --Enable/disable
  Gl_Index_Logic_Op                 : constant := 16#0BF1#;
	Gl_Color_Logic_Op                 : constant := 16#0BF2#;

  -- texture
  Gl_Texture_Priority               : constant := 16#8066#;
  Gl_Proxy_Texture_1d               : constant := 16#8063#;
    
-- WINGDIAPI void APIENTRY glCopyTexImage1D (GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLint border);
  procedure Glcopyteximage1d (
    Target : Win32.Gl.Glenum;
    Level  : Win32.Gl.Glint;
    Internalformat : Win32.Gl.Glenum;
    X              : Win32.Gl.Glint;
    Y              : Win32.Gl.Glint;
    Width          : Win32.Gl.Glsizei;
    Border         : Win32.Gl.Glint );
    
--WINGDIAPI void APIENTRY glCopyTexImage2D (GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border);
  procedure Glcopyteximage2d (
    Target : Win32.Gl.Glenum;
    Level  : Win32.Gl.Glint;
    Internalformat : Win32.Gl.Glenum;
    X              : Win32.Gl.Glint;
    Y              : Win32.Gl.Glint;
    Width          : Win32.Gl.Glsizei;
    Height         : Win32.Gl.Glsizei;
    Border         : Win32.Gl.Glint );

--WINGDIAPI void APIENTRY glCopyTexSubImage1D (GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width);
  procedure Glcopytexsubimage1d (
    Target  : Win32.Gl.Glenum;
    Level   : Win32.Gl.Glint;
    Xoffset : Win32.Gl.Glint;
    X              : Win32.Gl.Glint;
    Y              : Win32.Gl.Glint;
    Width          : Win32.Gl.Glsizei);
    
--WINGDIAPI void APIENTRY glCopyTexSubImage2D (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);
  procedure Glcopytexsubimage2d (
    Target  : Win32.Gl.Glenum;
    Level   : Win32.Gl.Glint;
    Xoffset : Win32.Gl.Glint;
    Yoffset : Win32.Gl.Glint;
    X              : Win32.Gl.Glint;
    Y              : Win32.Gl.Glint;
    Width          : Win32.Gl.Glsizei;
    Height         : Win32.Gl.Glsizei);
    
--WINGDIAPI void APIENTRY glBindTexture (GLenum target, GLuint texture);
  procedure Glbindtexture (
    Target  : Win32.Gl.Glenum;
    Texture : Win32.Gl.Gluint);   
    
--WINGDIAPI void APIENTRY glGenTextures (GLsizei n, GLuint *textures);
  procedure Glgentextures (
    Target  : Win32.Gl.Glenum;
    Texture : Win32.Gl.Ac_Gluint);   
    
--WINGDIAPI GLboolean APIENTRY glAreTexturesResident (GLsizei n, const GLuint *textures, GLboolean *residences);
	function Glaretexturesresident (
    N          : Win32.Gl.Glsizei;
    Textures   : Win32.Gl.Ac_Gluint;
    Residences : Ac_Glboolean) return Win32.Gl.Glboolean; 
    
--Wingdiapi Glboolean Apientry Glistexture (Gluint Texture);
  function  Glistexture (
    Texture		 : Win32.Gl.Gluint) return Win32.Gl.Glboolean; 
    
--WINGDIAPI void APIENTRY glPolygonOffset (GLfloat factor, GLfloat units);
  procedure Glpolygonoffset (
    Factor  : Win32.Gl.Glfloat;
    Units   : Win32.Gl.Glfloat);
      
--  procedure Gladdswaphintrectwin
--    (X      : Win32.Gl.Glint;    
--     Y      : Win32.Gl.Glint;    
--     Width  : Win32.Gl.Glsizei;    
--     Height : Win32.Gl.Glsizei);    

private

  pragma Import(Stdcall, Glcopyteximage1d,      External_Name => "glCopyTexImage1D");
  pragma Import(Stdcall, Glcopyteximage2d,      External_Name => "glCopyTexImage2D");
  pragma Import(Stdcall, Glcopytexsubimage1d,   External_Name => "glCopyTexSubImage1D");
  pragma Import(Stdcall, Glcopytexsubimage2d,   External_Name => "glCopyTexSubImage2D");
  pragma Import(Stdcall, Glbindtexture,         External_Name => "glBindTexture");
  pragma Import(Stdcall, Glgentextures,         External_Name => "glGenTextures");
  pragma Import(Stdcall, Glaretexturesresident, External_Name => "glAreTexturesResident");
  pragma Import(Stdcall, Glistexture,           External_Name => "glIsTexture");
  pragma Import(Stdcall, Glpolygonoffset,       External_Name => "glPolygonOffset");
--  pragma Import(Stdcall, Gladdswaphintrectwin,  External_Name => "GlAddSwapHintRectWIN");
  
end Win32_Addons.Gl;
