--                    -*- Mode: Ada -*-
-- Filename        : windows-ini_file.ads
-- Description     : Package for reading Windows Ini-Files. 
-- Author          : H. Altmann, M. Scharfetter
-- Created On      : 07. Jan. 1998
-- Last Modified By:
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Win32.Winbase;
with Ada.Exceptions;
with Interfaces.C;

package body Windows.Ini_File is
  
  use type Interfaces.C.Unsigned_Long;
    
  function Is_Entry(
              File_Name: in  String; 
              Section:   in  String;
              S_Entry:   in  String) return Boolean is 
    Ret    :Win32.Dword;
    C_Str  : aliased Interfaces.C.Char_Array(0 .. Max_Str_Length);
    Ada_Str: constant String:="";
  begin
	  Ret := Win32.Winbase.Getprivateprofilestring
	    (Lpappname        => Interfaces.C.To_C(Section)(0)'Unchecked_Access,
	     Lpkeyname        => Interfaces.C.To_C(S_Entry)(0)'Unchecked_Access,
	     Lpdefault        => Interfaces.C.To_C(Ada_Str)(0)'Unchecked_Access,
	     Lpreturnedstring => C_Str(0)'Unchecked_Access,
	     Nsize            => Win32.Dword(C_Str'Length),
	     Lpfilename       => Interfaces.C.To_C(File_Name)(0)'Unchecked_Access );
    return Ret > 0;
 end Is_Entry;

  function Get_Entry(
              File_Name: in  String; 
              Section:   in  String;
              S_Entry:   in  String) return String is 
    Ret    :Win32.Dword;
    C_Str  : aliased Interfaces.C.Char_Array(0 .. Max_Str_Length);
    Ada_Str: constant String:="";
  begin
	  Ret := Win32.Winbase.Getprivateprofilestring
	    (Lpappname        => Interfaces.C.To_C(Section)(0)'Unchecked_Access,
	     Lpkeyname        => Interfaces.C.To_C(S_Entry)(0)'Unchecked_Access,
	     Lpdefault        => Interfaces.C.To_C(Ada_Str)(0)'Unchecked_Access,
	     Lpreturnedstring => C_Str(0)'Unchecked_Access,
	     Nsize            => Win32.Dword(C_Str'Length),
	     Lpfilename       => Interfaces.C.To_C(File_Name)(0)'Unchecked_Access );
    Block:
    declare
      Str : constant String := Interfaces.C.To_Ada (C_Str);
    begin
      if Str'Length = 0 then
        Ada.Exceptions.Raise_Exception(Error'Identity,
           Message => "Error in " & File_Name & ": Section: '" & Section & 
                      "' Entry: '" & S_Entry & "' not found!");
      else
        return Str;
      end if;
    end Block;
 end Get_Entry;

 function Get_Entry(
              File_Name: in  String; 
              Section:   in  String;
              S_Entry:   in  String) return Integer is
  begin
    return Integer'Value(Get_Entry(File_Name, Section, S_Entry));
  exception
    when Constraint_Error =>
        Ada.Exceptions.Raise_Exception(Error'Identity,
           Message => "Error in " & File_Name & ": Section: '" & Section &
                      "': Entry: '" & S_Entry & "': Not an Integer-Value");
  end;

 function Get_Entry(
              File_Name: in  String; 
              Section:   in  String;
              S_Entry:   in  String) return Float is
  begin
    return Float'Value(Get_Entry(File_Name, Section, S_Entry));
  exception
    when Constraint_Error =>
        Ada.Exceptions.Raise_Exception(Error'Identity,
           Message => "Error in " & File_Name & ": Section: '" & Section &
                      "': Entry: '" & S_Entry & "': Not a valid Float-Value");
  end;
  
  
    
  

  
--
--  procedure Set_Entry(
--              File_Name: in  String; 
--              Section:   in  String;
--              S_Entry:   in  String;
--              Image:     in  String;
--              No_Error:  out Boolean) is
--  begin
--    null;
--  end;
--
--  procedure Set_Entry(
--              File_Name: in  String; 
--              Section:   in  String;
--              S_Entry:   in  String;
--              Value:     in  Integer;
--              No_Error:  out Boolean) is
--  begin
--    null;
--  end;
--
--  procedure Set_Entry(
--              File_Name: in  String; 
--              Section:   in  String;
--              S_Entry:   in  String;
--              Fvalue:    in  Float;
--              No_Error:  out Boolean) is 
--  begin
--    null;
--  end;
  
end Windows.Ini_File;

