--                              -*- Mode: Ada -*-
-- Filename        : windows-arrays.adb
-- Description     : Utilities for interfacing with c and win32
-- Author          : J. Schrer
-- Created On      : 19. Nov. 1997
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Ada.Unchecked_Conversion;
with Portable.Operating_System;
with System;

package body Windows.Arrays is
  ---------------------------------------------------------------------------
  function To_C_wchar_array(Item : in Win32.Wchar_Array) 
           return Interfaces.C.Wchar_Array is
    Ret : Interfaces.C.Wchar_Array(Interfaces.C.Size_T(Item'First) .. 
                                   Interfaces.C.Size_T(Item'Last));         
  begin
    for I in Ret'range loop
      Ret(I) := Item(Natural(I));
    end loop;
    return Ret;
  end To_C_wchar_array;  
  ---------------------------------------------------------------------------             
  function To_Ada (Item       : in Win32.Pwch)
           return Wide_String is
  begin
    return Interfaces.C.To_Ada(To_C_wchar_array
          (Wchar_Pointers.Value(Wchar_Pointers.Pointer(Item))));   
  end To_Ada;
  ---------------------------------------------------------------------------      
  function To_Ada (Item       : in Win32.Pcwch)
           return Wide_String is       
  begin
    return Interfaces.C.To_Ada(To_C_wchar_array
          (Wchar_Pointers.Value(To_Var_Ptr(Item))));       
  end To_Ada;
  ----------------------------------------------------------------------------              
  function To_Var_Ptr(Item : in Win32.Pcuchar) return Uchar_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcuchar, Target => Uchar_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcbyte) return Byte_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcbyte, Target => Byte_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcwch) return Wchar_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcwch, Target => Wchar_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcshort) return Short_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcshort, Target => Short_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcword) return Ushort_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcword, Target => Ushort_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcint) return Int_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcint, Target => Int_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pbool) return Int_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pbool, Target => Int_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcuint) return Uint_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcuint, Target => Uint_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pclong) return Long_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pclong, Target => Long_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
  function To_Var_Ptr(Item : in Win32.Pcdword) return Ulong_Pointers.Pointer is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcdword, Target => Ulong_Pointers.Pointer);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return Convert(Item);
  end To_Var_Ptr;
  ----------------------------------------------------------------------------
end Windows.Arrays;
