--                              -*- Mode: Ada -*-
-- Filename        : windows-utils.adb
-- Description     : Utilities for interfacing with c and win32
-- Author          : J. Schrer, M. Scharfetter (winsock-errors)
-- Created On      : 19. Nov. 1997
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :


with Portable.Operating_System;
with Portable.Utils;
with Ada.Calendar;
with Utilities.Calendar_Utilities;
with Win32.Winnt;
with Windows.Utils;

package body Windows.Messages is

  use type Interfaces.C.Unsigned_Long,
           Win32.Bool, Win32.Int,
           Ada.Text_Io.File_Access,
           Win32.Windef.Hwnd;
           
  package Calendar_Utilities renames Utilities.Calendar_Utilities;         
           
  Newline : constant String := Portable.Operating_System.New_Line;  
  ---------------------------------------------------------------------------
  function Message_String(Message_Id : in Interfaces.C.Unsigned_Long :=
                                          Win32.Winbase.Getlasterror)
           return String is

    Buffer : Interfaces.C.Char_Array(0 .. 1023);
    Length : constant Interfaces.C.Unsigned_Long :=
             Win32.Winbase.Formatmessage
            (Dwflags      => Win32.Winbase.Format_Message_From_System,
             Lpsource     => System.Null_Address,
             Dwmessageid  => Message_Id,
--             Dwlanguageid => Interfaces.C.Unsigned_Long(Win32.Winnt.Makelangid
--                            (P => Win32.Winnt.Lang_English, 
--                             S => Win32.Winnt.Sublang_English_Us)),
             Dwlanguageid => Interfaces.C.Unsigned_Long(Win32.Winnt.Makelangid
                            (P => Win32.Winnt.Lang_Neutral,
                             S => Win32.Winnt.Sublang_Default)),
             Lpbuffer     => Buffer(Buffer'First)'Unchecked_Access,
             Nsize        => Interfaces.C.Unsigned_Long(Buffer'Length));
  begin
    if Length = 0 then
      return "";
--      Ada.Exceptions.Raise_Exception(Error'Identity,
--         Message => "windows.utils.message_string failed.");
    else
      return Interfaces.C.To_Ada
            (Buffer(Buffer'First .. Interfaces.C.Size_T(Length)));
    end if;
  end Message_String;
  ---------------------------------------------------------------------------
  procedure Display_Message
           (The_Title           : in String := "Windows message";
            The_Message         : in String := "";
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := System.Null_Address;
            Beep                : in Natural           := 0) is
    -------------------------------------------------------------------------                            
    procedure Display(Windows_Info : in String := "") is

      Text     : constant String := The_Message & Windows_Info;
	    C_Text   : constant Interfaces.C.Char_Array := Interfaces.C.To_C(Text);
	    C_Title  : constant Interfaces.C.Char_Array :=
                          Interfaces.C.To_C(The_Title);
    begin
      if Log_File /= null and then Ada.Text_Io.Is_Open(Log_File.all) then
        Ada.Text_Io.Put_Line(Log_File.all, Text);
      end if;
	    if Win32.Winuser.Messagebox
	           (Hwnd        => The_Window,
			        Lptext      => C_Text(C_Text'First)'Unchecked_Access,
			        Lpcaption   => C_Title(C_Title'First)'Unchecked_Access,
	--            Wlanguageid => Win32.Winnt.Makelangid
	--                          (P => Win32.Winnt.Lang_English, 
	--                           S => Win32.Winnt.Sublang_English_Us),
			        Utype       => Win32.Winuser.Mb_Ok) = Win32.False then
			  Ada.Exceptions.Raise_Exception(Error'Identity, Message =>
	        "Not enough Memory to create Messagebox (Windows.Utils.Display_Message");
      end if;
    end Display;
    ------------
  begin
    if Beep > 0 then
      Utils.Beep(Times => Beep);
    end if;
    if Add_Windows_Message then
      declare
        Win_Info : constant String := Message_String;
      begin
        if Win_Info /= "" then
          Display(Windows_Info => Newline & "Windows info : " & Win_Info);
        else
          Display;
        end if;
      end;
    else
      Display;
    end if;
  end Display_Message;
  ---------------------------------------------------------------------------
  procedure Display_Message
           (The_Exception       : in Ada.Exceptions.Exception_Occurrence;
            Close_Window        : in Boolean := True;
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := System.Null_Address;
            Beep                : in Natural           := 0) is

    Dummy : Win32.Lresult;
  begin
    Display_Message
       (The_Title           => "Exception raised : " &
               	               Ada.Exceptions.Exception_Name(The_Exception),
	      The_Message         => Calendar_Utilities.Image_Of(Ada.Calendar.Clock) & 
                               Newline & 
                               Ada.Exceptions.Exception_Message(The_Exception),
        Log_File            => Log_File,
	      Add_Windows_Message => Add_Windows_Message,
	      Popup_Type          => Popup_Type,
        The_Window          => The_Window,
        Beep                => Beep);
    if Close_Window and then The_Window /= System.Null_Address then
	    Dummy := Win32.Winuser.Sendmessage
	              (Hwnd    => The_Window,
	               Msg     => Win32.Winuser.Wm_Destroy,
	               Wparam  => 0,
	               Lparam  => 0);
    end if;
  end Display_Message;
  ----------------------------------------------------------------------------
  procedure Display_Information
           (The_Exception       : in Ada.Exceptions.Exception_Occurrence;
            Close_Window        : in Boolean := True;
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := System.Null_Address;
            Beep                : in Natural           := 0) is

    Dummy : Win32.Lresult;
  begin
    Display_Message
       (The_Title           => "Exception raised : " &
               	               Ada.Exceptions.Exception_Name(The_Exception),
        The_Message         => Calendar_Utilities.Image_Of(Ada.Calendar.Clock) & 
                               Newline & 
                               Portable.Utils.Exception_Information(The_Exception),
        Log_File            => Log_File,
	  Add_Windows_Message => Add_Windows_Message,
	  Popup_Type          => Popup_Type,
        The_Window          => The_Window,
        Beep                => Beep);
    if Close_Window and then The_Window /= System.Null_Address then
	Dummy := Win32.Winuser.Sendmessage
	              (Hwnd    => The_Window,
	               Msg     => Win32.Winuser.Wm_Destroy,
	               Wparam  => 0,
	               Lparam  => 0);
    end if;
  end Display_Information;
  ----------------------------------------------------------------------------
end Windows.Messages;
