--                              -*- Mode: Ada -*-
-- Filename        : windows-messages.ads
-- Description     : Utilities for interfacing with win32
--                   Fetch error strings from error numbers
--                   Display error message strings in popup-window.
--                   Handle ada-exceptions from Windows applications.
-- Author          : J. Schrer
-- Created On      : 19. Nov. 1997
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Ada.Text_Io;
with Ada.Exceptions;
with Interfaces.C;
with system;
with Win32.Windef,
     Win32.Winbase,
     Win32.Winuser;

package Windows.Messages is

  function Message_String(Message_Id : in Interfaces.C.Unsigned_Long :=
                                          Win32.Winbase.Getlasterror)
           return String;

  -- Get the message string generated by windows using for a defined 
  -- given message_id using format_message. Uses last error as default.
  -- (Only for language_neutral, sublanguage_default).
  -- English messages for winsock errors are given.

  procedure Display_Message
           (The_Title           : in String := "Windows message";
            The_Message         : in String := "";
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := system.null_address;
            Beep                : in Natural           := 0);

  -- Opens a window and displays the_message.
  -- If Log_file /= null and log_file.all is open, then the
  -- the_message is written to log_file.all.                                     
  -- If Add_Windows_Message then the windows message retrieved by 
  -- format_message is added (on a new line if the_message /= "") 
  -- following "Windows info: ".

  procedure Display_Message
           (The_Exception       : in Ada.Exceptions.Exception_Occurrence;
            Close_Window        : in Boolean := True;
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := system.null_address;
            Beep                : in Natural           := 0);

  -- Opens a window and displays the Ada.Exceptions.Exception_Message.
  -- If Close_Window, then the WM_Destroy-message is sent to the_window.
  -- If Log_file /= null and log_file.all is open, then the
  -- exception_message is written to log_file.all.                                     
  -- If Add_Windows_Message then the windows message retrieved by 
  -- format_message is added (on a new line if the_message /= "")
  -- following "Windows info: ".

  procedure Display_Information
           (The_Exception       : in Ada.Exceptions.Exception_Occurrence;
            Close_Window        : in Boolean := True;
            Log_File            : in Ada.Text_Io.File_Access := null;
            Add_Windows_Message : in Boolean := True;
            Popup_Type          : in Interfaces.C.Unsigned :=
                                     Win32.Winuser.Mb_Applmodal;
            The_Window          : in Win32.Windef.Hwnd := system.null_address;
            Beep                : in Natural           := 0);

  -- Opens a window and displays the Ada.Exceptions.Exception_Information.
  -- If Close_Window, then the WM_Destroy-message is sent to the_window.
  -- If Log_file /= null and log_file.all is open, then the
  -- exception_message is written to log_file.all.                                     
  -- If Add_Windows_Message then the windows message retrieved by 
  -- format_message is added (on a new line if the_message /= "")
  -- following "Windows info: ".

end Windows.Messages;