--                              -*- Mode: Ada -*-
-- Filename        : windows-utils.adb
-- Description     : Utilities for interfacing with c and win32
-- Author          : J. Schrer
-- Created On      : 19. Nov. 1997
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Ada.Exceptions;
with Ada.Unchecked_Conversion;
with Utilities.C;
with Windows.Messages;
with Win32.Winbase;
with Win32.Windef;
with Win32.Winuser;
with Win32.Shellapi;

package body Windows.Utils is
  
  use type Win32.Bool;
  use type Win32.Dword;
  ---------------------------------------------------------------------------
  function Time_Zone_Bias_Minutes return Integer is
    Tz     : aliased Win32.Winbase.Time_Zone_Information;
    Result : Win32.Dword;  
  begin 
    Result := Win32.Winbase.Gettimezoneinformation(Tz'Unchecked_Access);
    if Result = 16#ffff_ffff# then
      Messages.Display_Message(The_Message => 
        "Windows.Utils.Time_Zone_Bias_Minutes, Winbase.Gettimezoneinformation failed");
    end if;
    return Integer(Tz.Bias);    
  end Time_Zone_Bias_Minutes; 
  ---------------------------------------------------------------------------
  procedure Beep(Times : in Positive := 1) is
  begin
    for I in 1 .. Times loop
      if Win32.Winuser.Messagebeep(Utype => Win32.Winuser.Mb_Ok) = Win32.False then
        Ada.Exceptions.Raise_Exception(Error'Identity,
          "Windows.Utils.Beep, Win32.Winuser.MessageBeep failed");
      end if;
    end loop;
  end Beep;  
  ---------------------------------------------------------------------------
  procedure Start_Application(Executable : in String;
                              Parameter  : in String := "") is
                                
    C_Operation  : aliased Interfaces.C.Char_Array :=
                           Interfaces.C.To_C("open");
    C_Executable : aliased Interfaces.C.Char_Array :=
                           Interfaces.C.To_C(Executable);
    C_Parameter  : aliased Interfaces.C.Char_Array :=
                           Interfaces.C.To_C(Parameter);
    Exe : Win32.Windef.Hinstance;
  begin
		Exe := Win32.Shellapi.Shellexecute
      (Hwnd         => Win32.Winuser.Getfocus,
	     Lpoperation  => C_Operation (C_Operation'First)'Unchecked_Access,
 	     Lpfile       => C_Executable(C_Executable'First)'Unchecked_Access,
	     Lpparameters => C_Parameter (C_Parameter'First)'Unchecked_Access,
	     Lpdirectory  => null,
	     Nshowcmd     => Win32.Winuser.Sw_Shownormal);
  end Start_Application;
  ---------------------------------------------------------------------------
  function To_Bool   (Item : in Boolean)    return Win32.Bool is
  begin
    return Win32.Bool'Val(Boolean'Pos(Item));
  end To_Bool;
  ---------------------------------------------------------------------------
  function To_Boolean(Item : in Win32.Bool) return Boolean is
  begin
    return Boolean'Val(Win32.Bool'Pos(Item));
  exception
    when Constraint_Error => return False;
  end To_Boolean;
  ---------------------------------------------------------------------------
  function To_C   (Item       : in String;
                   Append_Nul : in Boolean := True)
           return Win32.Char_Array is
  begin
    return Win32.To_Win(Interfaces.C.To_C(Item, Append_Nul));
  end To_C;
  ---------------------------------------------------------------------------
  function To_Ada (Item     : in Win32.Char_Array; 
                   Trim_Nul : in Boolean := True)
           return String is
  begin
    return Interfaces.C.To_Ada(Win32.To_C(Item), Trim_Nul);
  end To_Ada;
  ---------------------------------------------------------------------------
--  function To_C   (Item       : in String;
--                   Append_Nul : in Boolean := True)
--           return Win32.Pchar is
--
--    C_String : aliased Interfaces.C.Char_Array := 
--               Interfaces.C.To_C(Item, Append_Nul => True);
--  begin
--    return Win32.To_Pstr(Interfaces.C.Strings.To_Chars_Ptr(C_String'Unchecked_Access));
--  end To_C;
--  ---------------------------------------------------------------------------
--  function To_C   (Item       : in String;
--                   Append_Nul : in Boolean := True)
--           return Win32.Pcch is
--
--    C_String : aliased constant Interfaces.C.Char_Array := 
--               Interfaces.C.To_C(Item, Append_Nul => True);
--  begin
--    return Win32.To_Pcstr(Interfaces.C.Strings.To_Chars_Ptr(C_String'Unchecked_Access));
--  end To_C;
  ---------------------------------------------------------------------------
  function To_Ada (Item     : in Win32.Pchar)
           return String is
    use type Win32.Pchar;
  begin
    return Interfaces.C.Strings.Value(Win32.To_Chars_Ptr(Item));
  exception
    when Interfaces.C.Strings.Dereference_Error =>
      if Item = null then
        return "";
      else
        raise;
      end if;
  end To_Ada;
  ---------------------------------------------------------------------------
  function To_Ada (Item     : in Win32.Pcch)
           return String is
    use type Win32.Pcch;
  begin
    return Interfaces.C.Strings.Value(Win32.To_Chars_Ptr(Item));
  exception
    when Interfaces.C.Strings.Dereference_Error =>
      if Item = null then
        return "";
      else
        raise;
      end if;
  end To_Ada;
  ---------------------------------------------------------------------------
  function To_Ada (Item       : in Win32.Pbyte)
           return String is
   
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pbyte, Target => Win32.Pchar);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return To_Ada(Convert(Item));
  end To_Ada;
  ---------------------------------------------------------------------------  
  function To_Ada (Item       : in Win32.Puchar)
           return String is
  begin
    return Utilities.C.To_Ada(Utilities.C.Unsigned_Char_Variable.Pointer(Item));
  end To_Ada;
  ---------------------------------------------------------------------------  
  function To_Ada (Item       : in Win32.Pcbyte)
           return String is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcbyte, Target => Win32.Pchar);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return To_Ada(Convert(Item));
  end To_Ada;
  ---------------------------------------------------------------------------  
  function To_Ada (Item       : in Win32.Pcuchar)
           return String is
             
    function Convert is new Ada.Unchecked_Conversion
            (Source => Win32.Pcuchar, Target => Win32.Pchar);       
    pragma Inline(Convert); -- to eliminate additional call          
  begin
    return To_Ada(Convert(Item));
  end To_Ada;
  ---------------------------------------------------------------------------              
  function To_String_Vector(Item : in Interfaces.C.Char_Array)
           return Utilities.String_Vectors.String_Vector is
    use type Interfaces.C.Char;
  begin
    return Utilities.String_Vectors.Split
	        (The_String => Interfaces.C.To_Ada(Item, Trim_Nul => False), 
	         Separator  => Ascii.Nul);
  end To_String_Vector;
  ---------------------------------------------------------------------------
--  function To_String_Vector(Item : in Interfaces.C.Char_Array)
--           return Utilities.String_Vectors.String_Vector is
--             
--    Temp        : String(1 .. Item'Length);
--    The_Strings : Utilities.String_Vectors.String_Vector(Temp'range);
--    Count       : Natural;
--    Nul_Counter : Natural := 0;
--    Char_Counter: Interfaces.C.Size_T := Item'First;
--    use type Interfaces.C.Size_T;
--  begin
--    while Char_Counter < Item'Length loop
--      begin
--        Interfaces.C.To_Ada(Item     => Item(Char_Counter .. Item'Last),
--                            Target   => Temp,
--                            Count    => Count,
--                            Trim_Nul => True);
--      exception
--        when Interfaces.C.Terminator_Error => -- trailing nul missing
--          Interfaces.C.To_Ada(Item     => Item(Char_Counter .. Item'Last),
--                              Target   => Temp,
--                              Count    => Count,
--                              Trim_Nul => False);
--      end;                        
--      Nul_Counter              := Nul_Counter + 1;
--      The_Strings(Nul_Counter) := new String'(Temp(1 .. Count));
--      Char_Counter             := Char_Counter + Interfaces.C.Size_T(Count) + 1;
--    end loop;
--    return The_Strings(1 .. Nul_Counter);                    
--  end To_String_Vector;
  ---------------------------------------------------------------------------
  function To_String_Vector(Item : in Interfaces.C.Strings.Chars_Ptr)
           return Utilities.String_Vectors.String_Vector is
  begin
    return To_String_Vector(Interfaces.C.Strings.Value(Item));
  end To_String_Vector;
  ---------------------------------------------------------------------------
  function To_String_Vector(Item : in Win32.Pchar)
           return Utilities.String_Vectors.String_Vector is
  begin
    return To_String_Vector(Win32.To_Chars_Ptr(Item));
  end To_String_Vector;
  ---------------------------------------------------------------------------
  function To_String_Vector(Item : in Win32.Pcch)
           return Utilities.String_Vectors.String_Vector is
  begin
    return To_String_Vector(Win32.To_Chars_Ptr(Item));
  end To_String_Vector;
  ---------------------------------------------------------------------------
  function To_Char_Array(Item : in Utilities.String_Vectors.String_Vector)
           return Interfaces.C.Char_Array is
  begin
    return Interfaces.C.To_C(Utilities.String_Vectors.Assemble
                            (The_Strings => Item, Separator => Ascii.Nul));
  end To_Char_Array;
  ---------------------------------------------------------------------------
--  function To_Char_Array(Item : in Utilities.String_Vectors.String_Vector)
--           return Interfaces.C.Char_Array is
--    
--    use type Interfaces.C.Size_T;
--    Temp  : Interfaces.C.Char_Array(0 .. Interfaces.C.Size_T
--              (Utilities.String_Vectors.Total_Length(Item) + Item'Length) - 1);                  
--    First : Interfaces.C.Size_T := Temp'First;
--    Last  : Interfaces.C.Size_T := First;
--  begin
--    for I in Item'range loop
--      Last := First + Interfaces.C.Size_T(Item(I)'Length);
--      Temp(First .. Last) := Interfaces.C.To_C(Item(I).all); -- appends nul
--      First := Last + 1;
--    end loop;
--    return Temp;
--  end To_Char_Array;
  ---------------------------------------------------------------------------
--  function To_Chars_Ptr (Item : in Utilities.String_Vectors.String_Vector)
--           return Interfaces.C.Strings.Chars_Ptr is
--  begin
--    return Interfaces.C.Strings.To_Chars_Ptr(To_Char_Array(Item));
--  end To_Chars_Ptr;
--  ---------------------------------------------------------------------------
--  function To_Pchar     (Item : in Utilities.String_Vectors.String_Vector)
--           return Win32.Pchar is
--  begin
--    return Win32.To_Pstr(To_Chars_Ptr(Item));
--  end To_Pchar;
--  ---------------------------------------------------------------------------
--  function To_Pcch      (Item : in Utilities.String_Vectors.String_Vector)
--           return Win32.Pcch is
--  begin
--    return Win32.To_Pcstr(To_Chars_Ptr(Item));
--  end To_Pcch;
  ---------------------------------------------------------------------------
end Windows.Utils;
