--                              -*- Mode: Ada -*-
-- Filename        : windows-utils.ads
-- Description     : Utilities for interfacing with win32 
--                   basic types
-- Author          : J. Schrer
-- Created On      : 19. Nov. 1997
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Win32;
with Interfaces.C,
     Interfaces.C.Strings;
with Utilities.String_Vectors;

package Windows.Utils is
  
  function Time_Zone_Bias_Minutes return Integer;
  
  -- Returns the time-zone offset to UTC in minutes.
  -- If the time-zone is GMT+1h, then -60 is returned. 
  
  procedure Beep(Times : in Positive := 1);
  
  -- Beeps.
   
  procedure Start_Application(Executable : in String;
                              Parameter  : in String := "");
                              
  -- Looks for executable and starts it with parameter.
                                                            
  ----------------------------------------------------------------------------
  -- Conversion routines between standard.boolean and win32.bool:
  
  function To_Bool   (Item : in Boolean)    return Win32.Bool;
  
  -- Returns 0 for false, 1 for true.
  
  function To_Boolean(Item : in Win32.Bool) return Boolean;
  
  -- Returns true for 1, false for all other values.

  ----------------------------------------------------------------------------
  -- Conversion routines between ada strings and win32 char pointers.
  -- One for each new access to variable type
  -- and one for each new access to constant type declare in win32 are given.

  -- The To_C functions correspond to the functions Interfaces.C.To_C.
  -- For description of append_nul see the documentation for to_c in the LRM.   
  
  -- The To_Ada functions are similar to the ones from interfaces.c:        
  -- For the use of trim_nul see description of interfaces.c.to_ada in the LRM.

  	  
  function To_C   (Item       : in String;
                   Append_Nul : in Boolean := True)
           return Win32.Char_Array;

  function To_Ada (Item       : in Win32.Char_Array; 
                   Trim_Nul   : in Boolean := True)
           return String;
    
--  function To_C   (Item       : in String;
--                   Append_Nul : in Boolean := True)
--           return Win32.Pchar;
--
--  function To_C   (Item       : in String;
--                   Append_Nul : in Boolean := True)
--           return Win32.Pcch;

  -- The To_C functions are deactivated, because they do not work
  -- correctly, when a function from the win32ada binding with to
  -- strings is called.
  -- Unchecked_access on the first element of an interfaces.c.char_array
  -- should be used then.
           
  function To_Ada (Item       : in Win32.Pchar)
           return String;

  -- Use this function also for the subtypes of win32.pchar which are
  -- lpch, pch, npstr, lpstr, pstr, psz
           
  function To_Ada (Item       : in Win32.Pcch)
           return String;       
           
  -- Use this function also for the subtypes of win32.pcch which are
  -- pcchar, lpcch, lpcstr, pcstr
           
  ----------------------------------------------------------------------------              
  -- Conversion routines between ada strings and win32 unsigned_char pointers:
  -- One for each new access to variable type
  -- and one for each new access to constant type declare in win32 are given.

  function To_Ada (Item       : in Win32.Pbyte)
           return String;
           
  -- Use this function also for the subtypes of win32.pbyte which are
  -- lpbyte
           
  function To_Ada (Item       : in Win32.Puchar)
           return String;
           
  function To_Ada (Item       : in Win32.Pcbyte)
           return String;
           
  function To_Ada (Item       : in Win32.Pcuchar)
           return String;
           
  ----------------------------------------------------------------------------         
  -- Convert a char_array with ascii.nul delimiters to an
  -- array of string_access values.         
  -- The to_string_vector functions allocate memory on the heap.
  -- The returned string_vector(s) have to be deallocated with
  -- utilities.string_vectors.deallocate.
           
  function To_String_Vector(Item : in Interfaces.C.Char_Array)
           return Utilities.String_Vectors.String_Vector;
   
  function To_String_Vector(Item : in Interfaces.C.Strings.Chars_Ptr)
           return Utilities.String_Vectors.String_Vector;

  function To_String_Vector(Item : in Win32.Pchar)
           return Utilities.String_Vectors.String_Vector;

  function To_String_Vector(Item : in Win32.Pcch)
           return Utilities.String_Vectors.String_Vector;

  -- Convert a string_vector to C char_array. 
  -- The routines for chars_ctr, pchar, pcch are ommited, as they
  -- allocate memory on the heap.
            
  function To_Char_Array(Item : in Utilities.String_Vectors.String_Vector)
           return Interfaces.C.Char_Array;
                      
end Windows.Utils;           
 
