--                              -*- Mode: Ada -*-
-- Filename        : interfaces-c-pointers_to_constant.adb
-- Description     : This package is a duplicate of the predefined generic
--                   package interfaces.c.pointers that delivers subprograms 
--                   to handle pointers to C-like variables 
--                   (type Pointer is access all Element).
--                   interfaces-c-pointers_to_constant has the same interface
--                   but delivers subprograms to handle pointers to C-like 
--                   constants
--                   (type Pointer is access constant Element).
--
-- Author          : J. Schrer
-- Created On      : 20 Jan. 1998
-- Last Modified By: J. Schrer
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Interfaces.C.Pointers;
with System.Address_To_Access_Conversions;

package body Interfaces.C.Pointers_To_Constant is
  ----------------------------------------------------------------------------
  package Access_All_To_Constant is new 
          System.Address_To_Access_Conversions(Object => Element);     
       
  package C_Access_All is new Interfaces.C.Pointers
	 	     (Index              => Index,
		      Element            => Element,
		      Element_Array      => Element_Array,
		      Default_Terminator => Default_Terminator);
          
  use type C_Access_All.Pointer;        
  ---------------------------------------------------------------------------
  -- all subprograms are implemented as calls to the ones from the above
  -- instantiation of Interfaces.C.Pointers. Only type conversions from 
  -- 'access to all' to 'access to constant' "pointers" have to be done 
  -- with the above instantiation of System.Address_To_Access_Conversions. 
  ---------------------------------------------------------------------------   
  function Value
     (Ref        : in Pointer;
      Terminator : in Element := Default_Terminator)
      return       Element_Array is
  begin
    return C_Access_All.Value
          (Ref        => C_Access_All.Pointer
                        (Access_All_To_Constant.To_Pointer(Ref.all'Address)),
           Terminator => Terminator);
  end Value;
  ---------------------------------------------------------------------------   
  function Value
     (Ref    : in Pointer;
      Length : in Ptrdiff_T)
      return   Element_Array is
  begin
    return C_Access_All.Value
          (Ref    => C_Access_All.Pointer
                    (Access_All_To_Constant.To_Pointer(Ref.all'Address)),
           Length => Length);
  end Value;
  ---------------------------------------------------------------------------   
  function "+" (Left : in Pointer;   Right : in Ptrdiff_T) return Pointer is
  begin
    return Pointer(C_Access_All.Pointer
          (Access_All_To_Constant.To_Pointer(Left.all'Address)) + Right);
  end "+";
  ---------------------------------------------------------------------------   
  function "+" (Left : in Ptrdiff_T; Right : in Pointer)   return Pointer is
  begin
    return Pointer(Left + C_Access_All.Pointer
                 (Access_All_To_Constant.To_Pointer(Right.all'Address)));
  end "+";
  ---------------------------------------------------------------------------   
  function "-" (Left : in Pointer;   Right : in Ptrdiff_T) return Pointer is
  begin
    return Pointer(C_Access_All.Pointer
          (Access_All_To_Constant.To_Pointer(Left.all'Address)) - Right);
  end "-";
  ---------------------------------------------------------------------------   
  function "-" (Left : in Pointer;   Right : in Pointer)   return Ptrdiff_T is
  begin
    return C_Access_All.Pointer
          (Access_All_To_Constant.To_Pointer(Left.all'Address)) - 
           C_Access_All.Pointer
          (Access_All_To_Constant.To_Pointer(Right.all'Address));
  end "-";
  ---------------------------------------------------------------------------   
  procedure Increment (Ref : in out Pointer) is
  begin
    Ref := Ref + 1;
  end Increment;
  ---------------------------------------------------------------------------   
  procedure Decrement (Ref : in out Pointer) is
  begin
    Ref := Ref - 1;
  end Decrement;
  ---------------------------------------------------------------------------   
  function Virtual_Length
     (Ref        : in Pointer;
      Terminator : in Element := Default_Terminator)
      return       Ptrdiff_T is
  begin
    return C_Access_All.Virtual_Length
          (Ref        => C_Access_All.Pointer
                        (Access_All_To_Constant.To_Pointer(Ref.all'Address)),
           Terminator => Terminator);
  end Virtual_Length;
  ---------------------------------------------------------------------------   
  procedure Copy_Terminated_Array
     (Source     : in Pointer;
      Target     : in Pointer;
      Limit      : in Ptrdiff_T := Ptrdiff_T'Last;
      Terminator : in Element := Default_Terminator) is
  begin
    C_Access_All.Copy_Terminated_Array
      (Source     => C_Access_All.Pointer
                    (Access_All_To_Constant.To_Pointer(Source.all'Address)),
       Target     => C_Access_All.Pointer
                    (Access_All_To_Constant.To_Pointer(Target.all'Address)),
       Limit      => Limit,
       Terminator => Terminator);
  end Copy_Terminated_Array;
  ---------------------------------------------------------------------------   
  procedure Copy_Array
     (Source  : in Pointer;
      Target  : in Pointer;
      Length  : in Ptrdiff_T) is
  begin
    C_Access_All.Copy_Array
      (Source     => C_Access_All.Pointer
                    (Access_All_To_Constant.To_Pointer(Source.all'Address)),
       Target     => C_Access_All.Pointer
                    (Access_All_To_Constant.To_Pointer(Target.all'Address)),
       Length     => Length);
  end Copy_Array;
  ---------------------------------------------------------------------------   
end Interfaces.C.Pointers_To_Constant;

