--                              -*- Mode: Ada -*-
-- Filename        : interfaces-c-pointers_to_constant.ads
-- Description     : This package is a duplicate of the predefined generic
--                   package interfaces.c.pointers that delivers subprograms 
--                   to handle pointers to C-like variables (see declaration
--                   type Pointer is access all Element;).
--                   interfaces-c-pointers_to_constant has the same interface
--                   but delivers subprograms to handle pointers to C-like 
--                   constants (see declaration below
--                   type Pointer is access constant Element;).
--
--                   To use the subprograms correctly, refer to the 
--                   documentation for interfaces.c.pointers in the 
--                   LRM, Appendix B.3.2
--
-- Author          : J. Schrer
-- Created On      : 20 Jan. 1998
-- Last Modified By: J. Schrer
-- Last Modified On:
-- Update Count    : 1
-- Status          :

generic
   type Index is (<>);
   type Element is private;
   type Element_Array is array (Index range <>) of aliased Element;
   Default_Terminator : Element;

package Interfaces.C.Pointers_To_Constant is

   pragma Preelaborate (Pointers_To_Constant);

   type Pointer is access constant Element;

   function Value
     (Ref        : in Pointer;
      Terminator : in Element := Default_Terminator)
      return       Element_Array;

   function Value
     (Ref    : in Pointer;
      Length : in Ptrdiff_T)
      return   Element_Array;

   Pointer_Error : exception;

   --------------------------------
   -- C-style Pointer Arithmetic --
   --------------------------------

   function "+" (Left : in Pointer;   Right : in Ptrdiff_T) return Pointer;
   function "+" (Left : in Ptrdiff_T; Right : in Pointer)   return Pointer;
   function "-" (Left : in Pointer;   Right : in Ptrdiff_T) return Pointer;
   function "-" (Left : in Pointer;   Right : in Pointer)   return Ptrdiff_T;

   procedure Increment (Ref : in out Pointer);
   procedure Decrement (Ref : in out Pointer);

   pragma Convention (Intrinsic, "+");
   pragma Convention (Intrinsic, "-");
   pragma Convention (Intrinsic, Increment);
   pragma Convention (Intrinsic, Decrement);

   function Virtual_Length
     (Ref        : in Pointer;
      Terminator : in Element := Default_Terminator)
      return       Ptrdiff_T;

   procedure Copy_Terminated_Array
     (Source     : in Pointer;
      Target     : in Pointer;
      Limit      : in Ptrdiff_T := Ptrdiff_T'Last;
      Terminator : in Element := Default_Terminator);

   procedure Copy_Array
     (Source  : in Pointer;
      Target  : in Pointer;
      Length  : in Ptrdiff_T);

private
   pragma Inline ("+");
   pragma Inline ("-");
   pragma Inline (Decrement);
   pragma Inline (Increment);

end Interfaces.C.Pointers_To_Constant;
  
