--                              -*- Mode: Ada -*-
-- Filename        : utilities-c.ads
-- Description     : Utilities for interfacing with c
-- Author          : J. Schrer
-- Created On      : 14. Jan. 1998
-- Last Modified By:
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Interfaces.C;
with Interfaces.C.Pointers;

package Utilities.C is
  
  -----------------------------------------------------------------------------
  -- Utilities to manipulate arrays of interfaces.c.unsigned_char.
  
  type Unsigned_Char_Array is array(Interfaces.C.Size_T range <>) of aliased
       Interfaces.C.Unsigned_Char;
  
  -- equivalent to interfaces.c.char_array
       
  Unsigned_Nul : constant Interfaces.C.Unsigned_Char :=
                 Interfaces.C.Unsigned_Char
                (Interfaces.C.Char'Pos(Interfaces.C.Nul));     
       
  -- package to process Unsigned_Char_Array from an access to variable pointer.
  
  package Unsigned_Char_Variable is new Interfaces.C.Pointers
    (Index              => Interfaces.C.Size_T,
     Element            => Interfaces.C.Unsigned_Char,
     Element_Array      => Unsigned_Char_Array,
     Default_Terminator => Unsigned_Nul);
  
  ------------------------------------------------------   
  -- Higher functions to convert between C-like strings:
  -- The next two functions correspond to the functions interfaces.c.to_c.
  -- For description of append_nul see the documentation for to_c in the LRM.   
        
  function To_Char_Array        (Item       : in Unsigned_Char_Array;
                                 Append_Nul : in Boolean := True) 
           return Interfaces.C.Char_Array;
           
  -- Conversion of all elements from unsigned_char to char.
  -- Length of returned array is item'length + 1 if Append_Nul = true.

  function To_Unsigned_Char_Array(Item      : in Interfaces.C.Char_Array;
                                 Append_Nul : in Boolean := True) 
           return Unsigned_Char_Array;
           
  -- Conversion of all elements from char to unsigned_char.         
  -- Length of returned array is item'length + 1 if Append_Nul = true.
           
  -----------------------------------------------------------------------------         
  -- The To_Ada functions are similar to the ones from interfaces.c:        
  -- For the use of trim_nul see description of interfaces.c.to_ada in the LRM.
           
  function To_Ada(Item     : in Unsigned_Char_Array;
                  Trim_Nul : in Boolean := True)
           return String;
           
  -- Conversion to ada string using interfaces.c.to_ada(item, trim_nul).
           
  function To_Ada(Item     : in Unsigned_Char_Variable.Pointer;
                  Trim_Nul : in Boolean := True)
           return String;
           
  -- If item is null, the empty string "" is returned.
  -- (Interfaces.C.Strings.Dereference_Error is not raised in value.)
  -- If item /= null converts item to ada string using 
  -- interfaces.c.to_ada(item, trim_nul).

  ----------------------------------------------------------------------------
         
  type C_Float_Vector is array(Natural range <>) of aliased 
       Interfaces.C.C_Float;          
                      
  package C_Float_Pointers is new Interfaces.C.Pointers
	 	     (Index              => Natural,
		      Element            => Interfaces.C.C_Float,
		      Element_Array      => C_Float_Vector,
		      Default_Terminator => Interfaces.C.C_Float'First);
          
  type Double_Vector is array(Natural range <>) of aliased 
       Interfaces.C.Double;          
                      
  -- Pointer objects of type pfloat          
  -- have to be converted to c_float_pointers.pointer for calls to subprograms of
  -- c_float_Pointers. 

  package Double_Pointers is new Interfaces.C.Pointers
	 	     (Index              => Natural,
		      Element            => Interfaces.C.Double,
		      Element_Array      => Double_Vector,
		      Default_Terminator => Interfaces.C.Double'First);
          
end Utilities.C;
