--                              -*- Mode: Ada -*-
-- Filename        : utilities-character_utilities.adb
-- Description     : Taken from "Software components with Ada" by Grady Booch
--                   p.391
-- Author          : J.Schroeer
-- Created On      : Wed Mar 26 17:13:13 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  2 13:38:39 1997
-- Update Count    : 2
-- Status          :

package body Utilities.Character_Utilities is

  Lexical_Offset      : constant Integer := Character'Pos('A');
  Convert_Case_Offset : constant Integer := Character'Pos('a') - Lexical_Offset;
  Digit_Offset        : constant Integer := Character'Pos('0');
  -----------------------------------------------------------------------------
  procedure Make_Uppercase (The_Character : in out Character) is
  begin
    if The_Character in Lowercase_Character then
      The_Character := Character'Val
                        (Character'Pos(The_Character) - Convert_Case_Offset);
    end if;
  end Make_Uppercase;
  -----------------------------------------------------------------------------
  procedure Make_Lowercase (The_Character : in out Character) is
  begin
    if The_Character in Uppercase_Character then
      The_Character := Character'Val
                        (Character'Pos(The_Character) + Convert_Case_Offset);
    end if;
  end Make_Lowercase;
  -----------------------------------------------------------------------------
  function Uppercase (The_Character : in Character) return Character is
  begin
    if The_Character in Lowercase_Character then
      return Character'Val(Character'Pos(The_Character) - Convert_Case_Offset);
    else
      return The_Character;
    end if;
  end Uppercase;
  -----------------------------------------------------------------------------
  function Lowercase (The_Character : in Character) return Character is
  begin
    if The_Character in Uppercase_Character then
      return Character'Val(Character'Pos(The_Character) + Convert_Case_Offset);
    else
      return The_Character;
    end if;
  end Lowercase;
  -----------------------------------------------------------------------------
  function Is_Control (The_Character : in Character) return Boolean is
  begin
    return The_Character in Control_Character;
  end Is_Control;
  -----------------------------------------------------------------------------
  function Is_Graphic (The_Character : in Character) return Boolean is
  begin
    return The_Character in Graphic_Character;
  end Is_Graphic;
  -----------------------------------------------------------------------------
  function Is_Uppercase (The_Character : in Character) return Boolean is
  begin
    return The_Character in Uppercase_Character;
  end Is_Uppercase;
  -----------------------------------------------------------------------------
  function Is_Lowercase (The_Character : in Character) return Boolean is
  begin
    return The_Character in Lowercase_Character;
  end Is_Lowercase;
  -----------------------------------------------------------------------------
  function Is_Digit (The_Character : in Character) return Boolean is
  begin
    return The_Character in Digit_Character;
  end Is_Digit;
  -----------------------------------------------------------------------------
  function Is_Alphabetic (The_Character : in Character) return Boolean is
  begin
    return Is_Uppercase (The_Character) or else Is_Lowercase(The_Character);
  end Is_Alphabetic;
  -----------------------------------------------------------------------------
  function Is_Alphanumeric (The_Character : in Character) return Boolean is
  begin
    return Is_Uppercase(The_Character) or else Is_Lowercase(The_Character)
           or else Is_Digit(The_Character);
  end Is_Alphanumeric;
  -----------------------------------------------------------------------------
  function Value_Of (The_Character : in Character) return Digit is
  begin
    if The_Character in '0' .. '9' then
      return Character'Pos(The_Character) - Digit_Offset;
    elsif Uppercase(The_Character) in 'A' .. 'F' then
      return Character'Pos(The_Character) - Lexical_Offset + 10;
    else
      raise Lexical_Error;
    end if;
  end Value_Of;
  -----------------------------------------------------------------------------
  function Image_Of (The_Digit : in Digit) return Character is
  begin
    if The_Digit in 0 .. 9 then
      return Character'Val(Digit_Offset + The_Digit);
    elsif The_Digit in 10 .. 15 then
      return Character'Val(Lexical_Offset + The_Digit - 10);
    else
      raise Lexical_Error;
    end if;
  end Image_Of;
  -----------------------------------------------------------------------------
  function Index_Of (The_Character : in Character) return Letter is
  begin
    if Is_Uppercase(The_Character) or else Is_Lowercase(The_Character) then
      return Character'Pos(Uppercase(The_Character)) - Lexical_Offset;
    else
      raise Lexical_Error;
    end if;
  end Index_Of;
  -----------------------------------------------------------------------------
  function Uppercase_Of (The_Letter : in Letter) return Character is
  begin
    return Character'Val(The_Letter + Lexical_Offset - 1);
  end Uppercase_Of;
  -----------------------------------------------------------------------------
  function Lowercase_Of (The_Letter : in Letter) return Character is
  begin
    return Character'Val(The_Letter + Lexical_Offset - 1 + Convert_Case_Offset);
  end Lowercase_Of;
  -----------------------------------------------------------------------------
  function Is_Equal (Left           : in Character;
                     Right          : in Character;
                     Case_Sensitive : in Boolean := True)
                     return Boolean is
  begin
    if Case_Sensitive then
      return Left = Right;
    else
      return Uppercase(Left) = Uppercase(Right);
    end if;
  end Is_Equal;
  -----------------------------------------------------------------------------
  function Is_Less_Than (Left          : in Character;
                         Right          : in Character;
                         Case_Sensitive : in Boolean := True)
                         return Boolean is
  begin
    if Case_Sensitive then
      return Left < Right;
    else
      return Uppercase(Left) < Uppercase(Right);
    end if;
  end Is_Less_Than;
  -----------------------------------------------------------------------------
  function Is_Greater_Than (Left           : in Character;
                            Right          : in Character;
                            Case_Sensitive : in Boolean := True)
                            return Boolean is
  begin
    if Case_Sensitive then
      return Left > Right;
    else
      return Uppercase(Left) > Uppercase(Right);
    end if;
  end Is_Greater_Than;
  -----------------------------------------------------------------------------
end Utilities.Character_Utilities;

