--                              -*- Mode: Ada -*-
-- Filename        : utilities-character_utilities.ads
-- Description     : Taken from "Software components with Ada" by Grady Booch
--                   p.391
-- Author          : J. Schroeer
-- Created On      : Wed Mar 26 17:14:19 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  2 13:38:38 1997
-- Update Count    : 2
-- Status          : Unknown, Use with caution!

package Utilities.character_utilities is

  subtype control_character   is character range ascii.nul .. ascii.us;
  subtype graphic_character   is character range       ' ' .. '~';
  subtype uppercase_character is character range       'A' .. 'Z';
  subtype lowercase_character is character range       'a' .. 'z';
  subtype digit_character     is character range       '0' .. '9';
  subtype digit               is integer   range         0 .. 15;
  subtype letter              is integer   range         1 .. 26;

  procedure make_uppercase (the_character : in out character);

  procedure make_lowercase (the_character : in out character);

  function uppercase       (the_character : in character) return character;

  function lowercase       (the_character : in character) return character;

  function is_control      (the_character : in character) return boolean;

  function is_graphic      (the_character : in character) return boolean;

  function is_uppercase    (the_character : in character) return boolean;

  function is_lowercase    (the_character : in character) return boolean;

  function is_digit        (the_character : in character) return boolean;

  function is_alphabetic   (the_character : in character) return boolean;

  function is_alphanumeric (the_character : in character) return boolean;

  function value_of        (the_character : in character) return digit;

  function image_of        (the_digit     : in digit)     return character;

  function index_of        (the_character : in character) return letter;

  function uppercase_of    (the_letter    : in letter)    return character;

  function lowercase_of    (the_letter    : in letter)    return character;

  function is_equal        (left           : in character;
                            right          : in character;
                            case_sensitive : in boolean := true) return boolean;

  function is_less_than    (left           : in character;
                            right          : in character;
                            case_sensitive : in boolean := true) return boolean;

  function is_greater_than (left           : in character;
                            right          : in character;
                            case_sensitive : in boolean := true) return boolean;

  lexical_error : exception;

end Utilities.character_utilities;

