--                              -*- Mode: Ada -*-
-- Filename        : utilities-code_generator.adb
-- Description     : Routines for Ada code generation.
-- Author          : J. Schroeer
-- Created On      : Tue Jul  1 15:55:36 1997
-- Last Modified By:
-- Last Modified On: Mon Jul 21 15:27:45 1997
-- Update Count    : 17
-- Status          :

with Utilities.Calendar_Utilities;
with Utilities.String_Utilities;
with Utilities.Files;
with Portable.Operating_System;
with System;
with Ada.Calendar, Ada.Command_Line;

package body Utilities.Code_Generator is

  use Ada.Text_Io;
  ----------------------------------------------------------------------------
  procedure Put_Head (File              : in Ada.Text_Io.File_Type;
                      Description       : in String := "";
                      Created_From_File : in String := "") is

    Time        : constant Ada.Calendar.Time := Ada.Calendar.Clock;
    Filename    : constant String := Ada.Text_Io.Name(File);
    User        : constant String := Portable.Operating_System.
                                     Environment_Value_Of("USER");
    Program     : constant String := Ada.Command_Line.Command_Name;
  begin
    Put_Line(File,"--                              -*- Mode: Ada -*-");
    Put     (File,"-- Filename        : ");
    if Filename'Length < 50 then
      Put_Line(File, Filename);
    else
      declare
        Relative_Name : constant String :=
                        Utilities.Files.Relative_Name_Of(Filename);
      begin
        Put_Line(File, Filename(1 .. Filename'Length - Relative_Name'Length));
        Put_Line(File,"--                   " & Relative_Name);
      end;
    end if;
    Put     (File,"-- Description     : ");
    Put_Line(File, Utilities.String_Utilities.Exchange
                  (In_The_String => Description,
                   Old_String    => Portable.Operating_System.New_Line,
                   New_String    => Portable.Operating_System.New_Line &
                  "--                   "));
    Put_Line(File,"-- Author          : " & User);
    Put_Line(File,"-- Created On      : " &
                      Utilities.Calendar_Utilities.Image_Of
                     (Utilities.Calendar_Utilities.Time_Of(Time)));
    Put_Line(File,"-- Last Modified By:");
    Put_Line(File,"-- Last Modified On:");
    Put_Line(File,"-- Update Count    : 1");
    Put_Line(File,"-- Status          : ok");
    Put_Line(File,
    "-----------------------------------------------------------------------");
    Put_Line(File,"-- Remarks: Operating system : " &
                      Portable.Operating_System.
                      Environment_Value_Of("HOSTTYPE"));
    Put_Line(File,"--          Ada Compiler     : " &
                               System.Name'Image(System.System_Name));
    Put_Line(File,"--          Computer node    : " &
                      Portable.Operating_System.
                      Environment_Value_Of("SESSION_SVR"));
    if Created_From_File /= "" then
      Put_Line(File,"--");
      Put_Line(File,"--          Created by       : " & Program);
      Put_Line(File,"--          Input-file       : " & Created_From_File);
      Put_Line(File,"--          File time stamp  : " &
               Utilities.Calendar_Utilities.Image_Of
              (Utilities.Calendar_Utilities.Time_Of
             (Portable.Operating_System.File_Time_Stamp(Created_From_File))));
      Put_Line(File,"--");
      Put     (File,"--          Command          : " & Program);
      for I in 1 .. Ada.Command_Line.Argument_Count loop
        Put(File, " " & Ada.Command_Line.Argument(I));
      end loop;
      New_Line(File);
    end if;
    Put_Line(File,
    "-----------------------------------------------------------------------");
    New_Line(File);
  end Put_Head;
  ----------------------------------------------------------------------------
  procedure Put_Lines(File : in Ada.Text_Io.File_Type;

                      Line1, Line2, Line3, Line4, Line5,
                      Line6, Line7, Line8, Line9, Line10,

                      Line11,Line12,Line13,Line14,Line15,
                      Line16,Line17,Line18,Line19,Line20,

                      Line21,Line22,Line23,Line24,Line25,
                      Line26,Line27,Line28,Line29,Line30,

                      Line31,Line32,Line33,Line34,Line35,
                      Line36,Line37,Line38,Line39,Line40,

                      Line41,Line42,Line43,Line44,Line45,
                      Line46,Line47,Line48,Line49,Line50,

                      Line51,Line52,Line53,Line54,Line55,
                      Line56,Line57,Line58,Line59,Line60,

                      Offset : in String := "") is
  begin
    if Line1  /= "" then Put_Line(File, Offset & Line1);  else return; end if;
    if Line2  /= "" then Put_Line(File, Offset & Line2);  else return; end if;
    if Line3  /= "" then Put_Line(File, Offset & Line3);  else return; end if;
    if Line4  /= "" then Put_Line(File, Offset & Line4);  else return; end if;
    if Line5  /= "" then Put_Line(File, Offset & Line5);  else return; end if;
    if Line6  /= "" then Put_Line(File, Offset & Line6);  else return; end if;
    if Line7  /= "" then Put_Line(File, Offset & Line7);  else return; end if;
    if Line8  /= "" then Put_Line(File, Offset & Line8);  else return; end if;
    if Line9  /= "" then Put_Line(File, Offset & Line9);  else return; end if;
    if Line10 /= "" then Put_Line(File, Offset & Line10); else return; end if;

    if Line11 /= "" then Put_Line(File, Offset & Line11); else return; end if;
    if Line12 /= "" then Put_Line(File, Offset & Line12); else return; end if;
    if Line13 /= "" then Put_Line(File, Offset & Line13); else return; end if;
    if Line14 /= "" then Put_Line(File, Offset & Line14); else return; end if;
    if Line15 /= "" then Put_Line(File, Offset & Line15); else return; end if;
    if Line16 /= "" then Put_Line(File, Offset & Line16); else return; end if;
    if Line17 /= "" then Put_Line(File, Offset & Line17); else return; end if;
    if Line18 /= "" then Put_Line(File, Offset & Line18); else return; end if;
    if Line19 /= "" then Put_Line(File, Offset & Line19); else return; end if;
    if Line20 /= "" then Put_Line(File, Offset & Line20); else return; end if;

    if Line21 /= "" then Put_Line(File, Offset & Line21); else return; end if;
    if Line22 /= "" then Put_Line(File, Offset & Line22); else return; end if;
    if Line23 /= "" then Put_Line(File, Offset & Line23); else return; end if;
    if Line24 /= "" then Put_Line(File, Offset & Line24); else return; end if;
    if Line25 /= "" then Put_Line(File, Offset & Line25); else return; end if;
    if Line26 /= "" then Put_Line(File, Offset & Line26); else return; end if;
    if Line27 /= "" then Put_Line(File, Offset & Line27); else return; end if;
    if Line28 /= "" then Put_Line(File, Offset & Line28); else return; end if;
    if Line29 /= "" then Put_Line(File, Offset & Line29); else return; end if;
    if Line30 /= "" then Put_Line(File, Offset & Line30); else return; end if;

    if Line31 /= "" then Put_Line(File, Offset & Line31); else return; end if;
    if Line32 /= "" then Put_Line(File, Offset & Line32); else return; end if;
    if Line33 /= "" then Put_Line(File, Offset & Line33); else return; end if;
    if Line34 /= "" then Put_Line(File, Offset & Line34); else return; end if;
    if Line35 /= "" then Put_Line(File, Offset & Line35); else return; end if;
    if Line36 /= "" then Put_Line(File, Offset & Line36); else return; end if;
    if Line37 /= "" then Put_Line(File, Offset & Line37); else return; end if;
    if Line38 /= "" then Put_Line(File, Offset & Line38); else return; end if;
    if Line39 /= "" then Put_Line(File, Offset & Line39); else return; end if;
    if Line40 /= "" then Put_Line(File, Offset & Line40); else return; end if;

    if Line41 /= "" then Put_Line(File, Offset & Line41); else return; end if;
    if Line42 /= "" then Put_Line(File, Offset & Line42); else return; end if;
    if Line43 /= "" then Put_Line(File, Offset & Line43); else return; end if;
    if Line44 /= "" then Put_Line(File, Offset & Line44); else return; end if;
    if Line45 /= "" then Put_Line(File, Offset & Line45); else return; end if;
    if Line46 /= "" then Put_Line(File, Offset & Line46); else return; end if;
    if Line47 /= "" then Put_Line(File, Offset & Line47); else return; end if;
    if Line48 /= "" then Put_Line(File, Offset & Line48); else return; end if;
    if Line49 /= "" then Put_Line(File, Offset & Line49); else return; end if;
    if Line50 /= "" then Put_Line(File, Offset & Line50); else return; end if;

    if Line51 /= "" then Put_Line(File, Offset & Line51); else return; end if;
    if Line52 /= "" then Put_Line(File, Offset & Line52); else return; end if;
    if Line53 /= "" then Put_Line(File, Offset & Line53); else return; end if;
    if Line54 /= "" then Put_Line(File, Offset & Line54); else return; end if;
    if Line55 /= "" then Put_Line(File, Offset & Line55); else return; end if;
    if Line56 /= "" then Put_Line(File, Offset & Line56); else return; end if;
    if Line57 /= "" then Put_Line(File, Offset & Line57); else return; end if;
    if Line58 /= "" then Put_Line(File, Offset & Line58); else return; end if;
    if Line59 /= "" then Put_Line(File, Offset & Line59); else return; end if;
    if Line60 /= "" then Put_Line(File, Offset & Line60); else return; end if;
  end Put_Lines;
  -----------------------------------------------------------------------------
  procedure Create_Empty_Package(Name   : in String;
                                 Source : in String := "") is
    ---------------------------------------------------
    function Exists(Name : in String) return Boolean is
    begin
      if Utilities.Files.Exists(Name) then
        return True;
      elsif Utilities.String_Utilities.Uppercase
              (System.Name'Image(System.System_Name)) = "SYSTEM_NAME_GNAT" then
        return Portable.Operating_System.Locate_Regular_File
              (File_Name => Name,
               Path      => Portable.Operating_System.
                            Environment_Value_Of("ADA_INCLUDE_PATH")) /= "";
      else
        return Portable.Operating_System.Locate_Regular_File(Name) /= "";
      end if;
    end Exists;
    -----------------------------------------------------------------
    File     : Ada.Text_Io.File_Type;
    Filename : constant String := Utilities.String_Utilities.Replaced
                                 (The_Character      => '.',
                                  With_The_Character => '-',
                                  In_The_String      => Name) & ".ads";
  begin
    if not Exists(Filename) then
      Ada.Text_Io.Create(File, Ada.Text_Io.Out_File, Filename);
      Put_Head(File              => File,
               Description       => "Empty parent package",
               Created_From_File => Source);
      Ada.Text_Io.Put_Line(File, "package " & Name & " is end;");
      Ada.Text_Io.Put_Line(Utilities.String_Utilities.Right_Justified
             (Natural'Image(Natural(Ada.Text_Io.Line(File))),
              In_The_Width => 5) & " lines on file " &
              Ada.Text_Io.Name(File) & " created.");
      Ada.Text_Io.Close(File);
    end if;
  end Create_Empty_Package;
  -----------------------------------------------------------------------------
end Utilities.Code_Generator;
