--                              -*- Mode: Ada -*-
-- Filename        : utilities-code_generator.ads
-- Description     : Routines for Ada code generation.
-- Author          : J. Schroeer
-- Created On      : Tue Jul  1 15:56:33 1997
-- Last Modified By:
-- Last Modified On: Mon Jul 21 10:06:58 1997
-- Update Count    : 8
-- Status          :

with Ada.Text_Io;

package Utilities.Code_Generator is

  procedure Put_Head (File              : in Ada.Text_Io.File_Type;
                      Description       : in String := "";
                      Created_From_File : in String := "");

  -- Writes head in Emacs Ada style, to file, example:


  --                              -*- Mode: Ada -*-
  -- Filename        : /home/schroeer/ada95/tools/coco/cocosyn.s
  -- Chopped filename: coco-syntax_analyser.ads
  -- Description     : Generated parser
  -- Author          : schroeer
  -- Created On      : WEDNESDAY JULY      16, 1997, 10:20:27:00
  -- Last Modified By:
  -- Last Modified On:
  -- Update Count    : 1
  -- Status          : ok
  -----------------------------------------------------------------------
  -- Remarks: Operating system : solaris2.5
  --          Ada Compiler     : SYSTEM_NAME_GNAT
  --          Computer node    : prg11
  --
  --          Created by       : coco95
  --          Input-file       : /home/schroeer/ada95/tools/coco/coco.ag
  --          File time stamp  : MONDAY    JULY      14, 1997, 11:44:31:00
  --
  --          Command          : coco95 coco.ag
  -----------------------------------------------------------------------


  -- Filename         is ada.text_io.name(file).
  -- Desciption       may be expanded to multiple lines using string for
  --                  new_line (single character standard.ascii.lf on unix
  --                  files, see function operating_system.new_line).
  --                  No comment prefixes '--' have to be given in the
  --                  continuation lines and the lines should be short,
  --                  as they are aligned with the rest.
  -- Author           is environment variable USER.
  -- Created on       is current date and time = ada.calendar.clock.
  -- Last Modified By is blank
  -- Last Modified On is blank
  -- Update Count     is 1
  -- Status           is ok
  -- Operating system is environment variable HOSTTYPE.
  -- Ada Compiler     is system.system_name.
  -- Computer node    is environment variable SESSION_SVR.

  -- If created_from_file /= "" then the following fields are written:

  -- Created by      is the generating program = ada.command_line.command_name,
  -- Input-file      = created_from_file,
  -- File time stamp is the time-stamp of input-file,
  -- Command         = loop over ada.command_line.argument(s).

  -- If long_filename /= "" then a line like

  -- Chopped filename: long_filename

  -- is written, showing, that the generated file will be chopped using
  -- gnatchop.


  procedure put_lines(file : in ada.text_io.file_type;
                      line1, line2, line3, line4, line5,
                      line6, line7, line8, line9, line10,

                      line11,line12,line13,line14,line15,
                      line16,line17,line18,line19,line20,

                      line21,line22,line23,line24,line25,
                      line26,line27,line28,line29,line30,

                      line31,line32,line33,line34,line35,
                      line36,line37,line38,line39,line40,

                      line41,line42,line43,line44,line45,
                      line46,line47,line48,line49,line50,

                      line51,line52,line53,line54,line55,
                      line56,line57,line58,line59,line60,

                      offset : in string := "");

  -- Writing up to 60 lines to file.
  -- Procedure is left via return when first line = "" is recognized.
  -- Offset is for blanks or comments.
  -- Ada.text_io.put_line(file, offset & linexx)
  -- is called for all lines till the first empty string is encountered.


  procedure Create_Empty_Package(Name   : in String;
                                 Source : in String := "");

  -- If no file with the name 'name'.ads exists on the $PATH,
  -- creates an empty parent package with that filename.
  -- If the compiler (system.system_name) is GNAT, then $ADA_INCUDE_PATH is
  -- taken instead of $PATH.

  -- Source is the name of the input file of the generation program.

end Utilities.Code_Generator;
