--                              -*- Mode: Ada -*-
-- Filename        : utilities-files.adb
-- Description     :
-- Author          : J. Schrer, AMST
-- Created On      : Wed Feb  5 14:41:56 1997
-- Last Modified By:
-- Last Modified On: Mon Apr  7 12:07:57 1997
-- Update Count    : 13
-- Status          : ok

with Ada.Text_Io,
     Portable.Operating_System;

package body Utilities.Files is
  -----------------------------------------------------------------------------
--   function has(the_string, the_substring : in string) return boolean is
--   begin
--     return string_utilities.location_of(the_string    => the_substring,
--                                         in_the_string => the_string) > 0;
--   end has;
  -----------------------------------------------------------------------------
  function Exists(Pathname : in String) return Boolean is
    File : Ada.Text_Io.File_Type;
  begin
    Ada.Text_Io.Open(File, Ada.Text_Io.In_File, Pathname);
    Ada.Text_Io.Close(File);
    return True;
  exception
    when Ada.Text_Io.Use_Error  => return True;
         -- External file with name pathname is already open,
         -- see LRM, chapter A.8.2 File Management, procedure open
         -- see e.g. GNAT reference manual chapter 6.8, shared files.
         -- If pathname is a directory use_error is raised as well.
    when Ada.Text_Io.Name_Error => return False;
         -- External file with name pathname does not exist,
         -- see LRM, chapter A.8.2 File Management, procedure open.
  end Exists;
  -----------------------------------------------------------------------------
  function Suffix_Of(Pathname : in String) return String is

    Directory_Separator : constant Character :=
                          Portable.Operating_System.Directory_Separator;
  begin
    for I in reverse Pathname'range loop
      exit when Pathname(I) = Directory_Separator and then I < Pathname'Last;
      if Pathname(I) = '.' and then I < Pathname'Last then
        return Pathname(I .. Pathname'Last);
      end if;
    end loop;
    return "";
  end Suffix_Of;
  -----------------------------------------------------------------------------
  function Prefix_Of(Pathname : in String) return String is
  begin
    for I in reverse Pathname'range loop
      if Pathname(I) = '.' then
        if I > Pathname'First then
          return Pathname(Pathname'First .. I - 1);
        else
          return "";
        end if;
      end if;
    end loop;
    return Pathname;
  end Prefix_Of;
  -----------------------------------------------------------------------------
  function Relative_Name_Of  (Pathname : in String;
                              Slashes  : in Natural := 0) return String is

--    Directory_Separator : constant Character :=
--                          Operating_system.Directory_Separator;
    Count : Natural := 0;
  begin
    for I in reverse Pathname'range loop
--      if pathname(i) = Directory_Separator then
      -- In GNAT 3.09 for Windows the gnat_oslib function returns '\'
      -- but in filenames returned by ada.text_io.name there is a '/'.
      if Pathname(I) = '/' or else Pathname(I) = '\' then
        if Count = Slashes then
          if I < Pathname'Last then
            return Pathname(I + 1 .. Pathname'Last);
          else
            return ""; -- pathname(pathname'last) = Directory_Separator
          end if;
        end if;
        Count := Count + 1;
      end if;
    end loop;
    return Pathname;
  end Relative_Name_Of;
  -----------------------------------------------------------------------------
end Utilities.Files;