--                              -*- Mode: Ada -*-
-- Filename        : utilities-files.ads
-- Description     : Utilities for files
-- Author          : J. Schroeer
-- Created On      : Wed Feb  5 14:32:00 1997
-- Last Modified By:
-- Last Modified On: Mon Apr  7 12:23:11 1997
-- Update Count    : 9
-- Status          : ok

package Utilities.Files is

  function Exists       (pathname : in string) return Boolean;

  -- True if file with name pathname exists, false otherwise.
  -- Works also for already open files (USE_ERROR is handled).

  function Prefix_Of    (pathname : in string) return String;

  -- Gives parts of pathname in front of a '.'.
  -- If pathname does not contain a '.', pathname is returned.
  --
  -- prefix_of("home/ada/../utilities-files.ads") =
  -- "home/ada/../utilities-files"

  function Suffix_Of    (pathname : in string) return string;

  -- Gives parts of pathname after a '.'.
  -- If pathname does not contain a '.', the empty string "" is returned.
  --
  -- Suffix_of("home/ada/../utilities-files.ads") = ".ads"

  function Relative_name_Of  (pathname : in string;
                              slashes  : in natural := 0) return string;

  -- Strips parts of pathname, gives relative filename by counting the
  -- character operating_system.directory_separator. Returns a string,
  -- that contains 'slashes' times this character.
  --
  -- Relative_name_Of("/home/ada/utilities-files.ads") =
  --   "utilities-files.ads"
  -- Relative_name_Of("/home/ada/utilities-files.ads", slashes => 1) =
  --   "ada/utilities-files.ads"
  -- Relative_name_Of("/home/ada/utilities-files.ads", slashes => 10) =
  --   "/home/ada/utilities-files.ads"

end Utilities.Files;