--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-block_stack.ads
-- Description     : Efficient unbounded stack
-- Author          : A. Schwarte, J. Schroeer
-- Created On      : Wed Jul  2 12:43:19 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  2 13:46:38 1997
-- Update Count    : 3
-- Status          :

-- Copyright (C) 1988 by dSPACE GmbH, FRG, Paderborn
--
-- author : Albert Schwarte
-- date   : 14-Mar-88   Vs 1.0  initial release
--          09-Jun-88   Vs 1.1  block_size now generic parameter
--                              new exception block_size_error
--

generic
  Block_Size_In_Byte : in Natural := 1024;

  type Element       is private;
  type Index         is range <>;
  type Vector        is array (Index range <>) of Element;
  type Vector_Access is access all Vector;

package Utilities.Generics.Block_Stack is

--  type stack is limited private;

  type Stack is private;

  procedure Push (Item : in     Element;
                  Onto : in out Stack);

  procedure Pop (Item :    out Element;
                 From : in out Stack);

  function  Top_Of (The_Stack       : in Stack) return Element;

  procedure Change_Top (Of_Stack    : in out Stack;
                        To_Element  : in     Element);

  function Depth_Of (The_Stack : in Stack) return Index;

  function Is_Empty (The_Stack : in Stack) return Boolean;

  procedure Convert (The_Stack     : in out Stack;
                     To_Vector     : in out Vector;
                     Reverse_Order : in     Boolean := False);

  procedure Convert (The_Stack     : in out Stack;
                     To_Vector     : in out Vector_Access;
                     Reverse_Order : in     Boolean := False);

  procedure Deallocate (The_Stack : in out Stack);

  Underflow, Overflow,
  Vector_Conversion_Error, Block_Size_Error : exception;

private

  type Block_Cell;
  type Block_Link is access Block_Cell;

  type Stack is record
    Depth             : Index := 0;
    Position_In_Block : Index := 0;
    Current_Block,
    First_Block       : Block_Link;
  end record;

end Utilities.Generics.Block_Stack;
