--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-doubly_linked_list.ads
-- Description     : Doubly linked list
-- Author          : J. Schrer
-- Created On      : Wed Jul  2 13:43:18 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  2 14:49:28 1997
-- Update Count    : 2
-- Status          :

with Ada.Unchecked_Deallocation;

package body Utilities.Generics.Doubly_Linked_List is

  type Cell is
    record
      The_Item   : Item;
      Previous   : Pointer;
      Next       : Pointer;
    end record;
  ----------------------------------------------------------------------------
  procedure Deallocate (Access_Value : in out Pointer) is
    procedure Deallocate_Pointer is new Ada.Unchecked_Deallocation
	     (Object => Cell, Name => Pointer);
  begin
    if Access_Value /= null then
      Deallocate (Access_Value.The_Item);
      Deallocate_Pointer (Access_Value);
    end if;
  end Deallocate;
  ----------------------------------------------------------------------------
  procedure Append      (The_Item     : in     Item;
			 To_The_List  : in out List) is
  begin
    if To_The_List.Tail = null then
      To_The_List.Tail := new Cell'(The_Item   => The_Item,
				    Previous   => null,
				    Next       => null);
      To_The_List.Head    := To_The_List.Tail;
      To_The_List.Current := To_The_List.Tail;
    else
      To_The_List.Tail.Next := new Cell'(The_Item   => The_Item,
					 Previous   => To_The_List.Tail,
					 Next       => null);
      To_The_List.Tail := To_The_List.Tail.Next;
    end if;
  exception
    when storage_error => raise overflow;
  end append;
  ----------------------------------------------------------------------------
  procedure append      (the_list     : in out list;
			 to_the_list  : in out list) is
  begin
    if to_the_list.tail /= null then
      to_the_list.tail.next := the_list.head;
      if the_list.head /= null then
	the_list.head.previous := to_the_list.tail;
	to_the_list.tail       := the_list.tail;
      end if;
    else
      To_The_List := List'(Head    => The_List.Head,
			   Current => The_List.Head,
			   Tail    => The_List.Tail);
    end if;
    The_List := List'(Head => null, Current => null, Tail => null);
  end Append;
  ----------------------------------------------------------------------------
  procedure Insert      (The_Item     : in     Item;
			 In_The_List  : in out List) is
  begin
    if In_The_List.Current = null then
      In_The_List.Current := new Cell'
			    (The_Item   => The_Item,
			     Previous   => null,
			     Next       => null);
      In_The_List.Head := In_The_List.Current;
      In_The_List.Tail := In_The_List.Current;
    else
      In_The_List.Current := new Cell'
			    (The_Item   => The_Item,
			     Previous   => In_The_List.Current.Previous,
			     Next       => In_The_List.Current);
      if In_The_List.Current.Previous /= null then
	In_The_List.Current.Previous.Next := In_The_List.Current;
      else
	In_The_List.Head := In_The_List.Current;
      end if;
      In_The_List.Current.Next.Previous := In_The_List.Current;
    end if;
  exception
    when Storage_Error => raise Overflow;
  end Insert;
  ----------------------------------------------------------------------------
  procedure Insert      (The_List     : in out List;
			 In_The_List  : in out List) is
    Temp : Pointer;
  begin
    if The_List.Head /= null then
      if In_The_List.Current /= null then
	Temp := In_The_List.Current;
	In_The_List.Current    := The_List.Head;
	The_List.Head.Previous := Temp.Previous;
	if The_List.Head.Previous /= null then
	  The_List.Head.Previous.Next := In_The_List.Current;
	else
	  In_The_List.Head := In_The_List.Current;
	end if;
	The_List.Tail.Next          := Temp;
	The_List.Tail.Next.Previous := The_List.Tail;
      else
	In_The_List := List'(Head    => The_List.Head,
			     Current => The_List.Head,
			     Tail    => The_List.Tail);
      end if;
    end if;
    The_List := List'(Head => null, Current => null, Tail => null);
  end Insert;
  ----------------------------------------------------------------------------
  procedure Add_On_Head (The_Item     : in     Item;
			 To_The_List  : in out List) is
  begin
    To_The_List.Head := new Cell'(The_Item   => The_Item,
				  Previous   => null,
				  Next       => To_The_List.Head);
    if To_The_List.Head.Next = null then
      To_The_List.Current := To_The_List.Head;
      To_The_List.Tail    := To_The_List.Head;
    else
      To_The_List.Head.Next.Previous := To_The_List.Head;
    end if;
  exception
    when Storage_Error => raise Overflow;
  end Add_On_Head;
  ----------------------------------------------------------------------------
  procedure Add_On_Head (The_List     : in out List;
			 To_The_List  : in out List) is
  begin
    Append (The_List => To_The_List, To_The_List => The_List);
    To_The_List := The_List;
    The_List := List'(Head => null, Current => null, Tail => null);
  end Add_On_Head;
  ----------------------------------------------------------------------------
  procedure Remove_Head     (In_The_List : in out List) is
    Temp : Pointer;
  begin
    if In_The_List.Head /= null then
      if In_The_List.Current = In_The_List.Head then
	In_The_List.Current := In_The_List.Head.Next;
      end if;
      Temp := In_The_List.Head;
      if In_The_List.Tail = In_The_List.Head then
	In_The_List := List'(Head => null, Current => null, Tail => null);
      else
	In_The_List.Head          := In_The_List.Head.Next;
	In_The_List.Head.Previous := null;
      end if;
      Deallocate (Temp);
    end if;
  end Remove_Head;
  ----------------------------------------------------------------------------
  procedure Remove_Tail     (In_The_List : in out List) is
    Temp : Pointer;
  begin
    if In_The_List.Tail /= null then
      if In_The_List.Current = In_The_List.Tail then
	In_The_List.Current := In_The_List.Tail.Previous;
      end if;
      Temp := In_The_List.Tail;
      if In_The_List.Head = In_The_List.Tail then
	In_The_List := List'(Head => null, Current => null, Tail => null);
      else
	In_The_List.Tail      := In_The_List.Tail.Previous;
	In_The_List.Tail.Next := null;
      end if;
      Deallocate (Temp);
    end if;
  end Remove_Tail;
  ----------------------------------------------------------------------------
  procedure Remove_Current  (In_The_List : in out List) is
    Temp : Pointer;
  begin
    if In_The_List.Current /= null then
      Temp := In_The_List.Current;
      if In_The_List.Current /= In_The_List.Tail then
	In_The_List.Current          := In_The_List.Current.Next;
	In_The_List.Current.Previous := Temp.Previous;
	if Temp /= In_The_List.Head then
	  In_The_List.Current.Previous.Next := In_The_List.Current;
	else
	  In_The_List.Head := In_The_List.Current;
	end if;
      elsif In_The_List.Current /= In_The_List.Head then
	In_The_List.Current   := In_The_List.Current.Previous;
	In_The_List.Tail      := In_The_List.Current;
	In_The_List.Tail.Next := null;
      else
	In_The_List := List'(Head => null, Current => null, Tail => null);
      end if;
      deallocate (temp);
    end if;
  end remove_current;
  ----------------------------------------------------------------------------
  procedure remove_previous (in_the_list : in out list) is
    temp : pointer;
  begin
    if in_the_list.current /= in_the_list.head then
      temp := in_the_list.current.previous;
      if temp /= in_the_list.head then
	temp.previous.next := in_the_list.current;
	in_the_list.current.previous := temp.previous;
      else
	In_The_List.Head          := In_The_List.Current;
	In_The_List.Head.Previous := null;
      end if;
      Deallocate (Temp);
    end if;
  end Remove_Previous;
  ----------------------------------------------------------------------------
  procedure Copy  (The_List      : in     List;
		   To_The_List   : in out List) is

    Old_Current  : Pointer := The_List.Head;
    Old_Tail     : Pointer renames The_List.Tail;
    List_Current : Pointer renames The_List.Current;
    New_Tail     : Pointer renames To_The_List.Tail;
  begin
    Deallocate (To_The_List);
    if The_List.Head /= null then
      To_The_List.Head := new Cell'(The_Item   => Copy (Old_Current.The_Item),
				    Previous   => null,
				    Next       => null);
      To_The_List.Current := To_The_List.Head;
      To_The_List.Tail    := To_The_List.Head;
      Old_Current         := Old_Current.Next;
      while Old_Current /= null loop
	New_Tail.Next := new Cell'(The_Item   => Copy (Old_Current.The_Item),
				   Previous   => New_Tail,
				   Next       => null);
	New_Tail := New_Tail.Next;
	if Old_Current = List_Current then
	  To_The_List.Current := New_Tail;
	end if;
	Old_Current := Old_Current.Next;
      end loop;
    end if;
  exception
    when Storage_Error => raise Overflow;
  end Copy;
  ----------------------------------------------------------------------------
  procedure Deallocate (The_List : in out List) is
    Head    : Pointer := The_List.Head;
    Current : Pointer;
    Tail    : Pointer renames The_List.Tail;
  begin
    while Head /= Tail loop
      Current := Head;
      Head    := Head.Next;
      Deallocate (Current);
    end loop;
    Deallocate (Tail);
    The_List.Head    := null;
    The_List.Current := null;
  end Deallocate;
  ----------------------------------------------------------------------------
  procedure Set_Head    (Of_The_List : in out List;
			 To_The_Item : in     Item) is
  begin
    Of_The_List.Head.The_Item := To_The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Set_Head;
  ----------------------------------------------------------------------------
  procedure Set_Current (Of_The_List : in out List;
			 To_The_Item : in     Item) is
  begin
    Of_The_List.Current.The_Item := To_The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Set_Current;
  ----------------------------------------------------------------------------
  procedure Set_Tail    (Of_The_List : in out List;
			 To_The_Item : in     Item) is
  begin
    Of_The_List.Tail.The_Item := To_The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Set_Tail;
  ----------------------------------------------------------------------------
--  procedure swap_tail   (of_the_list  : in out list;
--                         and_the_list : in out list);
  ----------------------------------------------------------------------------
  procedure Iterate_Foreward    (In_The_List : in out List) is
  begin
    In_The_List.Current := In_The_List.Current.Next;
  exception
    when Constraint_Error => raise Position_Error;
  end Iterate_Foreward;
  ----------------------------------------------------------------------------
  procedure Iterate_Backward    (In_The_List : in out List) is
  begin
    In_The_List.Current := In_The_List.Current.Previous;
  exception
    when Constraint_Error => raise Position_Error;
  end Iterate_Backward;
  ----------------------------------------------------------------------------
  procedure Set_Current_To_Head (In_The_List : in out List) is
  begin
    In_The_List.Current := In_The_List.Head;
  end Set_Current_To_Head;
  ----------------------------------------------------------------------------
  procedure Set_Current_To_Tail (In_The_List : in out List) is
  begin
    In_The_List.Current := In_The_List.Tail;
  end Set_Current_To_Tail;
  ----------------------------------------------------------------------------
  procedure Set_Current_To      (Position    : in     Index;
				 In_The_List : in out List) is
    current : pointer := in_the_list.head;
    counter : index := 1;
  begin
    if position = 0 then
      return;
    end if;
    while counter < position loop
      current := current.next;
      counter := counter + 1;
    end loop;
    in_the_list.current := current;
  exception
    when constraint_error => raise position_error;
  end Set_Current_To;
  ----------------------------------------------------------------------------
  function Is_Equal     (Left     : in List;
			 Right    : in List) return Boolean is

    Left_Current  : Pointer := Left.Current;
    Right_Current : Pointer := Right.Current;
    Left_Tail     : Pointer renames Left.Tail;
    Right_Tail    : Pointer renames Right.Tail;
  begin
    if Left_Current = null xor Right_Current = null then
      return False;
    end if;
    loop
      if not Is_Equal (Left_Current.The_Item, Right_Current.The_Item) then
	return False;
      end if;
      exit when Left_Current = Left_Tail or Right_Current = Right_Tail;
      Left_Current  := Left_Current.Next;
      Right_Current := Right_Current.Next;
    end loop;
    return Left_Current = Left_Tail and Right_Current = Right_Tail;
  end Is_Equal;
  ----------------------------------------------------------------------------
  function Is_Shared    (Left     : in List;
			 Right    : in List) return Boolean is
    Left_Current  : Pointer;
    Right_Current : Pointer := Right.Head;
    Left_Tail     : Pointer renames Left.Tail;
    Right_Tail    : Pointer renames Right.Tail;
  begin
    if Right_Current = null or Left.Current = null then
      return False;
    end if;
    loop
      Left_Current := Left.Head;
      loop
	if Left_Current = Right_Current then
	  return True;
	end if;
	exit when Left_Current = Left_Tail;
	Left_Current := Left_Current.Next;
      end loop;
      exit when Right_Current = Right_Tail;
      Right_Current := Right_Current.Next;
    end loop;
    return False;
  end Is_Shared;
  ----------------------------------------------------------------------------
  function Is_Identical (Left     : in List;
			 Right    : in List) return Boolean is
    Left_Current  : Pointer := Left.Head;
    Right_Current : Pointer := Right.Head;
    Left_Tail     : Pointer renames Left.Tail;
    Right_Tail    : Pointer renames Right.Tail;
  begin
    if Left_Current /= Right_Current or Left_Tail /= Right_Tail then
      return False;
    end if;
    loop
      if Left_Current /= Right_Current or
	 (Left_Current = Left_Tail xor Right_Current = Right_Tail) then
	return False;
      end if;
      Left_Current  := Left_Current.Next;
      Right_Current := Right_Current.Next;
    end loop;
    return True;
  end Is_Identical;
  ----------------------------------------------------------------------------
  function Length_Of    (The_List : in List) return Index is
    Current : Pointer := The_List.Head;
    Counter : Index;
    Tail    : Pointer renames The_List.Tail;
  begin
    if Current = null then
      return 0;
    end if;
    counter := 1;
    while current /= tail loop
      counter := counter + 1;
      current := current.next;
    end loop;
    return counter;
  end length_of;
  ----------------------------------------------------------------------------
  function position_of  (the_list : in list) return index is
    current      : pointer := the_list.head;
    list_current : pointer renames the_list.current;
    counter      : index;
  begin
    if Current = null then
      return 0;
    end if;
    Counter := 1;
    while Current /= List_Current loop
      Counter := Counter + 1;
      Current := Current.Next;
    end loop;
    return Counter;
  end Position_Of;
  ----------------------------------------------------------------------------
  function Is_Null      (The_List : in List) return Boolean is
  begin
    return The_List.Head = null;
  end Is_Null;
  ----------------------------------------------------------------------------
  function Is_At_Head  (The_List : in List) return Boolean is
  begin
    return The_List.Head = The_List.Current;
  end Is_At_Head;
  ----------------------------------------------------------------------------
  function Is_At_Tail  (The_List : in List) return Boolean is
  begin
    return The_List.Tail = The_List.Current;
  end Is_At_Tail;
  ----------------------------------------------------------------------------
  function Head_Of      (The_List : in List) return Item is
  begin
    return The_List.Head.The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Head_Of;
  ----------------------------------------------------------------------------
  function Tail_Of      (The_List : in List) return Item is
  begin
    return The_List.Tail.The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Tail_Of;
  ----------------------------------------------------------------------------
  function Current_Of   (The_List : in List) return Item is
  begin
    return The_List.Current.The_Item;
  exception
    when Constraint_Error => raise List_Is_Null;
  end Current_Of;
  ----------------------------------------------------------------------------
  function Give_Item (With_Index : in Index;
		      From_List  : in List) return Item is
    Temp    : Pointer := From_List.Head;
    Counter : Index;
  begin
    if Temp = null then
      raise List_Is_Null;
    end if;
    Counter := 1;
    while Counter < With_Index loop
      Temp    := Temp.Next;
      Counter := Counter + 1;
    end loop;
    return Temp.The_Item;
  exception
    when Constraint_Error => raise Position_Error;
  end Give_Item;
  ----------------------------------------------------------------------------
  function Give_Index (Of_Item : in Item;
		       In_List : in List) return Index is
    Temp    : Pointer := In_List.Head;
    Counter : Index;
  begin
    if Temp = null then
      return 0;
    end if;
    Counter := 1;
    while Temp.The_Item /= Of_Item loop
      Temp    := Temp.Next;
      Counter := Counter + 1;
    end loop;
    return Counter;
  exception
    when Constraint_Error => return 0;
  end Give_Index;
  ----------------------------------------------------------------------------
end Utilities.Generics.Doubly_Linked_List;