--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-doubly_linked_list.ads
-- Description     : Doubly linked list
-- Author          : J. Schrer
-- Created On      : Wed Jul  2 13:43:18 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  2 14:49:28 1997
-- Update Count    : 2
-- Status          :

generic
  type Item  is private;
  type Index is range <>;

  with function Is_Equal (Left, Right : in Item) return Boolean is <>;
  with function Copy     (The_Item    : in Item) return Item    is <>;
  with procedure Deallocate (The_Item : in out Item) is <>;

package Utilities.Generics.Doubly_Linked_List is

  type List is private;
--
  procedure Append      (The_Item     : in     Item;
			 To_The_List  : in out List);

  procedure Append      (The_List     : in out List;
			 To_The_List  : in out List);
--
  procedure Insert      (The_Item     : in     Item;
			 In_The_List  : in out List);

  procedure Insert      (The_List     : in out List;
			 In_The_List  : in out List);
--
  procedure Add_On_Head (The_Item     : in     Item;
			 To_The_List  : in out List);

  procedure Add_On_Head (The_List     : in out List;
			 To_The_List  : in out List);
--
  procedure Remove_Head     (In_The_List : in out List);

  procedure Remove_Tail     (In_The_List : in out List);

  procedure Remove_Current  (In_The_List : in out List);

  procedure Remove_Previous (In_The_List : in out List);
--
  procedure Copy  (The_List      : in     List;
		   To_The_List   : in out List);
--
  procedure Deallocate (The_List : in out List);
--
  procedure Set_Head    (Of_The_List : in out List;
			 To_The_Item : in     Item);

  procedure Set_Current (Of_The_List : in out List;
			 To_The_Item : in     Item);

  procedure Set_Tail    (Of_The_List : in out List;
			 To_The_Item : in     Item);
--
--  procedure swap_tail   (of_the_list  : in out list;
--                         and_the_list : in out list);
--
  procedure Iterate_Foreward    (In_The_List : in out List);

  procedure Iterate_Backward    (In_The_List : in out List);
--
  procedure Set_Current_To_Head (In_The_List : in out List);

  procedure Set_Current_To_Tail (In_The_List : in out List);

  procedure Set_Current_To      (Position    : in     Index;
                                 In_The_List : in out List);
--
  function Is_Equal     (Left     : in List;
			 Right    : in List) return Boolean;

  function Is_Shared    (Left     : in List;
			 Right    : in List) return Boolean;

  function Is_Identical (Left     : in List;
			 Right    : in List) return Boolean;

  function Length_Of   (The_List : in List) return Index;

  function Position_Of (The_List : in List) return Index;

  function Is_Null     (The_List : in List) return Boolean;

  function Is_At_Head  (The_List : in List) return Boolean;

  function Is_At_Tail  (The_List : in List) return Boolean;
--
  function Head_Of     (The_List : in List) return Item;

  function Tail_Of     (The_List : in List) return Item;

  function Current_Of  (The_List : in List) return Item;
--
  function Give_Item   (With_Index : in Index;
                        From_List  : in List) return Item;

  function Give_Index  (Of_Item    : in Item;
                        In_List    : in List) return Index;
--
  Overflow              : exception;
  List_Is_Null          : exception;
  Position_Error        : exception;

private

  type Cell;
  type Pointer is access Cell;

  type List is
    record
      Head     : Pointer;
      Current  : Pointer;
      Tail     : Pointer;
    end record;

end Utilities.Generics.Doubly_Linked_List;