--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-float_utilities.adb
-- Description     : Floating point utilities, taken from Grady Booch
-- Author          : J. Schroeer
-- Created On      : Wed Jul  2 13:00:46 1997
-- Last Modified By:
-- Last Modified On: Mon Jul  7 18:17:47 1997
-- Update Count    : 2
-- Status          : Unknown, Use with caution!

with Utilities.String_Utilities;

package body Utilities.Generics.Float_Utilities is

  Log2_10 : constant Number := 3.321928;

  Max_String_Length : Integer := Integer(Number(Number'digits) * Log2_10) + 12;
  -----------------------------------------------------------------------------
  function Integer_Part (The_Number : in Number) return Integer_Type is
  begin
    if Is_Negative(The_Number) then
      return Ceiling(The_Number);
    else
      return Floor(The_Number);
    end if;
  end Integer_Part;
  -----------------------------------------------------------------------------
  function Real_Part (The_Number : in Number) return Number is
  begin
    return abs(The_Number - Number(Integer_Part(The_Number)));
  end Real_Part;
  -----------------------------------------------------------------------------
  function Floor (The_Number : in Number) return Integer_Type is
    Result : Integer_Type;
  begin
    Result := Integer_Type(The_Number);
    if Number(Result) > The_Number then
      return (Result - 1);
    else
      return Result;
    end if;
  exception
    when Constraint_Error => raise Lexical_Error;
  end Floor;
  -----------------------------------------------------------------------------
  function Ceiling (The_Number : in Number) return Integer_Type is
    Result : Integer_Type;
  begin
    Result := Integer_Type(The_Number);
    if Number(Result) < The_Number then
      return (Result + 1);
    else
      return Result;
    end if;
  exception
    when Constraint_Error => raise Lexical_Error;
  end Ceiling;
  -----------------------------------------------------------------------------
  function Min (Left  : in Number;
                Right : in Number) return Number is
  begin
    if Left > Right then
      return Right;
    else
      return Left;
    end if;
  end Min;
  -----------------------------------------------------------------------------
  function Min (The_Numbers : in Numbers) return Number is
    Smallest_Number : Number := The_Numbers(The_Numbers'First);
  begin
    for I in The_Numbers'First + 1 .. The_Numbers'Last loop
      if The_Numbers(I) < Smallest_Number then
        Smallest_Number := The_Numbers(I);
      end if;
    end loop;
    return Smallest_Number;
  end Min;
  -----------------------------------------------------------------------------
  function Max (Left : in Number;
                Right : in Number) return Number is
  begin
    if Left > Right then
      return Left;
    else
      return Right;
    end if;
  end Max;
  -----------------------------------------------------------------------------
  function Max (The_Numbers : in Numbers) return Number is
    Greatest_Number : Number := The_Numbers(The_Numbers'First);
  begin
    for I in The_Numbers'First + 1 .. The_Numbers'Last loop
      if The_Numbers(I) > Greatest_Number then
        Greatest_Number := The_Numbers(I);
      end if;
    end loop;
    return Greatest_Number;
  end Max;
  -----------------------------------------------------------------------------
  function Is_Positive (The_Number : in Number) return Boolean is
  begin
    return (The_Number > 0.0);
  end Is_Positive;
  -----------------------------------------------------------------------------
  function Is_Natural (The_Number : in Number) return Boolean is
  begin
    return (The_Number >= 0.0);
  end Is_Natural;
  -----------------------------------------------------------------------------
  function Is_Negative (The_Number : in Number) return Boolean is
  begin
    return (The_Number < 0.0);
  end Is_Negative;
  -----------------------------------------------------------------------------
  function Is_Zero (The_Number : in Number) return Boolean is
  begin
    return (abs(The_Number) < Number'Model_Small);
  end Is_Zero;
  -----------------------------------------------------------------------------
  function Is_Equal (Left  : in Number;
                     Right : in Number) return Boolean is
  begin
    return (abs(Left - Right) < Number'Model_Small);
  end Is_Equal;
  -----------------------------------------------------------------------------
  function Image_Of     (The_Number    : in Number;
                         With_Aft      : in Ada.Text_Io.Field := Default_Aft;
                         With_Exp      : in Ada.Text_Io.Field := Default_Exp)
                         return String is
    Str : String(1 .. Max_String_Length);
  begin
    Float_Text_Io.Put(To   => Str,
                      Item => The_Number,
                      Aft  => Ada.Text_Io.Field(With_Aft),
                      Exp  => Ada.Text_Io.Field(With_Exp) );
    return Utilities.String_Utilities.Stripped_Leading(Str);
  exception
    when Ada.Text_Io.Data_Error => raise Lexical_Error;
  end Image_Of;
  -----------------------------------------------------------------------------
  function Value_Of     (The_Image     : in String) return Number is
    Numb : Number;
    Posi : Positive;
  begin
    Float_Text_Io.Get(From => The_Image,
                      Item => Numb,
                      Last => Posi);
    return Numb;
  exception
    when Ada.Text_Io.Data_Error => raise Lexical_Error;
  end Value_Of;
  -----------------------------------------------------------------------------
end Utilities.Generics.Float_Utilities;
