--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-float_utilities.ads
-- Description     : Floating point utilities, taken from Grady Booch
-- Author          : J. Schroeer
-- Created On      : Wed Jul  2 13:04:06 1997
-- Last Modified By: 
-- Last Modified On: Wed Jul  2 13:04:53 1997
-- Update Count    : 1
-- Status          :

with Ada.Text_Io;

generic
  type Number       is digits <>;
  type Integer_Type is range <>;
  type Index        is range <>;
  type Numbers      is array (Index range <>) of Number;

  with package Float_Text_Io is new Ada.Text_Io.Float_Io (Num => Number);

  Default_Aft  : Ada.Text_Io.Field := Float_Text_Io.Default_Aft;
  Default_Exp  : Ada.Text_Io.Field := Float_Text_Io.Default_Aft;


package Utilities.Generics.Float_Utilities is

  type Base is range 2 .. 16;

  function Integer_Part (The_Number    : in Number)     return Integer_Type;

  function Real_Part    (The_Number    : in Number)     return Number;

  function Floor        (The_Number    : in Number)     return Integer_Type;

  function Ceiling      (The_Number    : in Number)     return Integer_Type;

  function Min          (Left          : in Number;
                         Right         : in Number)     return Number;

  function Min          (The_Numbers   : in Numbers)    return Number;

  function Max          (Left          :  in Number;
                         Right         :  in Number)    return Number;

  function Max          (The_Numbers   : in Numbers)    return Number;

  function Is_Positive  (The_Number    : in Number)     return Boolean;

  function Is_Natural   (The_Number    : in Number)     return Boolean;

  function Is_Negative  (The_Number    : in Number)     return Boolean;

  function Is_Zero      (The_Number    : in Number)     return Boolean;

  function Image_Of     (The_Number    : in Number;
                         With_Aft      : in Ada.Text_Io.Field := Default_Aft;
                         With_Exp      : in Ada.Text_Io.Field := Default_Exp)
                         return String;

  function Value_Of     (The_Image     : in String) return Number;

  function Is_Equal     (Left          : in Number;
                         Right         : in Number)     return Boolean;

  Lexical_Error : exception;

end Utilities.Generics.Float_Utilities;

