--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-integer_utilities.adb
-- Description     : Integer utilities taken from Grady Booch
-- Author          : J. Schroeer
-- Created On      : Wed Jul  2 13:08:36 1997
-- Last Modified By:
-- Last Modified On: Mon Jul  7 18:21:21 1997
-- Update Count    : 4
-- Status          :

with Utilities.String_Utilities;

package body Utilities.Generics.Integer_Utilities is

  Log2_10 : constant Float := 3.321928;

  Max_String_Length : constant Natural :=
                      Natural(Float(Number'Width) * Log2_10) + 10;
  -----------------------------------------------------------------------------
  function Min (Left  : in Number;
                Right : in Number) return Number is
  begin
    if Left > Right then
      return Right;
    else
      return Left;
    end if;
  end Min;
  -----------------------------------------------------------------------------
  function Min (The_Numbers : in Numbers) return Number is
    Smallest_Number : Number := The_Numbers(The_Numbers'First);
  begin
    for I in The_Numbers'First + 1 .. The_Numbers'Last loop
      if The_Numbers(I) < Smallest_Number then
        Smallest_Number := The_Numbers(I);
      end if;
    end loop;
    return Smallest_Number;
  end Min;
  -----------------------------------------------------------------------------
  function Max (Left : in Number;
                Right : in Number) return Number is
  begin
    if Left > Right then
      return Left;
    else
      return Right;
    end if;
  end Max;
  -----------------------------------------------------------------------------
  function Max (The_Numbers : in Numbers) return Number is
    Greatest_Number : Number := The_Numbers(The_Numbers'First);
  begin
    for I in The_Numbers'First + 1 .. The_Numbers'Last loop
      if The_Numbers(I) > Greatest_Number then
        Greatest_Number := The_Numbers(I);
      end if;
    end loop;
    return Greatest_Number;
  end Max;
  -----------------------------------------------------------------------------
  function Is_Positive (The_Number : in Number) return Boolean is
  begin
    return (The_Number > 0) ;
  end Is_Positive;
  -----------------------------------------------------------------------------
  function Is_Natural (The_Number : in Number) return Boolean is
  begin
    return (The_Number >= 0);
  end Is_Natural;
  -----------------------------------------------------------------------------
  function Is_Negative (The_Number : in Number) return Boolean is
  begin
    return (The_Number < 0);
  end Is_Negative;
  -----------------------------------------------------------------------------
  function Is_Zero (The_Number : in Number) return Boolean is
  begin
    return (The_Number = 0);
  end Is_Zero;
  -----------------------------------------------------------------------------
  function Is_Odd (The_Number : in Number) return Boolean is
  begin
    return (The_Number mod 2 /= 0);
  end Is_Odd;
  -----------------------------------------------------------------------------
  function Is_Even (The_Number : in Number) return Boolean is
  begin
    return (The_Number mod 2 = 0);
  end Is_Even;
  -----------------------------------------------------------------------------
  function Image_Of (The_Number    : in Number;
                     With_The_Base : in Base := 10)  return String is
    Str : String(1 .. Max_String_Length);
  begin
    Integer_Text_Io.Put(To   => Str,
                        Item => The_Number);
    return Utilities.String_Utilities.Stripped_Leading(Str);
  exception
    when Ada.Text_Io.Data_Error => raise Lexical_Error;
  end Image_Of;
  -----------------------------------------------------------------------------
  function Value_Of (The_Image     : in String;
                     With_The_Base : in Base := 10) return Number is
    Numb : Number;
    Posi : Positive;
  begin
    Integer_Text_Io.Get(From => The_Image,
                        Item => Numb,
                        Last => Posi);
    return Numb;
  exception
    when Ada.Text_Io.Data_Error => raise Lexical_Error;
  end Value_Of;
  -----------------------------------------------------------------------------
end Utilities.Generics.Integer_Utilities;
