--                              -*- Mode: Ada -*-
-- Filename        : utilities-generics-sorting_and_searching.adb
-- Description     : Generic quicksort and generic binary search
-- Author          : J. Schroeer
-- Created On      : Wed Jul  2 13:15:34 1997
-- Last Modified By: 
-- Last Modified On: Wed Jul  2 13:19:37 1997
-- Update Count    : 1
-- Status          :

package body Utilities.Generics.Sorting_And_Searching is
  -----------------------------------------------------------------------------
  -- see for example :
  -- N. Wirth : Algorithmen und Datenstrukturen , Page 99
  -- 3. Auflage, Teubner Stuttgart, 1983

  procedure Quicksort (The_Vector : in out Vector) is 
    ----------------------------------
    procedure Sort (L, R : in Integer) is 
      I :          Integer := L;  
      J :          Integer := R;  
      W :          Element;  
      X : constant Element := The_Vector (Index (L + R) / 2);  
    begin
      while I <= J loop
        while The_Vector (Index (I)) < X loop
          I := I + 1;
        end loop;
        while X < The_Vector (Index (J)) loop
          J := J - 1;
        end loop;
        if I <= J then
          W := The_Vector (Index (I));
          The_Vector (Index (I)) := The_Vector (Index (J));
          The_Vector (Index (J)) := W;
          I := I + 1;
          J := J - 1;
        end if;
      end loop;
      if L < J then
        Sort (L, J);
      end if;
      if I < R then
        Sort (I, R);
      end if;
    end Sort;
    ---------
  begin
    if The_Vector'Length > 0 then
      Sort (Integer (The_Vector'First), Integer (The_Vector'Last));
    end if;
  end Quicksort;
  -----------------------------------------------------------------------------
  function Binary_Search (The_Element : in Element; 
                          The_Vector  : in Vector) return Index is 
    I, J, K : Integer;  
  begin
    if    The_Element < The_Vector (The_Vector'First) then
      return The_Vector'First - 1;
    elsif The_Vector (The_Vector'Last) < The_Element then
      return The_Vector'Last + 1;
    elsif The_Element = The_Vector (The_Vector'Last) then
      return The_Vector'Last;
    else
      I := Integer (The_Vector'First);
      J := Integer (The_Vector'Last);
      loop
        K := (I + J) / 2;
        exit when The_Vector (Index (K)) <= The_Element and then
                  The_Element < The_Vector (Index (K) + 1);
        if The_Element <= The_Vector (Index (K)) then
          J := K - 1;
        else
          I := K + 1;
        end if;
      end loop;
      return Index (K);
    end if;
  end Binary_Search;
  -----------------------------------------------------------------------------
end Utilities.Generics.Sorting_And_Searching;
