--                              -*- Mode: Ada -*-
-- Filename        : utilities-prime_numbers.adb
-- Description     : Prime number algorithm and container
-- Author          : J. Schroeer
-- Created On      : Wed Jul  2 13:13:42 1997
-- Last Modified By:
-- Last Modified On: Thu Jul  3 16:38:50 1997
-- Update Count    : 11
-- Status          :

with Portable.Standard, Math.Integer_32.Arrays.Types,
     Utilities.Integer_32.Sorting_And_Searching;

package body Utilities.Prime_Numbers is

  Prime_Vector      : Math.Integer_32.Arrays.Types.Vector_Access;

  Last_Keys         : String (1 .. 3) := (others => ' ');
  Table_Length_Last,
  Number_Last,
  Index_Last        : Natural;
  -----------------------------------------------------------------------------
  procedure Calculate_Primes (Up_To_Number : in Positive) is

    Temp_Array : Math.Integer_32.Arrays.Types.Vector(2 .. Up_To_Number) :=
                (others => 1);
    N : Positive := 3;
    I : Positive := 4;
    use type Portable.Standard.Integer_32;
  begin
    while I <= Up_To_Number loop
      Temp_Array(I) := 0;
      I := I + 2;
    end loop;
    while N <= Up_To_Number / 2 loop
      if Temp_Array(N) = 1 then
        I := N * 2;
        while I <= Up_To_Number loop
          Temp_Array(I) := 0;
          I := I + N;
        end loop;
      end if;
      N := N + 2;
    end loop;

    N := 2;
    I := 2;
    while I <= Up_To_Number loop
      if Temp_Array(I) = 1 then
        Temp_Array(N) := Portable.Standard.Integer_32(I);
        N := N + 1;
      end if;
      I := I + 1;
    end loop;

    Math.Integer_32.Arrays.Types.Deallocate(Prime_Vector);
    Prime_Vector := new Math.Integer_32.Arrays.Types.Vector'
                       (Temp_Array(2 .. N - 1));
  end Calculate_Primes;
  -----------------------------------------------------------------------------
  procedure Deallocate_Primes is
  begin
    Math.Integer_32.Arrays.Types.Deallocate(Prime_Vector);
  end Deallocate_Primes;
  -----------------------------------------------------------------------------
  function Give_Next_Prime
          (Near       : in Positive;
           Difference : in Difference_Type := Greater) return Positive is

    Pos : Positive;
    use type Math.Integer_32.Arrays.Types.Vector_Access;
    use type Portable.Standard.Integer_32;
  begin
    if Prime_Vector = null or else
       Integer(Prime_Vector(Prime_Vector'Last)) < Integer(Near) then
      Calculate_Primes(Up_To_Number => Near + 100);
    end if;

    Pos := Utilities.Integer_32.Sorting_And_Searching.Binary_Search
                   (The_Element => Portable.Standard.Integer_32(Near),
                    The_Vector  => Prime_Vector.all);

    if Pos < Prime_Vector'First then
      Pos := Prime_Vector'First;
    end if;
    if Difference = Greater then
      Pos := Pos + 1;
    end if;
    if Pos > Prime_Vector'Last then
      Pos := Prime_Vector'Last;
    end if;

    return Positive(Prime_Vector(Pos));
  end Give_Next_Prime;
  -----------------------------------------------------------------------------
  function Standard_Hash_Function
          (Key          : in String;
           Table_Length : in Natural) return Natural is
             
    Keys : String (1 .. 3);
  begin
    if Key'Length = 0 then
      return 0;
    end if;  
    Keys := (1 => Key (Key'First),
             2 => Key (Key'First + Key'Length / 2),
             3 => Key (Key'Last));
    if Keys = Last_Keys then
      if Table_Length_Last /= Table_Length then
        Table_Length_Last := Table_Length;
        Index_Last        := Number_Last rem Table_Length;
      end if;
    else
      Last_Keys         := Keys;
      Table_Length_Last := Table_Length;
      -- numbers 43, 41, 67 give good results for hash-tables of length 997
      Number_Last       :=  Character'Pos (Keys(1)) * 43 +
                            Character'Pos (Keys(2)) * 41 +
                            Character'Pos (Keys(3)) * 67;
      Index_Last        :=  Number_Last rem Table_Length;
    end if;
    return Index_Last;
  end Standard_Hash_Function;
  -----------------------------------------------------------------------------
  function Standard_Hash_Function
          (Key          : in String_Utilities.String_Access;
           Table_Length : in Natural) return Natural is
  begin
    return Standard_Hash_Function(Key.all, Table_Length);
  end Standard_Hash_Function;
  -----------------------------------------------------------------------------
  function Standard_Hash_Function
          (Key          : in Ada.Strings.Unbounded.String_Access;
           Table_Length : in Natural) return Natural is
  begin
    return Standard_Hash_Function(Key.all, Table_Length);
  end Standard_Hash_Function;
  -----------------------------------------------------------------------------
begin
  Calculate_Primes(Up_To_Number => 3000);
end Utilities.Prime_Numbers;

--  function standard_hash_function
--          (key          : in string;
--           table_length : in natural) return natural is
--
--    first : positive := character'pos (key (key'first)) - 31;
--    mid   : positive := character'pos (key (key'first + key'length / 2)) - 31;
--    last  : positive := character'pos (key (key'last)) - 31;
--  begin
--    -- numbers 43, 41, 67 give good results for hash-tables of length 997
--    return (character'pos (key (key'first))                  * 43 +
--            character'pos (key (key'first + key'length / 2)) * 41 +
--            character'pos (key (key'last))                   * 67)
--           rem table_length;
--    return (((first * 11 + mid) * 7 + last) * 3) rem table_length;
--    return (first * 43 + mid * 41 + last * 67) rem table_length;
--    return (first * 11 + mid * 13 + last * 17) rem table_length;
--    return (first * prime_vector(mid   rem 6 + 3) +
--            mid   * prime_vector(last  rem 7 + 3) +
--            last  * prime_vector(first rem 8 + 3)) rem table_length;
--  end standard_hash_function;
