--                              -*- Mode: Ada -*-
-- Filename        : utilities-prime_numbers.ads
-- Description     : Prime number algorithm and container
-- Author          : J. Schrer
-- Created On      : Wed Jul  2 13:24:40 1997
-- Last Modified By: J. Schrer
-- Last Modified On: 4. dez. 1998, js, added Standard_Hash_Function for
--                                      ada.strings.unbounded.string_access
-- Update Count    : 
-- Status          :

with Ada.Strings.Unbounded;
with Utilities.String_Utilities;

package Utilities.Prime_Numbers is

  procedure Calculate_Primes (Up_To_Number : in Positive);
  
  -- Calculates all prime-numbers that are less or equal to Up_To_Number.

  procedure Deallocate_Primes;
  
  -- Frees the internal array of prime-numbers.

  type Difference_Type is (Smaller, Greater);

  function Give_Next_Prime
          (Near       : in Positive;
           Difference : in Difference_Type := Greater) return Positive;
           
  -- Returns the next prive smaller or greater to the number near.         

  -- The 3 Standard_Hash_Functions calculate the index of key in a 
  -- hash table array whose length is table_length.        
           
  function Standard_Hash_Function
          (Key          : in String;
           Table_Length : in Natural) return Natural;

  function Standard_Hash_Function
          (Key          : in String_Utilities.String_Access;
           Table_Length : in Natural) return Natural;
           
  function Standard_Hash_Function
          (Key          : in Ada.Strings.Unbounded.String_Access;
           Table_Length : in Natural) return Natural;

end Utilities.Prime_Numbers;
