--                              -*- Mode: Ada -*-
-- Filename        : utilities-scanner-get_symbol.adb
-- Description     : Get new symbol from input file
-- Author          : J. Schroeer
-- Created On      : Thu Jul  3 22:13:06 1997
-- Last Modified By:
-- Last Modified On: Mon Jul  7 18:52:04 1997
-- Update Count    : 8
-- Status          :

separate (Utilities.Scanner)

procedure Get_Symbol is
  Line_Number      : Natural      renames The_Source.Line_Number;
  Last_Sign        : Natural      renames The_Source.Last_Sign;
  Column_Number    : Natural      renames The_Source.Column_Number;
  C                : Character    renames The_Source.Last_Character;
                     -- last read character, global to all routines
  Dot_Saved        : Boolean      renames The_Source.Dot_Saved;
  Apostrophe_Saved : Boolean      renames The_Source.Apostrophe_Saved;
  Read_Blanks      : Natural      renames The_Source.Read_Blanks;
  Line_Length      : Natural      renames The_Source.Line_Length;
  Symbol_Counter   : Natural      renames The_Source.Symbol_Counter;
  Line             : Ada.Strings.Unbounded.String_Access renames The_Source.Line;

  Class            : Symbol_Class renames Current_Symbol.Class;
  Is_Correct       : Boolean      renames Current_Symbol.Is_Correct;
  Name             : String       renames Current_Symbol.Name;
  Length           : Natural      renames Current_Symbol.Length;
  Key              : Symbol       renames Current_Symbol.Key;
  ---------------------------------------------------------------------------
  procedure Skip_Line is
  begin
    Last_Sign        := Line_Length;
    Line_Number      := Line_Number + 1;
    Column_Number    := 0;
    Read_Blanks      := 0;
    Apostrophe_Saved := False;
    Dot_Saved        := False;
  end Skip_Line;
  ---------------------------------------------------------------------------
  procedure Get_Character is
    -- gives next character outoff source
  begin
    if Last_Sign < Line_Length then                        -- not end of line
      Last_Sign := Last_Sign + 1;
      C         := Line (Last_Sign);
      case C is
        when ' '      => Column_Number := Column_Number + 1;
                         Read_Blanks   := Read_Blanks   + 1;
        when Ascii.Ht => Column_Number := Column_Number + Tab_Length -
                                         (Read_Blanks rem Tab_Length);
                         Read_Blanks   := 0;
        when others   => Column_Number := Column_Number + 1;
                         Read_Blanks   := 0;
      end case;
    else                                                   -- read new line
      declare
        Terminator       : Character;
        File             : Ada.Text_Io.File_Type renames
                           Source_File(Source_Counter);
        -- components of internal_text
        Source_Type      : Source_Class renames The_Source.Source_Type;
        Last_Line_Length : Natural      renames The_Source.Last_Line_Length;
        Last_Line_Number : Natural      renames The_Source.Last_Line_Number;
        Last_Line        : Ada.Strings.Unbounded.String_Access renames
                           The_Source.Last_Line;
        Temp             : Ada.Strings.Unbounded.String_Access;
      begin
        case Source_Type is
          when Ascii_File =>
            Temp      := Last_Line;
            Last_Line := Line;
            Line      := Temp;

            Last_Line_Number := Line_Number - 1;
            Last_Line_Length := Line_Length;

            <<Read_Line>>

            Line_Length := 0;
            if Ada.Text_Io.End_Of_File(File) then
              Terminator := Eof;
            else
              begin
                Ada.Text_Io.Get_Line (File => File,
                                      Item => Line(1 .. Line'Last - 1),
                                      Last => Line_Length);
                Terminator := Eol;
              exception
                when Ada.Text_Io.End_Error =>
                  Terminator := Eof;
              end;
            end if;

            -- skip leading blanks and tabs of read line
            Last_Sign := 1;
            while Last_Sign <= Line_Length loop
              C := Line(Last_Sign);
              case C is
                when ' '      =>
                  Column_Number := Column_Number + 1;
                  Read_Blanks   := Read_Blanks   + 1;
                when Ascii.Ht =>
                  Column_Number := Column_Number + Tab_Length -
                                  (Read_Blanks rem Tab_Length);
                  Read_Blanks   := 0;
                when others   =>
                  Column_Number := Column_Number + 1;
                  Read_Blanks   := 0;
                  exit;
              end case;
              Last_Sign := Last_Sign + 1;
            end loop;

            -- c now is first non-blank character of meaningfull line
            -- all empty or comment-lines are skipped here, to avoid
            -- recursive calls of get_symbol for those cases
            if Last_Sign > Line_Length then -- empty line
              if Terminator = Eol then
                Skip_Line; -- eof found on empty line
                goto Read_Line; -->
              else -- terminator = eof found on empty line
                Line_Length       := Line_Length + 1;
                Line(Line_Length) := Terminator;
                C                 := Terminator;
              end if;
            elsif Last_Sign < Line_Length and
              Line(Last_Sign .. Last_Sign + 1) = "--" then -- comment line
              Skip_Line;
              goto Read_Line;   -->
            elsif Line_Length < Line'Last - 1 then
              -- normal action, line < 255 characters, end_of_line read
              while Line_Length > 0 and then Line(Line_Length) = ' ' loop
                Line_Length := Line_Length - 1;
              end loop;
              Line_Length       := Line_Length + 1;
              Line(Line_Length) := Terminator;
            else
              -- 255 characters read, but no line-terminator found,
              if Line (Line_Length) = ' ' then -- skip trailing ' '
                while Line_Length > 0 and then Line(Line_Length) = ' ' loop
                  Line_Length := Line_Length - 1;
                end loop;
                -- save 1 ' ' to separate identifiers of same source-line
                -- in subsequent read line.all objects
                Line_Length       := Line_Length + 1;
                Line(Line_Length) := ' ';
              end if;
              if Terminator = Eof then
                Line_Length       := Line_Length + 1;
                Line(Line_Length) := Terminator;
              end if;
            end if;
          when Normal_String =>
            Close;
            return;
        end case;
        Line_Message_Number := 0;
      end;
    end if;
  end Get_Character;
  ---------------------------------------------------------------------------
  procedure Test_For_Valid_Underline is
  begin
    if Length > 0 and then Name(Length) = '_' then
      Print_Message("**LEX  no valid underline, DIGIT or LETTER expected");
    end if;
  end Test_For_Valid_Underline;
  ---------------------------------------------------------------------------
  procedure Get_Number is
    -- tests string, to be an integer- or floating-point-number
    -- and converts it
    Value  : Float_64 renames Current_Symbol.Value;
    Number : Integer  renames Current_Symbol.Number;
    -------------------------------------------------------------------------
    procedure Get_Char is      -- adds one character in "number-string"
    begin
      Length := Length + 1;
      Name(Length) := C;
      Get_Character;
    end Get_Char;
    -------------------------------------------------------------------------
    procedure Get_Digits is    -- reads digits
    begin
      Get_Char;
      loop
        case C is
          when '0' .. '9' => Get_Char;
          when '_'        => Test_For_Valid_Underline;
                             Get_Char;
          when others     => exit;
        end case;
      end loop;
    end Get_Digits;
    -------------------------------------------------------------------------
    procedure Get_Hex_Digits is    -- reads digits
    begin
      Get_Char;
      loop
        case C is
          when '0' .. '9' |
               'a' .. 'f' |
               'A' .. 'F' => Get_Char;
          when '_'        => Test_For_Valid_Underline;
                             Get_Char;
          when others     => exit;
        end case;
      end loop;
    end Get_Hex_Digits;
    -------------------------------------------------------------------------
    procedure Get_Exp is             -- read exponent
    begin
      Get_Char;
      case C is
        when '-' | '+'  => Get_Char;
        when others     => null;
      end case;
      case C is
        when '0' .. '9' => Get_Digits;
        when others     => Print_Message("**LEX  Digit expected");
      end case;
    end Get_Exp;
  ---------------------------------------------------------------------------
  begin
    Get_Digits;                    -- integer or "fore" of float
    case C is
      when '.'            =>
        Get_Char;                  -- decimal-point of float
        case C is
          when '.'        => Length    := Length - 1; -- symbol ".."
                             Dot_Saved := True;
                             Class     := Integer_Number;
          when '0' .. '9' => Get_Digits; -- "aft" of float
                             case C is -- start of exponent of float
                               when 'e' | 'E' => Get_Exp;
                                                 Class := Float_Number;
                               when others    => Class := Float_Number;
                             end case;
         when others     => Print_Message("**LEX  Digit expected");
        end case;
      when 'e' | 'E'      =>
        Get_Exp;            -- start of exponent of integer
        Class := Integer_Number;
      when '#'       =>
        Get_Char;
        case C is
          when '0' .. '9' |
               'a' .. 'f' |
               'A' .. 'F' =>
            Get_Hex_Digits;
            case C is
              when '.'            =>
                Get_Char;                  -- decimal-point of float
                case C is
                  when '0' .. '9' |
                       'a' .. 'f' |
                       'A' .. 'F' =>
                    Class := Float_Number;
                    Get_Hex_Digits; -- "aft" of float
                    case C is
                      when  '#'   =>
                        Get_Char;
                        case C is
                          when 'e' | 'E' =>
                            Get_Exp;       -- start of exponent of float
                          when others    => null;
                        end case;
                      when others =>  Print_Message("**LEX  '#' expected");
                    end case;
                  when others =>
                    Print_Message("**LEX  (Hexadecimal) digit expected");
                end case;
              when '#'    =>
                Class := Integer_Number;
                Get_Char;
                case C is
                  when 'e' | 'E' =>
                    Get_Exp;       -- start of exponent of integer
                  when others    => null;
                end case;
              when others => Print_Message("**LEX  '#' expected");
            end case;
          when others =>
            Print_Message("**LEX  (Hexadecimal) digit expected");
        end case;
      when others    => Class := Integer_Number;
    end case;

    begin
      case Class is                     -- convert string to
        when Float_Number   => Value  := Float_64'Value(Name(1 .. Length));
        when Integer_Number => Number := Integer' Value(Name(1 .. Length));
        when others         => Print_Message
                              ("Unexpected error in get_number, class = " &
                               Symbol_Class'Image(Class));
      end case;
    exception -- exceptions can be raised during conversion
      when Ada.Text_Io.Data_Error =>
        Print_Message
        ("**LEX  no correct number-literal or number not in valid range");
      when Numeric_Error | Constraint_Error =>
        Print_Message ("**LEX  number not in valid range");
    end;
  end Get_Number;
  ---------------------------------------------------------------------------
  procedure Get_String is
    -- strings are enclosed with quotations '"'.
  begin
    loop
      Get_Character;
      exit when C = '"' or else C = Eol;
      Length := Length + 1;
      Name(Length) := C;
    end loop;
    if C /= '"' then
      Print_Message("**LEX  string must end on this line, " &
                    '"' & " expected");
      Skip_Line; -- Schroer, 24.2.1992
    end if;
    Get_Character;
    Class := Ada_String;
  end Get_String;
  ---------------------------------------------------------------------------
  procedure Get_Identifier is
    -- tests a legal identifier startin with 'a'..'z' or 'A'..'Z' to be
    -- a normal identifier,
    -- underlines "_" and digits are allowed as in ADA
  begin
    loop
      case C is
        when 'a' .. 'z' => Length := Length + 1;
                           Name(Length) := Character'Val
                                          (Character'Pos(C) - 32);
                           Get_Character;
        when 'A' .. 'Z' |
             '0' .. '9' => Length := Length + 1;
                           Name(Length) := C;
                           Get_Character;
        when '_'        => Test_For_Valid_Underline;
                           Length := Length + 1;
                           Name(Length) := C;
                           Get_Character;
        when others     => exit;
      end case;
    end loop;
    Test_For_Valid_Underline;
    Class := Identifier;
  end Get_Identifier;
  ---------------------------------------------------------------------------
begin

  <<Start>>

  Is_Correct := True;
  Length     := 0;
  while C = ' ' or else C = Ascii.Ht loop -- skip blanks and tabs
    Get_Character;
    Apostrophe_Saved := False;
  end loop;

  case C is
    when '0' .. '9' => Get_Number;
    when '-'        => Get_Character;
                       case C is
                         when '-'    =>                -- comment "--"
                           Skip_Line;
                           Get_Character;
--                             get_symbol;
                           Symbol_Counter := 0;        -- is inc. at end
                           goto Start;
                         when others => Class := Separator;
                                        Key   := '-';
                       end case;
    when 'a' .. 'z' |
         'A' .. 'Z' => Get_Identifier; -- letters
    when '"' => case Apostrophe_Saved is
                  when True  => Apostrophe_Saved := False;
                                Get_Character;
                                case C is
                                  when '''    => Class := Separator;
                                                 Key   := '"';
                                  when others => Get_String; -- "string"
                                end case;
                  when False => Get_String;
                end case;
    when Eol => Skip_Line;
                Get_Character;
--                  get_symbol;
                Symbol_Counter := 0;        -- is inc. at end
                goto Start;
    when '+' => Class := Separator;
                Key   := '+';
                Get_Character;
    when '*' => Get_Character;
                Class := Separator;
                case C is
                  when '*'    => Key := Exponentiate;
                                 Get_Character;
                  when others => Key := '*';
                end case;
    when '/' => Get_Character;
                Class := Separator;
                case C is
                  when '='    => Key := Not_Equal;
                                 Get_Character;
                  when others => Key := '/';
                end case;
    when ':' => Get_Character;
                Class := Separator;
                case C is
                  when '='    => Key := Becomes;
                                 Get_Character;
                  when others => Key := ':';
                end case;
    when '(' => Class := Separator;
                Key   := '(';
                Get_Character;
    when ')' => Class := Separator;
                Key   := ')';
                Get_Character;
    when ',' => Class := Separator;
                Key   := ',';
                Get_Character;
    when ';' => Class := Separator;
                Key   := ';';
                Get_Character;
    when '.' => Class := Separator;
                case Dot_Saved is
                  when True  => Dot_Saved := False;
                                Key       := Double_Dot;
                                Get_Character;
                  when False => Get_Character;
                                Class := Separator;
                                case C is
                                  when '.'    => Key := Double_Dot;
                                                 Get_Character;
                                  when others => Key := '.';
                                end case;
                end case;
    when '`' => Class := Separator;
                Key   := '`';
                Get_Character;
    when '=' => Get_Character;
                Class := Separator;
                case C is
                  when '>'    => Key := Arrow;
                                 Get_Character;
                  when others => Key := '=';
                end case;
    when '<' => Get_Character;
                Class := Separator;
                case C is
                  when '='    => Key := Less_Than_Or_Equal;
                                 Get_Character;
                  when others => Key := '<';
                end case;
    when '>' => Get_Character;
                Class := Separator;
                case C is
                  when '='    => Key := Greater_Than_Or_Equal;
                                 Get_Character;
                  when others => Key   := '>';
                end case;
    when ''' => Apostrophe_Saved := True;
                Class := Separator;
                Key   := ''';
                Get_Character;
    when '&' => Class := Separator;
                Key   := '&';
                Get_Character;
    when '|' => Class := Separator;
                Key   := '|';
                Get_Character;
    when Eof => Class := Separator;                    -- end of file
                Key   := End_Of_Source;
--                get_character;
    when '#' => Class := Separator;
                Key   := '#';
                Get_Character;
    when '_' => Class := Separator;
                Key   := '_';
                Get_Character;
    when '!' => Class := Separator;
                Key   := '!';
                Get_Character;
    when '$' => Class := Separator;
                Key   := '$';
                Get_Character;
    when '%' => Class := Separator;
                Key   := '%';
                Get_Character;
    when '?' => Class := Separator;
                Key   := '?';
                Get_Character;
    when '@' => Class := Separator;
                Key   := '@';
                Get_Character;
    when '[' => Class := Separator;
                Key   := '[';
                Get_Character;
    when ']' => Class := Separator;
                Key   := ']';
                Get_Character;
    when '^' => Class := Separator;
                Key   := '^';
                Get_Character;
    when '{' => Class := Separator;
                Key   := '{';
                Get_Character;
    when '}' => Class := Separator;
                Key   := '}';
                Get_Character;
    when '~' => Class := Separator;
                Key   := '~';
                Get_Character;
    when '\' => Class := Separator;
                Key   := '\';
                Get_Character;
    when Start_Char => Get_Character;
--                         get_symbol;
                       Symbol_Counter := 0;        -- is inc. at end
                       goto Start;
    when others     => Class := Separator;
                       Print_Message ("**LEX  unknown character");
  end case;

  Symbol_Counter := Symbol_Counter + 1;
end Get_Symbol;