--                              -*- Mode: Ada -*-
-- Filename        : utilities-scanner.ads
-- Description     : Hand written scanner for syntax with Ada lexical elements
-- Author          : J. Schroeer
-- Created On      : Thu Jul  3 20:33:13 1997
-- Last Modified By:
-- Last Modified On: Sun 28.1.2001, added switch in open.
-- Update Count    : 
-- Status          :

with Portable.Standard; use Portable.Standard;
with Ada.Text_Io;

package Utilities.Scanner is

  -- special symbols handled by scanner

  type Symbol is ('.', ':', '<', '>', '=', '(', ')', ',', '`',
                  ''', '&', '|', ';',
                  Not_Equal, Greater_Than_Or_Equal, Less_Than_Or_Equal,
                --  "/="             ">="                 "<="
                  Becomes, Arrow, Double_Dot,
                --  ":="    "=>"    ".."
                  Exponentiate, '+', '-', '*', '/', End_Of_Source, End_Of_Line,
                --    "**"                             "EOF"          "EOL"
                  '#', '_', '!', '$', '%', '?', '@', '[', ']', '^', '{', '}',
                  '~', '\', '"');

  function Image (Sym : in Symbol) return String;

  -- symbol-classes handled by scanner

  type Symbol_Class is (Identifier,
                        Ada_String,
                        Float_Number,
                        Integer_Number,
                        Separator);

--  type found_symbol (class : symbol_class := identifier) is
--    record
--      is_correct : boolean;
--      case class is
--        when identifier |
--             ada_string     => name   : string_utilities.string_access;
--        when float_number   => value  : float_64;
--        when integer_number => number : integer;
--        when separator      => key    : symbol;
--      end case;
--    end record;

  -- for efficiency reasons changed

  Max_Line_Length : constant Positive := 256;

  type Found_Symbol is record
    Class      : Symbol_Class;
    Is_Correct : Boolean;
    Name       : String (1 .. Max_Line_Length);
    Length     : Natural;
    Value      : Float_64;
    Number     : Integer;
    Key        : Symbol;
  end record;

  --***************************************************************************
  Current_Symbol : Found_Symbol;          -- global object for parsing-routines
  --***************************************************************************

  -- type and procedures to handle the intern file-stack ----------------------

  type Source_Class is (Ascii_File,
                        Normal_String);

  procedure Open (Filename : in String       := "";
                  Source   : in Source_Class := Ascii_File;
                  Listing  : in Boolean      := True);

  procedure Close;

  -- GET_SYMBOL updates global object CURRENT_SYMBOL --------------------------

  procedure Get_Symbol;

  -- central message-handling type, file and procedures -----------------------

  type Message_Type is (Error, Warning, Comment, Message);

  -- writes to session.listing_file

  procedure print_message (info           : in string;
                           treat_as       : in message_type := error;
                           line_increment : in natural := 1);

  -- functions for access of intern information -------------------------------

  function source_number return natural;

  function end_of_source return boolean;

  function source_type   (source_number : in natural := 0) return source_class;

  function line             (source_number : in natural := 0) return string;
  function file_name        (source_number : in natural := 0) return string;
  function line_number      (source_number : in natural := 0) return natural;
  function column_number    (source_number : in natural := 0) return natural;

  function last_line        (source_number : in natural := 0) return string;
  function last_line_number (source_number : in natural := 0) return natural;
  function symbol_counter   (source_number : in natural := 0) return natural;

  function listing_line_number return positive;
  ---------------------------------------------------------------------------
  name_error             : exception;
  status_error           : exception;
  too_many_sources_error : exception;
  no_open_file_error     : exception;
  ---------------------------------------------------------------------------
  listing_file : Ada.text_io.file_type;

  procedure save_line_number_difference;

  procedure save_current_line_number;

  function  current_line_number return positive;

end Utilities.scanner;



