--                              -*- Mode: Ada -*-
-- Filename        : utilities-string_utilities.adb
-- Description     : Taken from "Software components with Ada" by Grady Booch
--                   p.391
-- Author          : J. Schroeer
-- Created On      : Wed Mar 26 17:17:36 1997
-- Last Modified By:
-- Last Modified On: Wed Jul  9 20:03:00 1997
-- Update Count    : 6
-- Status          :

with Utilities.Character_Utilities, Ada.Unchecked_Deallocation;

package body Utilities.String_Utilities is

  package Character_Utilities renames Utilities.Character_Utilities;
  -----------------------------------------------------------------------------
  procedure Make_Uppercase (The_String         : in out String) is
  begin
    for I in The_String'range loop
      Character_Utilities.Make_Uppercase(The_String(I));
    end loop;
  end Make_Uppercase;
  -----------------------------------------------------------------------------
  procedure Make_Lowercase (The_String         : in out String) is
  begin
    for I in The_String'range loop
      Character_Utilities.Make_Lowercase(The_String(I));
    end loop;
  end Make_Lowercase;
  -----------------------------------------------------------------------------
  procedure Capitalize     (The_String         : in out String) is
  begin
    Character_Utilities.Make_Uppercase(The_String(The_String'First));
  end Capitalize;
  -----------------------------------------------------------------------------
  procedure Uncapitalize   (The_String         : in out String) is
  begin
    Character_Utilities.Make_Lowercase(The_String(The_String'First));
  end Uncapitalize;
  -----------------------------------------------------------------------------
  procedure Replace        (The_Character      : in Character;
                            With_The_Character : in Character;
                            In_The_String      : in out String;
                            Case_Sensitive     : in Boolean := True) is
    Local_Character          : Character;
    Local_With_The_Character : Character;
  begin
    if Case_Sensitive then
      for I in In_The_String'range loop
        if In_The_String(I) = The_Character then
          In_The_String(I) := With_The_Character;
        end if;
      end loop;
    else
      Local_Character := Character_Utilities.Uppercase(The_Character);
      Local_With_The_Character := Character_Utilities.
                                  Uppercase(With_The_Character);
      for I in In_The_String'range loop
        if Character_Utilities.Uppercase(In_The_String(I)) =
                               Local_Character then
          In_The_String(I) := Local_With_The_Character;
        end if;
      end loop;
    end if;
  end Replace;
  -----------------------------------------------------------------------------
  function Uppercase       (The_String         : in String) return String is
    Local_String : String (1 .. The_String'Length) := The_String;
  begin
    Make_Uppercase(Local_String);
    return Local_String;
  end Uppercase;
  -----------------------------------------------------------------------------
  function Lowercase       (The_String         : in String) return String is
    Local_String : String (1 .. The_String'Length) := The_String;
  begin
    Make_Lowercase(Local_String);
    return Local_String;
  end Lowercase;
  -----------------------------------------------------------------------------
  function Capitalized     (The_String         : in String) return String is
  begin
    return Character_Utilities.Uppercase(The_String(The_String'First)) &
           The_String(The_String'First + 1 .. The_String'Last);
  end Capitalized;
  -----------------------------------------------------------------------------
  function Uncapitalized   (The_String         : in String) return String is
  begin
    return Character_Utilities.Lowercase(The_String(The_String'First)) &
           The_String(The_String'First + 1 .. The_String'Last);
  end Uncapitalized;
  -----------------------------------------------------------------------------
  function Replaced        (The_Character      : in Character;
                            With_The_Character : in Character;
                            In_The_String      : in String;
                            Case_Sensitive     : in Boolean := True)
                                                              return String is
    Local_String : String (1 .. In_The_String'Length) := In_The_String;
  begin
    Replace(The_Character,With_The_Character,Local_String,Case_Sensitive);
    return Local_String;
  end Replaced;
  -----------------------------------------------------------------------------
  function Is_Null         (The_String         : in String) return Boolean is
  begin
    return The_String'Length = 0;
  end Is_Null;
  -----------------------------------------------------------------------------
  function Is_Control      (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if not Character_Utilities.Is_Control(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Control;
  -----------------------------------------------------------------------------
  function Is_Graphic      (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if not Character_Utilities.Is_Graphic(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Graphic;
  -----------------------------------------------------------------------------
  function Is_Uppercase    (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if Character_Utilities.Is_Lowercase(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Uppercase;
  -----------------------------------------------------------------------------
  function Is_Lowercase    (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if Character_Utilities.Is_Uppercase(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Lowercase;
  -----------------------------------------------------------------------------
  function Is_Digit        (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if not Character_Utilities.Is_Digit(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Digit;
  -----------------------------------------------------------------------------
  function Is_Alphabetic   (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if not Character_Utilities.Is_Alphabetic(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Alphabetic;
  -----------------------------------------------------------------------------
  function Is_Alphanumeric (The_String         : in String) return Boolean is
  begin
    for I in The_String'range loop
      if not Character_Utilities.Is_Alphanumeric(The_String(I)) then
        return False;
      end if;
    end loop;
    return True;
  end Is_Alphanumeric;
  -----------------------------------------------------------------------------
  function Centered        (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                          return String is
    Left_Margin  : Natural;
    Right_Margin : Natural;
  begin
    Left_Margin := (In_The_Width - The_String'Length) / 2;
    Right_Margin := In_The_Width - The_String'Length - Left_Margin;
    return (String'(1 .. Left_Margin => With_The_Filler) &
           The_String &
           String'(1 .. Right_Margin => With_The_Filler));
  exception
    when Constraint_Error => raise Lexical_Error;
  end Centered;
  -----------------------------------------------------------------------------
  function Left_Justified  (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                          return String is
    Local_String : String (1 .. In_The_Width);
  begin
    Local_String(1 .. The_String'Length) := The_String;
    Local_String(The_String'Length + 1 .. Local_String'Last) :=
                (others => With_The_Filler);
    return Local_String;
  exception
    when Constraint_Error => raise Lexical_Error;
  end Left_Justified;
  -----------------------------------------------------------------------------
  function Right_Justified (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                          return String is
    Local_String : String (1 .. In_The_Width);
    Shift_Offset : constant Natural := In_The_Width - The_String'Length;
  begin
    Local_String(1 .. Shift_Offset) := (others => With_The_Filler);
    Local_String(Shift_Offset + 1 .. Local_String'Last) := The_String;
    return Local_String;
  exception
    when Constraint_Error => raise Lexical_Error;
  end Right_Justified;
  -----------------------------------------------------------------------------
  function Stripped        (From_The_String    : in String;
                            The_Character      : in Character := ' ';
                            Case_Sensitive     : in Boolean := True)
                                                               return String is
    Temporary_String : String (1 .. From_The_String'Length);
    Counter          : Natural := 0;
    Local_Character  : Character;
  begin
    if Case_Sensitive then
      for I in From_The_String'range loop
        if The_Character /= From_The_String(I) then
          Counter := Counter + 1;
          Temporary_String(Counter) := From_The_String(I);
        end if;
      end loop;
    else
      Local_Character := Character_Utilities.Uppercase(The_Character);
      for I in From_The_String'range loop
--        if character_utilities.uppercase(temporary_string(i)) /=
--CHG, Schroeer, 25.3.93
        if Local_Character /= Character_Utilities.
                              Uppercase(From_The_String(I)) then
          Counter := Counter + 1;
          Temporary_String(Counter) := From_The_String(I);
        end if;
      end loop;
    end if;
    return Temporary_String(Temporary_String'First .. Counter);
  end Stripped;
  -----------------------------------------------------------------------------
  function Stripped_Leading  (From_The_String    : in String;
                              The_Character      : in Character := ' ';
                              Case_Sensitive     : in Boolean := True)
                                                               return String is
    Local_Character : Character;
  begin
    if Case_Sensitive then
      for I in From_The_String'range loop
        if The_Character /= From_The_String(I) then
          return From_The_String(I .. From_The_String'Last);
        end if;
      end loop;
    else
      Local_Character := Character_Utilities.Uppercase(The_Character);
      for I in From_The_String'range loop
        if Local_Character /= Character_Utilities.
                              Uppercase(From_The_String(I)) then
          return From_The_String(I .. From_The_String'Last);
        end if;
      end loop;
    end if;
    return "";
  end Stripped_Leading;
  -----------------------------------------------------------------------------
  function Stripped_Trailing (From_The_String    : in String;
                              The_Character      : in Character := ' ';
                              Case_Sensitive     : in Boolean := True)
                                                        return String is
    Local_Character : Character;
  begin
    if Case_Sensitive then
      for I in reverse From_The_String'range loop
        if The_Character /= From_The_String(I) then
          return From_The_String(From_The_String'First .. I);
        end if;
      end loop;
    else
      Local_Character := Character_Utilities.Uppercase(The_Character);
      for I in reverse From_The_String'range loop
        if Local_Character /= Character_Utilities.
                              Uppercase(From_The_String(I)) then
          return From_The_String(From_The_String'First .. I);
        end if;
      end loop;
    end if;
    return "";
  end Stripped_Trailing;
  -----------------------------------------------------------------------------
  function Stripped          (From_The_String    : in String;
                              The_String         : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                               return String is
    Location : constant Natural := 
               Location_Of(The_String,
                           In_The_String  => From_The_String,
                           Case_Sensitive => Case_Sensitive,
                           Forward        => Forward);
  begin
    if Location in From_The_String'range then
      return Stripped(From_The_String => From_The_String
                                        (From_The_String'First .. Location - 1),
                      The_String      => The_String,
                      Case_Sensitive  => Case_Sensitive,
                      Forward         => Forward);
    else
      return From_The_String;
    end if;
  end Stripped;                                                                   
  -----------------------------------------------------------------------------
  function Number_Of         (The_Character      : in Character;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True)
                                                        return Natural is
    Counter         : Natural := 0;
    Local_Character : Character;
  begin
    if Case_Sensitive then
      for I in In_The_String'range loop
        if In_The_String(I) = The_Character then
          Counter := Counter + 1;
        end if;
      end loop;
    else                                       
      Local_Character := Character_Utilities.Uppercase(The_Character);
      for I in In_The_String'range loop
        if Character_Utilities.Uppercase(In_The_String(I)) =
           Local_Character then
          Counter := Counter + 1;
        end if;
      end loop;
    end if;
    return Counter;
  end Number_Of;
  -----------------------------------------------------------------------------
  function Number_Of         (The_String         : in String;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Overlapped         : in Boolean := True)
                                                        return Natural is
    Counter           : Natural := 0;
    Character_Counter : Natural := In_The_String'First - 1;
  begin
    if Case_Sensitive then
      if Overlapped then
        while Character_Counter <=
              In_The_String'Length - The_String'Length loop
          if In_The_String(Character_Counter + 1 .. Character_Counter +
                The_String'Length) = The_String then
            Counter := Counter + 1;
          end if;
          Character_Counter := Character_Counter + 1;
        end loop;
      else
        while Character_Counter <=
              In_The_String'Length - The_String'Length loop
          if In_The_String(Character_Counter + 1 .. Character_Counter +
                The_String'Length) = The_String then
            Counter := Counter + 1;
            Character_Counter := Character_Counter + The_String'Length;
          else
            Character_Counter := Character_Counter + 1;
          end if;
        end loop;
      end if;
    else
      declare
        Local_The_String    : constant String := Uppercase(The_String);
        Local_In_The_String : constant String := Uppercase(In_The_String);
      begin
        if Overlapped then
          while Character_Counter <=
                Local_In_The_String'Length - Local_The_String'Length loop
            if Local_In_The_String(Character_Counter + 1 .. Character_Counter +
                  Local_The_String'Length) = Local_The_String then
              Counter := Counter + 1;
            end if;
            Character_Counter := Character_Counter + 1;
          end loop;
	      else
	        while Character_Counter <=
	              Local_In_The_String'Length - Local_The_String'Length loop
	          if Local_In_The_String(Character_Counter + 1 .. Character_Counter +
	                   Local_The_String'Length) = Local_The_String then
	            Counter := Counter + 1;
	            Character_Counter := Character_Counter + Local_The_String'Length;
	          else
	            Character_Counter := Character_Counter + 1;
	          end if;
	        end loop;
        end if;
      end;
    end if;
    return Counter;
  end Number_Of;
  -----------------------------------------------------------------------------
  function Location_Of       (The_Character      : in Character;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                      return Natural is
    Local_Character : Character;
  begin
    if Forward then
      if Case_Sensitive then
        for I in In_The_String'range loop
          if In_The_String(I) = The_Character then
            return I;
          end if;
        end loop;
      else
        Local_Character := Character_Utilities.Uppercase(The_Character);
        for I in In_The_String'range loop
          if Character_Utilities.Uppercase(In_The_String(I)) =
             Local_Character then
            return I;
          end if;
        end loop;
      end if;
    else
      if Case_Sensitive then
        for I in reverse In_The_String'range loop
          if In_The_String(I) = The_Character then
            return I;
          end if;
        end loop;
      else
        Local_Character := Character_Utilities.Uppercase(The_Character);
        for I in reverse In_The_String'range loop
          if Character_Utilities.Uppercase(In_The_String(I)) =
             Local_Character then
            return I;
          end if;
        end loop;
      end if;
    end if;
    return In_The_String'First - 1;
  end Location_Of;
  -----------------------------------------------------------------------------
  function Location_Of       (The_String         : in String;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                              return Natural is
  begin
    if Forward then
      if Case_Sensitive then
        for I in In_The_String'First ..
                 In_The_String'Last - The_String'Length + 1 loop
          if In_The_String(I .. I + The_String'Length - 1) = The_String then
            return I;
          end if;
        end loop;
      else
        declare
          Local_The_String    : constant String := Uppercase(The_String);
          Local_In_The_String : constant String := Uppercase(In_The_String);
        begin
	        for I in Local_In_The_String'First ..
	                 Local_In_The_String'Last - Local_The_String'Length + 1 loop
	          if Local_In_The_String(I .. I + Local_The_String'Length - 1) =
	             Local_The_String then
	            return I;
	          end if;
	        end loop;
         end;
       end if;
    else
-- CHG, Schroeer, 25.3.93,  the_string'last ---> the_string'length
      if Case_Sensitive then
        for I in reverse In_The_String'First .. 
                         In_The_String'Last - The_String'Length + 1 loop
          if In_The_String(I .. I + The_String'Length - 1) = The_String then
            return I;
          end if;
        end loop;
      else
        declare
          Local_The_String    : constant String := Uppercase(The_String);
          Local_In_The_String : constant String := Uppercase(In_The_String);
        begin
	        for I in reverse Local_In_The_String'First .. 
                           Local_In_The_String'Last - 
                           Local_The_String'Length + 1 loop
	          if Local_In_The_String(I .. I + Local_The_String'Length - 1) = 
                                   Local_The_String then
	            return I;
	          end if;
          end loop;
        end;
      end if;
    end if;
    return In_The_String'First - 1;
  end Location_Of;
  -----------------------------------------------------------------------------
  function Is_Equal          (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                        return Boolean is
  begin
    if Case_Sensitive then
      return Left = Right;
    else
      return Uppercase(Left) = Uppercase(Right);
    end if;
  end Is_Equal;
  -----------------------------------------------------------------------------
  function Is_Less_Than      (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                        return Boolean is
  begin
    if Case_Sensitive then
      return Left < Right;
    else
      return Uppercase(Left) < Uppercase(Right);
    end if;
  end Is_Less_Than;
  -----------------------------------------------------------------------------
  function Is_Greater_Than   (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                        return Boolean is
  begin
    if Case_Sensitive then
      return Left > Right;
    else
      return Uppercase(Left) > Uppercase(Right);
    end if;
  end Is_Greater_Than;
  -----------------------------------------------------------------------------
  --
  -- additional types, functions and procedures
  --
  -----------------------------------------------------------------------------
  function Has              (The_String          : in String;        
                             The_Character       : in Character;
                             Case_Sensitive      : in Boolean := True)
           return Boolean is
  begin
    return Location_Of(The_Character, 
                       In_The_String  => The_String,
                       Case_Sensitive => Case_Sensitive) in The_String'range;
  end Has;
  -----------------------------------------------------------------------------
  function Has              (The_String          : in String;        
                             The_Substring       : in String;
                             Case_Sensitive      : in Boolean := True)
           return Boolean is
  begin
    return Location_Of(The_String     => The_Substring, 
                       In_The_String  => The_String,
                       Case_Sensitive => Case_Sensitive) in The_String'range;
  end Has;
  -----------------------------------------------------------------------------
  function Indices_Of        (The_Character      : in Character;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True)
                                                        return Index_Array is
    Counter         : Natural := 0;
    Local_Character : Character;
    Indices         : Index_Array(1 .. In_The_String'Length);
  begin
    if Case_Sensitive then
      for I in In_The_String'range loop
        if In_The_String(I) = The_Character then
          Counter := Counter + 1;
          Indices(Counter) := I;
        end if;
      end loop;
    else
      Local_Character := Character_Utilities.Uppercase(The_Character);
      for I in In_The_String'range loop
        if Character_Utilities.Uppercase(In_The_String(I)) =
           Local_Character then
          Counter := Counter + 1;
          Indices(Counter) := I;
        end if;
      end loop;
    end if;
    return Indices(1 .. Counter);
  end Indices_Of;
  -----------------------------------------------------------------------------
  function Indices_Of        (The_String         : in String;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Overlapped         : in Boolean := True)
                                                        return Index_Array is
    Counter           : Natural := 0;
    Character_Counter : Natural := In_The_String'First - 1;
    Indices           : Index_Array(1 .. In_The_String'Length);
  begin
    if Case_Sensitive then
      if Overlapped then
        while Character_Counter <=
              In_The_String'Length - The_String'Length loop
          if In_The_String(Character_Counter + 1 .. Character_Counter +
                The_String'Length) = The_String then
            Counter := Counter + 1;
            Indices(Counter) := Character_Counter + 1;
          end if;
          Character_Counter := Character_Counter + 1;
        end loop;
      else
        while Character_Counter <=
              In_The_String'Length - The_String'Length loop
          if In_The_String(Character_Counter + 1 .. Character_Counter +
                The_String'Length) = The_String then
            Counter := Counter + 1;
            Indices(Counter)  := Character_Counter + 1;
            Character_Counter := Character_Counter + The_String'Length;
          else
            Character_Counter := Character_Counter + 1;
          end if;
        end loop;
      end if;
    else
      declare        
        Local_String_1 : constant String := Uppercase(The_String);
        Local_String_2 : constant String := Uppercase(In_The_String);
      begin
        if Overlapped then
	        while Character_Counter <=
	              In_The_String'Length - The_String'Length loop
	          if Local_String_2(Character_Counter + 1 .. Character_Counter +
	                The_String'Length) = Local_String_1 then
	            Counter := Counter + 1;
	            Indices(Counter) := Character_Counter + 1;
	          end if;
	          Character_Counter := Character_Counter + 1;
	        end loop;
	      else
	        while Character_Counter <=
	              In_The_String'Length - The_String'Length loop
	          if Local_String_2(Character_Counter + 1 .. Character_Counter +
	                   The_String'Length) = Local_String_1 then
	            Counter := Counter + 1;
	            Indices(Counter)  := Character_Counter + 1;
	            Character_Counter := Character_Counter + The_String'Length;
	          else
	            Character_Counter := Character_Counter + 1;
	          end if;
	        end loop;
        end if;
      end;
    end if;
    return Indices(1 .. Counter);
  end Indices_Of;
  -----------------------------------------------------------------------------
  procedure Allocate (Item        : in out String_Access;
                      First       : in     Integer := 1;
                      Last        : in     Integer) is
  begin
    if Item = null then
      Item := new String(First .. Last);
    elsif Item'Last /= Last or Item'First /= First then
      Deallocate (Item);
      Item := new String(First .. Last);
    end if;
  end Allocate;
  -----------------------------------------------------------------------------
  procedure Deallocate (Item : in out String_Access) is
    procedure Deallocate_String is new Ada.Unchecked_Deallocation
           (Object => String, Name => String_Access);
  begin
    Deallocate_String (Item);
  end Deallocate;
  -----------------------------------------------------------------------------
  function Is_Equal (Left           : in String_Access;
                     Right          : in String_Access) return Boolean is
  begin
    return Left.all = Right.all;
  exception
    when Constraint_Error => return Left = Right;
  end Is_Equal;
  -----------------------------------------------------------------------------
  function Copy (The_String : in String_Access) return String_Access is
  begin
    return new String'(The_String.all);
  exception
    when Constraint_Error => return new String'("");
  end Copy;
  -----------------------------------------------------------------------------
  function Length_Of (The_String : in String) return Natural is
  begin
    return The_String'Length;
  end Length_Of;
  -----------------------------------------------------------------------------
  function Type_Convert (The_Character : in Character) return String_Access is
    S : String(1 .. 1);
  begin
    S(1) := The_Character;
    return new String'(S);
  end Type_Convert;
  -----------------------------------------------------------------------------
  function Type_Convert (The_String : in String_Access) return Character is
  begin
    if The_String /= null and then The_String'Length = 1 then
      return The_String(The_String'First);
    else
      raise Lexical_Error;
    end if;
  end Type_Convert;
  -----------------------------------------------------------------------------
  function Type_Convert (The_String : in String) return String_Access is
  begin
    return new String'(The_String);
  end Type_Convert;
  -----------------------------------------------------------------------------
  function Type_Convert (The_String : in String_Access) return String is
  begin
    return The_String.all;
  exception
    when Constraint_Error => return "";
  end Type_Convert;
  -----------------------------------------------------------------------------
  function Reverse_Order (The_String : in String) return String is
    S : String (1 .. The_String'Length);
  begin
    for I in S'range loop
      S(I) := The_String (The_String'Last + 1 - I);
    end loop;
    return S;
  end Reverse_Order;
  -----------------------------------------------------------------------------
  function Delete (The_String : in String;
                   First      : in Positive;
                   Last       : in Natural) return String is
    Str : String (1 .. The_String'Length - (Last - First + 1));
  begin
    if First > Last or First > The_String'Last then
      Str := The_String;
    else
      Str := The_String (The_String'First .. First - 1) &
             The_String (Last + 1 .. The_String'Last);
    end if;
    return Str;
  end Delete;
  -----------------------------------------------------------------------------
  function Delete_Slice (The_String : in String;
                         First      : in Positive;
                         Number     : in Natural) return String is
  begin
    return Delete (The_String, First, Last => First + Number - 1);
  end Delete_Slice;
  -----------------------------------------------------------------------------
  function Insert (The_String      : in String;
                   Into_The_String : in String;
                   After_Position  : in Natural) return String is
    Str : String (1 .. Into_The_String'Length + The_String'Length);
  begin
    if After_Position > Into_The_String'Last then
      Str := Into_The_String & The_String;
    else
      Str := Into_The_String (The_String'First .. After_Position) &
             The_String &
             Into_The_String (After_Position + 1 .. Into_The_String'Last);
    end if;
    return Str;
  end Insert;
  -----------------------------------------------------------------------------
  function Insert (The_Character   : in Character;
                   Into_The_String : in String;
                   After_Position  : in Natural) return String is
    Str : String (1 .. Into_The_String'Length + 1);
  begin
    if After_Position > Into_The_String'Last then
      Str := Into_The_String & The_Character;
    else
      Str := Into_The_String (Into_The_String'First .. After_Position) &
             The_Character &
             Into_The_String (After_Position + 1 .. Into_The_String'Last);
    end if;
    return Str;
  end Insert;
  -----------------------------------------------------------------------------
  function Copy (From_The_String : in String;
                 First           : in Positive;
                 Last            : in Natural) return String is
    Str : String (1 .. Last - First + 1);
  begin
    if First > From_The_String'Last or First > Last then
      Str := "";
    elsif Last > From_The_String'Last then
      Str := From_The_String (First .. From_The_String'Last);
    else
      Str := From_The_String (First .. Last);
    end if;
    return Str;
  end Copy;
  -----------------------------------------------------------------------------
  function Copy_Slice (From_The_String : in String;
                       First           : in Positive;
                       Number          : in Natural) return String is
  begin
    return Copy (From_The_String, First, Last => First + Number - 1);
  end Copy_Slice;
  -----------------------------------------------------------------------------
  function Copy (From_The_String : in String;
                 Position        : in Positive) return Character is
  begin
    return From_The_String (Position);
  exception
    when Constraint_Error => raise Lexical_Error;
  end Copy;
  -----------------------------------------------------------------------------
  function Left_Part (From_The_String : in String;
                      Number          : in Natural) return String is
  begin
    return Copy (From_The_String, First => 1, Last => Number);
  end Left_Part;
  -----------------------------------------------------------------------------
  function Right_Part (From_The_String : in String;
                       First           : in Positive) return String is
  begin
    return Copy (From_The_String, First, Last => From_The_String'Last);
  end Right_Part;
  -----------------------------------------------------------------------------
  function Right_Part_Slice (From_The_String : in String;
                             Number          : in Natural) return String is
  begin
    if Number >= From_The_String'Last then
      return From_The_String;
    else
      return Copy_Slice (From_The_String,
                         First  => From_The_String'Last - Number + 1,
                         Number => Number);
    end if;
  end Right_Part_Slice;
  -----------------------------------------------------------------------------
  function Exchange (In_The_String  : in String;
                     Old_String     : in String;
                     New_String     : in String := "";
                     Global,
                     Case_Sensitive : in Boolean := True) return String is
    ---------------------------------------------------------------------------
    function Do_Exchange
            (In_The_String,
             Old_String     : in String;
             New_String     : in String;
             Global         : in Boolean) return String is

      Pos : Natural := Location_Of (The_String     => Old_String,
                                    In_The_String  => In_The_String,
                                    Case_Sensitive => Case_Sensitive);
    begin
      if Pos > In_The_String'First - 1 then -- old_string found
        declare
          Ret : String (1 .. In_The_String'Length - Old_String'Length) :=
                Delete (The_String => In_The_String,
                        First      => Pos,
                        Last       => Pos + Old_String'Length - 1);
        begin
          declare
            Ret1 : String (1 .. Ret'Length + New_String'Length) :=
                   Insert (The_String      => New_String,
                           Into_The_String => Ret,
                           After_Position  => Pos - In_The_String'First);
          begin
            if Global then
              return Left_Part (From_The_String => Ret1,
                                Number          => Pos + New_String'Length -
                                                   In_The_String'First)
                      & -- RECURSION
                      Do_Exchange (In_The_String => Right_Part
                                  (From_The_String => Ret1,
                                   First           => Pos + New_String'Length -
                                                      In_The_String'First + 1),
                                   Old_String    => Old_String,
                                   New_String    => New_String,
                                   Global        => Global);
            else
              return Ret1;
            end if;
          end;
        end;
      else
        return In_The_String;
      end if;
    end Do_Exchange;
    ---------------------------------------------------------------------------
  begin
    if Case_Sensitive then
      return Do_Exchange (In_The_String  => In_The_String,
                          Old_String     => Old_String,
                          New_String     => New_String,
                          Global         => Global);
    else
      return Do_Exchange (In_The_String  => Uppercase(In_The_String),
                          Old_String     => Uppercase(Old_String),
                          New_String     => Uppercase(New_String),
                          Global         => Global);
    end if;
    return In_The_String;
  end Exchange;
  -----------------------------------------------------------------------------
end Utilities.String_Utilities;
