--                              -*- Mode: Ada -*-
-- Filename        : utilities-string_utilities.ads
-- Description     : Taken from "Software components with Ada" by Grady Booch
--                   p.391
-- Author          : J. Schroeer
-- Created On      : Wed Mar 26 17:11:14 1997
-- Last Modified By:
-- Last Modified On: Mon Jul 21 10:03:52 1997
-- Update Count    : 7
-- Status          :

package Utilities.String_Utilities is

  procedure Make_Uppercase (The_String         : in out String);

  procedure Make_Lowercase (The_String         : in out String);

  procedure Capitalize     (The_String         : in out String);

  procedure Uncapitalize   (The_String         : in out String);

  procedure Replace        (The_Character      : in Character;
                            With_The_Character : in Character;
                            In_The_String      : in out String;
                            Case_Sensitive     : in Boolean := True);

  function Uppercase       (The_String         : in String) return String;

  function Lowercase       (The_String         : in String) return String;

  function Capitalized     (The_String         : in String) return String;

  function Uncapitalized   (The_String         : in String) return String;

  function Replaced        (The_Character      : in Character;
                            With_The_Character : in Character;
                            In_The_String      : in String;
                            Case_Sensitive     : in Boolean := True)
                                                              return String;

  function Is_Null         (The_String         : in String) return Boolean;

  function Is_Control      (The_String         : in String) return Boolean;

  function Is_Graphic      (The_String         : in String) return Boolean;

  function Is_Uppercase    (The_String         : in String) return Boolean;

  function Is_Lowercase    (The_String         : in String) return Boolean;

  function Is_Digit        (The_String         : in String) return Boolean;

  function Is_Alphabetic   (The_String         : in String) return Boolean;

  function Is_Alphanumeric (The_String         : in String) return Boolean;

  function Centered        (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                            return String;

  function Left_Justified  (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                            return String;

  function Right_Justified (The_String         : in String;
                            In_The_Width       : in Positive;
                            With_The_Filler    : in Character := ' ')
                                                            return String;

  function Stripped        (From_The_String    : in String;
                            The_Character      : in Character := ' ';
                            Case_Sensitive     : in Boolean := True)
                                                               return String;

  function Stripped_Leading  (From_The_String    : in String;
                              The_Character      : in Character := ' ';
                              Case_Sensitive     : in Boolean := True)
                                                                return String;

  function Stripped_Trailing (From_The_String    : in String;
                              The_Character      : in Character := ' ';
                              Case_Sensitive     : in Boolean := True)
                                                                return String;

  function Stripped          (From_The_String    : in String;
                              The_String         : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                               return String;
                                                               
  function Number_Of         (The_Character      : in Character;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True)
                                                                return Natural;

  function Number_Of         (The_String         : in String;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Overlapped         : in Boolean := True)
                                                                return Natural;

  function Location_Of       (The_Character      : in Character;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                                return Natural;

  function Location_Of       (The_String         : in String;
                              In_The_String      : in String;
                              Case_Sensitive     : in Boolean := True;
                              Forward            : in Boolean := True)
                                                                return Natural;

  function Is_Equal          (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                                return Boolean;

  function Is_Less_Than      (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                                return Boolean;

  function Is_Greater_Than   (Left               : in String;
                              Right              : in String;
                              Case_Sensitive     : in Boolean := True)
                                                                return Boolean;

  Lexical_Error : exception;

--
-- additional types, functions and procedures
--

  function Has              (The_String          : in String;        
                             The_Substring       : in String;
                             Case_Sensitive      : in Boolean := True)
                                                                return Boolean;
                                                                
  function Has              (The_String          : in String;        
                             The_Character       : in Character;
                             Case_Sensitive      : in Boolean := True)
                                                                return Boolean;
                                                                
  type Index_Array is array (Positive range <>) of Integer;                                                                
                                                                
  function Indices_Of       (The_Character       : in Character;
                             In_The_String       : in String;
                             Case_Sensitive      : in Boolean := True)
                                                        return Index_Array;
                                                        
  function Indices_Of       (The_String          : in String;
                             In_The_String       : in String;
                             Case_Sensitive      : in Boolean := True;
                             Overlapped          : in Boolean := True)
                                                        return Index_Array;
  
  type String_Access is access all String;
  
  type String_Constant_Access is access constant String;

  subtype String_P is String_Access; -- for obsolet Ada 83 code

  procedure Allocate        (Item            : in out String_Access;
                             First           : in     Integer := 1;
                             Last            : in     Integer);

  procedure Deallocate      (Item            : in out String_Access);

  function Is_Equal         (Left            : in String_Access;
                             Right           : in String_Access)
           return Boolean;

  function Copy             (The_String      : in String_Access)
           return String_Access;

  function Length_Of        (The_String      : in String)    return Natural;

  function Type_Convert     (The_Character   : in Character)
           return String_Access;
  function Type_Convert     (The_String      : in String_Access)
           return Character;

  function Type_Convert     (The_String      : in String)
           return String_Access;
  function Type_Convert     (The_String      : in String_Access)
           return String;


  function Reverse_Order    (The_String      : in String)    return String;

  function Delete           (The_String      : in String;
                             First           : in Positive;
                             Last            : in Natural)   return String;

  function Delete_Slice     (The_String      : in String;
                             First           : in Positive;
                             Number          : in Natural)   return String;

  function Insert           (The_String      : in String;
                             Into_The_String : in String;
                             After_Position  : in Natural)   return String;

  function Insert           (The_Character   : in Character;
                             Into_The_String : in String;
                             After_Position  : in Natural)   return String;

  function Copy             (From_The_String : in String;
                             First           : in Positive;
                             Last            : in Natural)   return String;

  function Copy_Slice       (From_The_String : in String;
                             First           : in Positive;
                             Number          : in Natural)   return String;

  function Copy             (From_The_String : in String;
                             Position        : in Positive)  return Character;

  function Left_Part        (From_The_String : in String;
                             Number          : in Natural)   return String;

  function Right_Part       (From_The_String : in String;
                             First           : in Positive)  return String;

  function Right_Part_Slice (From_The_String : in String;
                             Number          : in Natural)   return String;

  function Exchange         (In_The_String   : in String;
                             Old_String      : in String;
                             New_String      : in String := "";
                             Global,
                             Case_Sensitive  : in Boolean := True)
                                                             return String;

end Utilities.String_Utilities;
