------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Unconstrained_Checked_SI.Generic_Vector_Space;

procedure Ambiguities is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      16 September 2002
  --====================================================================
  -- Show ambiguities that would arise if inner and cross products were
  -- operators.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  16.09.2002
  --====================================================================

  package SI is new Unconstrained_Checked_SI (Float);
  package VS is new SI.Generic_Vector_Space;
  use SI, VS;

  V1: constant Vector := (1.0 * One, 2.0 * One, 3.0 * One);
  V2: constant Vector := (3.0 * One, 2.0 * One, 1.0 * One);
  V3: constant Vector := (4.0 * One, 5.0 * One, 6.0 * One);

begin

  declare
    function "*" (Left, Right: Vector) return Item renames Inner;
  begin
    Put_Line ("Operators are evaluated left to right: " &
              Boolean'Image (V1 * V2 * V3 = (V1 * V2) * V3));  -- True
  end;

  declare
    function "*" (Left, Right: Vector) return Item renames Inner;
  begin
    Put_Line ("Operators are evaluated left to right: " &
              Boolean'Image (V1 * V2 * V3 = V1 * (V2 * V3)));  -- False
  end;

  declare
    function "*" (Left, Right: Vector) return Item   renames Inner;
    function "*" (Left, Right: Vector) return Vector renames Cross;
  begin
    -- Put_Line (Boolean'Image (V1 * V2 * V3 = (V1 * V2) * V3));
    null;  -- >>> ambiguous expression  ^
  end;

  declare
    function "*" (Left, Right: Vector) return Item   renames Inner;
    function "*" (Left, Right: Vector) return Vector renames Cross;
    X: Item;
    -- V: Vector;
  begin
    X := V1 * V2 * V3;
    Put_Line ("Operators are evaluated left to right: " &
              Boolean'Image (V1 * V2 * V3 = Inner (Cross (V1, V2), V3)));  -- True
    -- V := V1 * V2 * V3;
    -- >>>          ^ ambiguous expression
    X := (V1 * V2) * V3;
    Put_Line ("Operators are interpreted Cross times Inner: " &
              Boolean'Image ((V1 * V2) * V3 = Inner (Cross (V1, V2), V3)));  -- True
    -- V := (V1 * V2) * V3;
    -- >>>            ^ ambiguous expression
  end;

end Ambiguities;