------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2003 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Rational_Arithmetics;
use  Rational_Arithmetics;

with Constrained_Checked_SI.Generic_Polynomial_Numerics,
     Constrained_Checked_SI.Generic_Units,
     Constrained_Checked_SI.Generic_Text_IO;

with Test_Support;
use  Test_Support;

procedure Test_Constrained_Polynomial_Numerics is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      21 February 2003
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  21.02.2003
  --====================================================================

  package SI   is new Constrained_Checked_SI (Float);
  package Unit is new SI.Generic_Units;
  package IO   is new SI.Generic_Text_IO;
  package Poly is new SI.Generic_Polynomial_Numerics;
  use SI, IO, Unit, Poly;

  D0: constant Item :=  5.0  * Meter;
  D2: constant Item := -9.81 * Meter_p_Second_2 / 2.0;

  Coefficients: constant Table := (1.0, 0.0, Value (D2 / D0));

  T: constant Table (0 .. 2) := (   0.0    , Value (Sqrt (-D0/D2)),   4.0 );
  D: constant Table (0 .. 2) := (Value (D0),       0.0            , -73.48);

  Result: Item := Meter;
  Table : Table_2nd_Order;

begin

  Test_Header (Title       => "Test Polynomial_Numerics",
               Description => "Polynomes and linear interpolation for" &
                              " constrained items.");

  -----------------------------------------------------------------------
  Test_Step (Title       => "Test polynomes",
             Description => "");

  for I in T'Range loop

    Result := D0 * Polynomial (Coefficients, T (I));

    Put (Result);
    Assert (Condition         => abs (Result - D (I) * Meter) < 1.0E-6 * Meter,
            Message           => "  Index" & Whole'Image (I),
            Only_Report_Error => False);

  end loop;

  -----------------------------------------------------------------------
  Test_Step (Title       => "Test second order curve",
             Description => "");

  Table := Second_Order_Curve (T, D);

  for I in Table'Range loop

    Put (Table (I) * One);
    Assert (Condition         => abs (Table (I) / Value (D0) - Coefficients (I)) < 1.0E-6,
            Message           => "  Index" & Whole'Image (I),
            Only_Report_Error => False);

  end loop;

  -----------------------------------------------------------------------
  Test_Result;

end Test_Constrained_Polynomial_Numerics;
