------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Rational_Arithmetics;
use  Rational_Arithmetics;

with Unconstrained_Checked_SI.Generic_Polynomial_Numerics,
     Unconstrained_Checked_SI.Generic_Units,
     Unconstrained_Checked_SI.Generic_Text_IO;

with Test_Support;
use  Test_Support;

procedure Test_Polynomial_Numerics is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   2.0
  -- Date      26 July 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  19.05.1999
  --  C.G.    2.0  26.07.2002 Adapted to SI package
  --====================================================================

  package SI   is new Unconstrained_Checked_SI (Float);
  package Unit is new SI.Generic_Units;
  package IO   is new SI.Generic_Text_IO (Unit);
  package Poly is new SI.Generic_Polynomial_Numerics;
  use SI, IO, Poly;

  Coefficients: constant Table :=
    (1.0 * Meter, 3.0 * Meter/Second, 5.0 * Meter/Second**2);

  X: constant Table (0 .. 2) := (0.0 * Second, -2.0 * Second,  4.0 * Second);
  Y: constant Table (0 .. 2) := (1.0 * Meter , 15.0 * Meter , 93.0 * Meter);

  Result: Item;
  Table : Table_2nd_Order;

begin

  Test_Header (Title       => "Test Polynomial_Numerics",
               Description => "Polynomes and linear interpolation.");

  -----------------------------------------------------------------------
  Test_Step (Title       => "Test polynomes",
             Description => "");

  for I in X'Range loop

    Result := Polynomial (Coefficients, X (I));

    Put (Result);
    Assert (Condition         => Result = Y (I),
            Message           => "Index" & Whole'Image (I),
            Only_Report_Error => False);

  end loop;

  -----------------------------------------------------------------------
  Test_Step (Title       => "Test second order curve",
             Description => "");

  Table := Second_Order_Curve (X, Y);

  for I in Table'Range loop

    Put (Table (I));
    Assert (Condition         => Table (I) = Coefficients (I),
            Message           => "Index" & Whole'Image (I),
            Only_Report_Error => False);

  end loop;

  -----------------------------------------------------------------------
  Test_Result;

end Test_Polynomial_Numerics;
