------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

--#cc with Constrained_Checked_SI.Generic_Item_Subtype,
--#cc      Constrained_Checked_SI.Generic_Text_IO;
--#uc with Unconstrained_Checked_SI.Generic_Item_Subtype,
--#uc      Unconstrained_Checked_SI.Generic_Units,
--#uc      Unconstrained_Checked_SI.Generic_Text_IO;
--#cu with Constrained_Unchecked_SI.Generic_Item_Subtype,
--#cu      Constrained_Unchecked_SI.Generic_Text_IO;

procedure Test_SI_Subtyping is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      9 November 2002
  --====================================================================
  -- Test the "subtype" compatibility.
  --
  --   To get the version      uncomment the lines:
  -- ----------------------------------------------
  --   constrained   checked      "--#cc"
  -- unconstrained   checked      "--#uc"
  --   constrained unchecked      "--#cu"
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  09.11.2002
  --====================================================================

  --#cu package SI is new Constrained_Unchecked_SI (Float);
  --#cu package IO is new SI.Generic_Text_IO;
  --#cc package SI is new Constrained_Checked_SI (Float);
  --#cc package IO is new SI.Generic_Text_IO;
  --#uc package SI is new Unconstrained_Checked_SI (Float);
  --#uc package U  is new SI.Generic_Units;
  --#uc package IO is new SI.Generic_Text_IO (U);
  use SI, IO;

  Hour: constant Item := 60.0**2 * Second;

  package Car_Speed is new SI.Generic_Item_Subtype
                               (Min  =>   0.0 * Kilo*Meter/Hour,
                                Max  => 160.0 * Kilo*Meter/Hour);
  use Car_Speed;

  Curve_Radius: constant Item :=  30.0 * Meter;
  Speed       : constant Item := 120.0 * Kilo*Meter/Hour;

begin

  Put ("no subtype");
  Put (Speed**2 / Curve_Radius);
  New_Line;
  Put ("  subtyped");
  Put (Constrain(Speed)**2 / Curve_Radius);
  New_Line;

  begin
    Put (Constrain(2.0 * Speed)**2 / Curve_Radius);
  exception
    when Constraint_Error =>
      Put_Line ("Constraint_Error");
  end;

  begin
    Put (Constrain (Speed**2) / Curve_Radius);
  exception
    --#cc when Unit_Error =>
    --#cc   Put_Line ("Unit_Error");
    --#uc when Unit_Error =>
    --#uc   Put_Line ("Unit_Error");
    --#cu when Constraint_Error =>  -- no dimension
    --#cu   Put_Line ("Constraint_Error");
  end;

end Test_SI_Subtyping;