------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Unconstrained_Checked_SI.Generic_Vector_Space,
     Unconstrained_Checked_SI.Generic_Units,
     Unconstrained_Checked_SI.Generic_Text_IO;

procedure Test_Unconstrained_Checked_SI_Vector_Space is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      8 November 2002
  --====================================================================
  -- Test the matrix inversion.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  08.11.2002
  --====================================================================

  package SI is new Unconstrained_Checked_SI (Float);
  package U  is new SI.Generic_Units;
  package IO is new SI.Generic_Text_IO (U);
  package VS is new SI.Generic_Vector_Space;
  use SI, IO, VS;

  M: constant Matrix := (( 3.0 * One,  2.0 * One,  1.0 * One),
                         ( 1.0 * One,  0.0 * One,  2.0 * One),
                         ( 4.0 * One,  1.0 * One,  3.0 * One));
  N: constant Matrix := ((-0.4 * One, -1.0 * One,  0.8 * One),
                         ( 1.0 * One,  1.0 * One, -1.0 * One),
                         ( 0.2 * One,  1.0 * One, -0.4 * One));
  D: constant Item   := Det (M);

begin

  Put_Line ("Det (M) = 5 => " & Boolean'Image (Det (M) = 5.0 * One));  -- True
  Put_Line ("Inv (M) = N => " & Boolean'Image (Inv (M) = N));          -- True
  Put_Line ("Inv (N) = M => " & Boolean'Image (Inv (N) = M));          -- False (rounding errors)

  New_Line;
  Put_Line ("Inv (N) =>");
  for I in Axis loop
    for J in Axis loop
      Put (Inv (N)(I, J), Aft => 0, Exp => 0);
    end loop;
    New_Line;
  end loop;

end Test_Unconstrained_Checked_SI_Vector_Space;