------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Unconstrained_Checked_SI.Generic_Units, Unconstrained_Checked_SI.Generic_Text_IO;

procedure Test_Unconstrained_SI_Put is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      6 August 2002
  --====================================================================
  -- Test the Dim format modifier of Put.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  06.08.2002
  --====================================================================

  package Float_SI       is new Unconstrained_Checked_SI (Float);
  package Float_SI_Units is new Float_SI.Generic_Units;
  package Float_SI_IO    is new Float_SI.Generic_Text_IO (Float_SI_Units);
  use Float_SI, Float_SI_Units, Float_SI_IO;

begin

  Put (1.0 * Kilo * Meter / Second);                    New_Line;
  Put (1.0 * Kilo * Meter / Second, Dim => "km/s   ");  New_Line;
  Put (1.0 * Kilo * Meter / Second, Dim => "*km/s");    New_Line;
  Put (1.0 * Kilo * Meter / Second, Dim => "/ms*m");    New_Line;

  begin
    Put (1.0 * Kilo * Meter / Second, Dim => "Km/s");
    Put_Line ("=> Wrong Km/s");
  exception
    when Unit_Error => Put_Line ("Unit_Error Km/s");
  end;

  begin
    Put (1.0 * Kilo * Meter / Second, Dim => "km/sA");
    Put_Line ("=> Wrong km/sA");
  exception
    when Unit_Error => Put_Line ("Unit_Error km/sA");
  end;

  begin
    Put (1.0 * Kilo * Meter / Second, Dim => "km/s   A");
    Put_Line ("=> Wrong km/s   A");
  exception
    when Unit_Error => Put_Line ("Unit_Error km/s   A");
  end;

  begin
    Put (1.0 * Kilo * Meter / Second, Dim => "A");
    Put_Line ("=> Wrong A");
  exception
    when Unit_Error => Put_Line ("Unit_Error A");
  end;

  Put (1.0 * (Kilo * Meter / Second)**2, Dim => "km^2/s^2");      New_Line;
  Put (1.0 * (Kilo * Meter / Second)**2, Dim => "km^(2/1)/s^2");  New_Line;

  begin
    Put (1.0 * (Kilo * Meter / Second)**2, Dim => "km^(2/1/s^2");
    Put_Line ("=> Wrong km^(2/1/s^2");
  exception
    when Unit_Error => Put_Line ("Unit_Error km^(2/1/s^2");
  end;

  begin
    Put (1.0 * (Kilo * Meter / Second)**2, Dim => "(km/s)^2");
    Put_Line ("=> Wrong (km/s)^2");
  exception
    when Unit_Error => Put_Line ("Unit_Error (km/s)^2");
  end;

end Test_Unconstrained_SI_Put;
