------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Unconstrained_Unchecked_SI.Generic_Units,
     Unconstrained_Unchecked_SI.Generic_Text_IO,
     Unconstrained_Unchecked_SI.Generic_Temperatures.Generic_Text_IO;

procedure Test_Unconstrained_Unchecked_SI_Temperature_Differences is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      8 October 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  08.10.2002
  --====================================================================

  package SI      is new Unconstrained_Unchecked_SI (Float);
  package Units   is new SI.Generic_Units;
  package Temp    is new SI.Generic_Temperatures;
  package Temp_IO is new Temp.Generic_Text_IO;
  package IO      is new SI.Generic_Text_IO (Units);
  use SI, Units, Temp, Temp_IO, IO;

  type Result is record
    K : Item;
    C : Temperature_Difference (Celsius);
    F : Temperature_Difference (Fahrenheit);
    Rk: Temperature_Difference (Rankine);
    Rm: Temperature_Difference (Reaumur);
  end record;

  Expected: constant array (1 .. 5) of Result :=
    (1 => (K  => 1.0 * Kelvin,
           C  => to_Celsius    (1.00),
           F  => to_Fahrenheit (1.80),
           Rk => to_Rankine    (1.80),
           Rm => to_Reaumur    (0.80)),
     2 => (K  => 1.0 * Kelvin,
           C  => to_Celsius    (1.00),
           F  => to_Fahrenheit (1.80),
           Rk => to_Rankine    (1.80),
           Rm => to_Reaumur    (0.80)),
     3 => (K  => 0.56 * Kelvin,
           C  => to_Celsius    (0.56),
           F  => to_Fahrenheit (1.00),
           Rk => to_Rankine    (1.00),
           Rm => to_Reaumur    (0.44)),
     4 => (K  => 0.56 * Kelvin,
           C  => to_Celsius    (0.56),
           F  => to_Fahrenheit (1.00),
           Rk => to_Rankine    (1.00),
           Rm => to_Reaumur    (0.44)),
     5 => (K  => 1.25 * Kelvin,
           C  => to_Celsius    (1.25),
           F  => to_Fahrenheit (2.25),
           Rk => to_Rankine    (2.25),
           Rm => to_Reaumur    (1.00)));

begin

  IO.Default_Fore := 5;
  IO.Default_Aft  := 2;
  IO.Default_Exp  := 0;

  Temp_IO.Default_Fore := IO.Default_Fore;
  Temp_IO.Default_Aft  := IO.Default_Aft;
  Temp_IO.Default_Exp  := IO.Default_Exp;

  ------------------------------------------------------------------------

  Put_Line ("Temperature difference Kelvin -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).C);
    Put (Temperature_Difference'(to_Celsius (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Kelvin -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).F);
    Put (Temperature_Difference'(to_Fahrenheit (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Kelvin -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).Rk);
    Put (Temperature_Difference'(to_Rankine (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Kelvin -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).Rm);
    Put (Temperature_Difference'(to_Reaumur (Expected (I).K)));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature difference Celsius -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).C), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Celsius -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).C));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Celsius -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).C));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Celsius -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).C));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature difference Fahrenheit -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).F), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Fahrenheit -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).F));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Fahrenheit -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).F));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Fahrenheit -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).F));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature difference Rankine -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).Rk), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Rankine -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).Rk));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Rankine -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).Rk));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Rankine -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).Rk));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature difference Reaumur -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).Rm), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Reaumur -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).Rm));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Reaumur -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).Rm));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature difference Reaumur -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).Rm));
    New_Line;
  end loop;

end Test_Unconstrained_Unchecked_SI_Temperature_Differences;
