------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

with Unconstrained_Unchecked_SI.Generic_Units,
     Unconstrained_Unchecked_SI.Generic_Text_IO,
     Unconstrained_Unchecked_SI.Generic_Temperatures.Generic_Text_IO;

procedure Test_Unconstrained_Unchecked_SI_Temperatures is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      8 October 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  08.10.2002
  --====================================================================

  package SI      is new Unconstrained_Unchecked_SI (Float);
  package Units   is new SI.Generic_Units;
  package Temp    is new SI.Generic_Temperatures;
  package Temp_IO is new Temp.Generic_Text_IO;
  package IO      is new SI.Generic_Text_IO (Units);
  use SI, Units, Temp, Temp_IO, IO;

  type Result is record
    K : Item;
    C : Temperature (Celsius);
    F : Temperature (Fahrenheit);
    Rk: Temperature (Rankine);
    Rm: Temperature (Reaumur);
  end record;

  Expected: constant array (1 .. 4) of Result :=
    (1 => (K  => 0.0 * Kelvin,
           C  => to_Celsius    (-273.15),
           F  => to_Fahrenheit (-459.67),
           Rk => to_Rankine    (   0.0 ),
           Rm => to_Reaumur    (-218.52)),
     2 => (K  => 255.37 * Kelvin,
           C  => to_Celsius    ( -17.78),
           F  => to_Fahrenheit (   0.0 ),
           Rk => to_Rankine    ( 459.67),
           Rm => to_Reaumur    ( -14.22)),
     3 => (K  => 273.15 * Kelvin,
           C  => to_Celsius    (   0.0 ),
           F  => to_Fahrenheit (  32.0 ),
           Rk => to_Rankine    ( 491.67),
           Rm => to_Reaumur    (   0.0 )),
     4 => (K  => 373.15 * Kelvin,
           C  => to_Celsius    ( 100.0 ),
           F  => to_Fahrenheit ( 212.0 ),
           Rk => to_Rankine    ( 671.67),
           Rm => to_Reaumur    (  80.0 )));

begin

  IO.Default_Fore := 5;
  IO.Default_Aft  := 2;
  IO.Default_Exp  := 0;

  Temp_IO.Default_Fore := IO.Default_Fore;
  Temp_IO.Default_Aft  := IO.Default_Aft;
  Temp_IO.Default_Exp  := IO.Default_Exp;

  ------------------------------------------------------------------------

  Put_Line ("Temperature Kelvin -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).C);
    Put (Temperature'(to_Celsius (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Kelvin -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).F);
    Put (Temperature'(to_Fahrenheit (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Kelvin -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).Rk);
    Put (Temperature'(to_Rankine (Expected (I).K)));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Kelvin -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).K, Dim => "K");
    Put (Expected (I).Rm);
    Put (Temperature'(to_Reaumur (Expected (I).K)));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature Celsius -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).C), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Celsius -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).C));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Celsius -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).C));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Celsius -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).C);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).C));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature Fahrenheit -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).F), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Fahrenheit -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).F));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Fahrenheit -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).F));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Fahrenheit -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).F);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).F));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature Rankine -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).Rk), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Rankine -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).Rk));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Rankine -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).Rk));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Rankine -> Reaumur");

  for I in Expected'Range loop
    Put (Expected (I).Rk);
    Put (Expected (I).Rm);
    Put (to_Reaumur (Expected (I).Rk));
    New_Line;
  end loop;

  ------------------------------------------------------------------------

  New_Line;
  Put_Line ("Temperature Reaumur -> Kelvin");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).K, Dim => "K");
    Put (to_Kelvin (Expected (I).Rm), Dim => "K");
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Reaumur -> Celsius");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).C);
    Put (to_Celsius (Expected (I).Rm));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Reaumur -> Fahrenheit");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).F);
    Put (to_Fahrenheit (Expected (I).Rm));
    New_Line;
  end loop;

  New_Line;
  Put_Line ("Temperature Reaumur -> Rankine");

  for I in Expected'Range loop
    Put (Expected (I).Rm);
    Put (Expected (I).Rk);
    Put (to_Rankine (Expected (I).Rm));
    New_Line;
  end loop;

end Test_Unconstrained_Unchecked_SI_Temperatures;
