------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

package body Constrained_Checked_SI.Generic_Item_Subtype is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.2
  -- Date      26 November 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  09.11.2002
  --  C.G.    1.1  15.11.2002 Use Real'Base
  --  C.G.    1.2  26.11.2002 Constrain simplified (no explict dimension
  --                          checking necessary, implicit by "<=")
  --====================================================================

  use SI;

  function Constrain (X: Item) return Constrained is
    -- Dimension check is done implicitly by "<=".
  begin
    if not (Min <= X and X <= Max) then
      raise Constraint_Error;
    end if;
    return (Value => X);
  end Constrain;

  function Value (X: Constrained) return Real is
  begin
    return Value (X.Value);
  end Value;

  function "+" (Right: Constrained) return Item is
  begin
    return Right.Value;
  end "+";

  function "-" (Right: Constrained) return Item is
  begin
    return -Right.Value;
  end "-";

  function "+" (Left: Item; Right: Constrained) return Item is
  begin
    return Left + Right.Value;
  end "+";

  function "+" (Left: Constrained; Right: Item) return Item is
  begin
    return Right + Left;
  end "+";

  function "-" (Left: Item; Right: Constrained) return Item is
  begin
    return Left - Right.Value;
  end "-";

  function "-" (Left: Constrained; Right: Item) return Item is
  begin
    return Left.Value - Right;
  end "-";

  function "*" (Left: Item; Right: Constrained) return Item is
  begin
    return Left * Right.Value;
  end "*";

  function "*" (Left: Constrained; Right: Item) return Item is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Item; Right: Constrained) return Item is
  begin
    return Left / Right.Value;
  end "/";

  function "/" (Left: Constrained; Right: Item) return Item is
  begin
    return Left.Value / Right;
  end "/";

  function "*" (Left: Constrained; Right: Real'Base) return Item is
  begin
    return Left.Value * Right;
  end "*";

  function "*" (Left: Real'Base; Right: Constrained) return Item is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Constrained; Right: Real'Base) return Item is
  begin
    return Left.Value / Right;
  end "/";

  function "/" (Left: Real'Base; Right: Constrained) return Item is
  begin
    return Left / Right.Value;
  end "/";

  function "**" (Base: Constrained; Exponent: Whole) return Item is
  begin
    return Base.Value ** Exponent;
  end "**";

  function "**" (Base: Constrained; Exponent: Rational) return Item is
  begin
    return  Base.Value ** Exponent;
  end "**";

  function "**" (Base: Constrained; Exponent: Real'Base) return Item is
  begin
    return  Base.Value ** Exponent;
  end "**";

begin

  if Dimension_of (Min.Inner) /= Dimension_of (Max.Inner) then
    raise Unit_Error;
  end if;

end Constrained_Checked_SI.Generic_Item_Subtype;