------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Text_IO;
use  Ada.Text_IO;

generic
package Constrained_Checked_SI.Generic_Temperatures.Generic_Text_IO is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      8 October 2002
  --====================================================================
  -- Put writes the value in the numeric format given (Fore, Aft, Exp),
  -- followed by the dimension C for Celsius, F for Fahrenheit,
  -- Rank for Rankine, R for Raumur. (There is no space between the
  -- value and the dimension.)
  --
  -- Get reads as many characters as can be interpreted in the format
  -- that Put writes; the characters read are consumed. If a legal
  -- interpretation is found, it is returned; else the offending
  -- character remains in the input stream.
  -- Additionally to the exceptions that may be raised as with standard
  -- Text_IO, Unit_Error is raised if no legal dimension is given.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  08.10.2002
  --====================================================================

  pragma Elaborate_Body;

  Default_Fore: Field := 2;
  Default_Aft : Field := Real'Digits - 1;
  Default_Exp : Field := 3;

  function Get                      return Temperature;
  function Get (File: in File_Type) return Temperature;

  procedure Put (X   : in Temperature;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp);
  procedure Put (File: in File_Type;
                 X   : in Temperature;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp);

  function Get                      return Temperature_Difference;
  function Get (File: in File_Type) return Temperature_Difference;

  procedure Put (X   : in Temperature_Difference;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp);
  procedure Put (File: in File_Type;
                 X   : in Temperature_Difference;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp);

end Constrained_Checked_SI.Generic_Temperatures.Generic_Text_IO;
