------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

package body Constrained_Unchecked_SI.Generic_Item_Subtype is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      15 November 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  09.11.2002
  --  C.G.    1.1  15.11.2002 Use Real'Base
  --====================================================================

  use SI, Item_Subtype;

  function Constrain (X: Item) return Constrained is
  begin
    return (Value => Constrain (X.Inner));
  end Constrain;

  function Value (X: Constrained) return Real is
  begin
    return Value (X.Value);
  end Value;

  function "+" (Right: Constrained) return Item is
  begin
    return (Inner => Right.Value * SI.One);
  end "+";

  function "-" (Right: Constrained) return Item is
  begin
    return (Inner => -Right.Value * SI.One);
  end "-";

  function "+" (Left: Item; Right: Constrained) return Item is
  begin
    return (Inner => Left.Inner + Right.Value);
  end "+";

  function "+" (Left: Constrained; Right: Item) return Item is
  begin
    return Right + Left;
  end "+";

  function "-" (Left: Item; Right: Constrained) return Item is
  begin
    return (Inner => Left.Inner - Right.Value);
  end "-";

  function "-" (Left: Constrained; Right: Item) return Item is
  begin
    return (Inner => Left.Value - Right.Inner);
  end "-";

  function "*" (Left: Item; Right: Constrained) return Item is
  begin
    return (Inner => Left.Inner * Right.Value);
  end "*";

  function "*" (Left: Constrained; Right: Item) return Item is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Item; Right: Constrained) return Item is
  begin
    return (Inner => Left.Inner / Right.Value);
  end "/";

  function "/" (Left: Constrained; Right: Item) return Item is
  begin
    return (Inner => Left.Value / Right.Inner);
  end "/";

  function "*" (Left: Constrained; Right: Real'Base) return Item is
  begin
    return (Inner => Left.Value * Right);
  end "*";

  function "*" (Left: Real'Base; Right: Constrained) return Item is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Constrained; Right: Real'Base) return Item is
  begin
    return (Inner => Left.Value / Right);
  end "/";

  function "/" (Left: Real'Base; Right: Constrained) return Item is
  begin
    return (Inner => Left / Right.Value);
  end "/";

  function "**" (Base: Constrained; Exponent: Whole) return Item is
  begin
    return (Inner => Base.Value ** Exponent);
  end "**";

  function "**" (Base: Constrained; Exponent: Rational) return Item is
  begin
    return (Inner => Base.Value ** Exponent);
  end "**";

  function "**" (Base: Constrained; Exponent: Real'Base) return Item is
  begin
    return (Inner => Base.Value ** Exponent);
  end "**";

end Constrained_Unchecked_SI.Generic_Item_Subtype;