------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002, 2003 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Characters.Latin_1, Ada.Strings.Maps;
with IO_Customization;

package body Constrained_Unchecked_SI.Generic_Temperatures.Generic_Text_IO is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.2
  -- Date      11 March 2003
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  10.10.2002
  --  C.G.    1.1  15.11.2002 Use Real'Base
  --  C.G.    1.2  11.03.2003 IO_Customization
  --====================================================================

  LF : Character renames Ada.Characters.Latin_1.LF;

  package Real_Text_IO is new Ada.Text_IO.Float_IO (Real'Base);
  use Real_Text_IO;

  function Get (File: in File_Type) return Scale_Name is
    function Look_Ahead return Character is
      C  : Character;
      EoL: Boolean;
    begin
      Look_Ahead (File, C, EoL);
      if EoL then
        return LF;
      else
        return C;
      end if;
    end Look_Ahead;
    procedure Consume is
      C: Character;
    begin
      Get (File, C);
    end Consume;
    use IO_Customization, Ada.Strings.Maps;
  begin
    if not Is_In (Look_Ahead, Degree) then
      raise Unit_Error;
    end if;
    Consume;
    case Look_Ahead is
      when 'C' => Consume;
                  return Celsius;
      when 'F' => Consume;
                  return Fahrenheit;
      when 'R' => Consume;
                  if Look_Ahead = 'a' then
                    Consume;
                    if Look_Ahead = 'n' then
                      Consume;
                      if Look_Ahead = 'k' then
                        Consume;
                        return Rankine;
                      end if;
                    end if;
                    raise Unit_Error;
                  else
                    return Reaumur;
                  end if;
      when others => raise Unit_Error;
    end case;
  end Get;

  function Get return Temperature is
  begin
    return Get (Current_Input);
  end Get;

  function Get (File: in File_Type) return Temperature is
    Value: Real'Base;
  begin
    Get (File, Value);
    declare
      Temp: Temperature (Get (File));
    begin
      Temp.Value := Value;
      return Temp;
    end;
  end Get;

  procedure Put (X   : in Temperature;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp) is
  begin
    Put (Current_Output, X, Fore, Aft, Exp);
  end Put;

  procedure Put (File: in File_Type;
                 X   : in Temperature;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp) is
    use IO_Customization;
  begin
    Put (File, X.Value, Fore, Aft, Exp);
    case X.Scale is
      when Celsius    => Put (File, Default_Degree & 'C');
      when Fahrenheit => Put (File, Default_Degree & 'F');
      when Rankine    => Put (File, Default_Degree & "Rank");
      when Reaumur    => Put (File, Default_Degree & 'R');
    end case;
  end Put;

  function Get return Temperature_Difference is
  begin
    return Get (Current_Input);
  end Get;

  function Get (File: in File_Type) return Temperature_Difference is
    Value: Real'Base;
  begin
    Get (File, Value);
    declare
      Temp: Temperature_Difference (Get (File));
    begin
      Temp.Value := Value;
      return Temp;
    end;
  end Get;

  procedure Put (X   : in Temperature_Difference;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp) is
  begin
    Put (Current_Output, X, Fore, Aft, Exp);
  end Put;

  procedure Put (File: in File_Type;
                 X   : in Temperature_Difference;
                 Fore: in Field := Default_Fore;
                 Aft : in Field := Default_Aft;
                 Exp : in Field := Default_Exp) is
    use IO_Customization;
  begin
    Put (File, X.Value, Fore, Aft, Exp);
    case X.Scale is
      when Celsius    => Put (File, Default_Degree & 'C');
      when Fahrenheit => Put (File, Default_Degree & 'F');
      when Rankine    => Put (File, Default_Degree & "Rank");
      when Reaumur    => Put (File, Default_Degree & 'R');
    end case;
  end Put;

end Constrained_Unchecked_SI.Generic_Temperatures.Generic_Text_IO;
