------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

package body Constrained_Unchecked_SI.Generic_Vector_Space is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      18 November 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  18.11.2002
  --====================================================================

  use Vector_Space, SI;

  function "*" (Left: Proto_Vector; Right: Item) return Vector is
    V: constant Vector_Space.Vector := (Left (1) * SI.One, Left (2) * SI.One, Left (3) * SI.One);
  begin
    return (Inner => V * Right.Inner);
  end "*";

  function "*" (Left: Item; Right: Proto_Vector) return Vector is
  begin
    return Right * Left;
  end "*";

  function Normalize (X: Proto_Vector) return Vector is
    V: constant Vector := X * One;
  begin
    return V / abs V;
  end Normalize;

  function Get (V: Vector; A: Axis) return Item is
  begin
    return (Inner => V.Inner (Vector_Space.Axis (A)));
  end Get;

  procedure Set (V: in out Vector; A: in Axis; It: in Item) is
  begin
    V.Inner (Vector_Space.Axis (A)) := It.Inner;
  end Set;

  function "abs" (Right: Vector) return Item is
  begin
    return (Inner => abs Right.Inner);
  end "abs";

  function "+" (Right: Vector) return Vector is
  begin
    return Right;
  end "+";

  function "-" (Right: Vector) return Vector is
  begin
    return (Inner => -Right.Inner);
  end "-";

  function "+" (Left, Right: Vector) return Vector is
  begin
    return (Inner => Left.Inner + Right.Inner);
  end "+";

  function "-" (Left, Right: Vector) return Vector is
  begin
    return (Inner => Left.Inner - Right.Inner);
  end "-";

  function "*" (Left: Real'Base; Right: Vector) return Vector is
  begin
    return (Inner => Left * Right.Inner);
  end "*";

  function "*" (Left: Item; Right: Vector) return Vector is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

  function "*" (Left: Vector; Right: Real'Base) return Vector is
  begin
    return Right * Left;
  end "*";

  function "*" (Left: Vector; Right: Item) return Vector is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Vector; Right: Real'Base) return Vector is
  begin
    return (Inner => Left.Inner / Right);
  end "/";

  function "/" (Left: Vector; Right: Item) return Vector is
  begin
    return (Inner => Left.Inner / Right.Inner);
  end "/";

  function Inner (Left, Right: Vector) return Item is
  begin
    return (Inner => Inner (Left.Inner, Right.Inner));
  end Inner;

  function Cross (Left, Right: Vector) return Vector is
  begin
    return (Inner => Cross (Left.Inner, Right.Inner));
  end Cross;

  function "*" (Left: Proto_Matrix; Right: Item) return Matrix is
    M: constant Vector_Space.Matrix := ((Left (1, 1) * SI.One, Left (1, 2) * SI.One, Left (1, 3) * SI.One),
                                        (Left (2, 1) * SI.One, Left (2, 2) * SI.One, Left (2, 3) * SI.One),
                                        (Left (3, 1) * SI.One, Left (3, 2) * SI.One, Left (3, 3) * SI.One));
  begin
    return (Inner => M * Right.Inner);
  end "*";

  function "*" (Left: Item; Right: Proto_Matrix) return Matrix is
  begin
    return Right * Left;
  end "*";

  function Get (M: Matrix; A, B: Axis) return Item is
  begin
    return (Inner => M.Inner (Vector_Space.Axis (A), Vector_Space.Axis (B)));
  end Get;

  procedure Set (M: in out Matrix; A, B: in Axis; It: in Item) is
  begin
    M.Inner (Vector_Space.Axis (A), Vector_Space.Axis (B)) := It.Inner;
  end Set;

  function "+" (Right: Matrix) return Matrix is
  begin
    return Right;
  end "+";

  function "-" (Right: Matrix) return Matrix is
  begin
    return (Inner => -Right.Inner);
  end "-";

  function "+" (Left, Right: Matrix) return Matrix is
  begin
    return (Inner => Left.Inner + Right.Inner);
  end "+";

  function "-" (Left, Right: Matrix) return Matrix is
  begin
    return (Inner => Left.Inner - Right.Inner);
  end "-";

  function "*" (Left: Real'Base; Right: Matrix) return Matrix is
  begin
    return (Inner => Left * Right.Inner);
  end "*";

  function "*" (Left: Matrix; Right: Real'Base) return Matrix is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Matrix; Right: Real'Base) return Matrix is
  begin
    return (Inner => Left.Inner / Right);
  end "/";

  function "*" (Left: Item; Right: Matrix) return Matrix is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

  function "*" (Left: Matrix; Right: Item) return Matrix is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Matrix; Right: Item) return Matrix is
  begin
    return (Inner => Left.Inner / Right.Inner);
  end "/";

  function "*" (Left, Right: Matrix) return Matrix is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

  function Det (M: Matrix) return Item is
  begin
    return (Inner => Det (M.Inner));
  end Det;

  function Inv (M: Matrix) return Matrix is
  begin
    return (Inner => Inv (M.Inner));
  end Inv;

  function Tran (M: Matrix) return Matrix is
  begin
    return (Inner => Tran (M.Inner));
  end Tran;

  function "*" (Left: Matrix; Right: Vector) return Vector is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

  function "*" (Left: Vector; Right: Matrix) return Vector is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

end Constrained_Unchecked_SI.Generic_Vector_Space;
