------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

package body Constrained_Unchecked_SI is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.1
  -- Date      15 November 2002
  --====================================================================
  -- All operations are reduced to the ones in
  -- Unconstrained_Unchecked_SI.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  25.07.2002
  --  C.G.    1.1  15.11.2002 Use Real'Base
  --====================================================================

  use SI;

  function Value (X: Item) return Real is
  begin
    return Value (X.Inner);
  end Value;

  -- Operators

  function "abs" (Right: Item) return Item is
  begin
    return (Inner => abs Right.Inner);
  end "abs";

  function "+" (Right: Item) return Item is
  begin
    return Right;
  end "+";

  function "-" (Right: Item) return Item is
  begin
    return (Inner => -Right.Inner);
  end "-";

  function "+" (Left, Right: Item) return Item is
  begin
    return (Inner => Left.Inner + Right.Inner);
  end "+";

  function "-" (Left, Right: Item) return Item is
  begin
    return (Inner => Left.Inner - Right.Inner);
  end "-";

  function "*" (Left, Right: Item) return Item is
  begin
    return (Inner => Left.Inner * Right.Inner);
  end "*";

  function "/" (Left, Right: Item) return Item is
  begin
    return (Inner => Left.Inner / Right.Inner);
  end "/";

  function "*" (Left: Item; Right: Real'Base) return Item is
  begin
    return (Inner => Left.Inner * Right);
  end "*";

  function "*" (Left: Real'Base; Right: Item) return Item is
  begin
    return (Inner => Left * Right.Inner);
  end "*";

  function "/" (Left: Item ; Right: Real'Base) return Item is
  begin
    return (Inner => Left.Inner / Right);
  end "/";

  function "/" (Left: Real'Base; Right: Item) return Item is
  begin
    return (Inner => Left / Right.Inner);
  end "/";

  function "**" (Base: Item; Exponent: Whole) return Item is
  begin
    return (Inner => Base.Inner ** Exponent);
  end "**";

  function "**" (Base: Item; Exponent: Rational) return Item is
  begin
    return (Inner => Base.Inner ** Exponent);
  end "**";

  function "**" (Base: Item; Exponent: Real'Base) return Item is
  begin
    return (Inner => Base.Inner ** Exponent);
  end "**";

  function "<" (Left, Right: Item) return Boolean is
  begin
    return Left.Inner < Right.Inner;
  end "<";

  function "<=" (Left, Right: Item) return Boolean is
  begin
    return Left.Inner <= Right.Inner;
  end "<=";

  function ">=" (Left, Right: Item) return Boolean is
  begin
    return Left.Inner >= Right.Inner;
  end ">=";

  function ">" (Left, Right: Item) return Boolean is
  begin
    return Left.Inner > Right.Inner;
  end ">";

  -- Mathematics

  function Sqrt (X: Item) return Item is
  begin
    return (Inner => Sqrt (X.Inner));
  end Sqrt;

  function Cbrt (X: Item) return Item is
  begin
    return (Inner => Cbrt (X.Inner));
  end Cbrt;

  function Log (X: Item) return Item is
  begin
    return (Inner => Log (X.Inner));
  end Log;

  function Log (X, Base: Item) return Item is
  begin
    return (Inner => Log (X.Inner, Base.Inner));
  end Log;

  function Exp (X: Item) return Item is
  begin
    return (Inner => Exp (X.Inner));
  end Exp;

  function Sin (X: Item) return Item is
  begin
    return (Inner => Sin (X.Inner));
  end Sin;

  function Sin (X, Cycle: Item) return Item is
  begin
    return (Inner => Sin (X.Inner, Cycle.Inner));
  end Sin;

  function Cos (X: Item) return Item is
  begin
    return (Inner => Cos (X.Inner));
  end Cos;

  function Cos (X, Cycle: Item) return Item is
  begin
    return (Inner => Cos (X.Inner, Cycle.Inner));
  end Cos;

  function Tan (X: Item) return Item is
  begin
    return (Inner => Tan (X.Inner));
  end Tan;

  function Tan (X, Cycle: Item) return Item is
  begin
    return (Inner => Tan (X.Inner, Cycle.Inner));
  end Tan;

  function Cot (X: Item) return Item is
  begin
    return (Inner => Cot (X.Inner));
  end Cot;

  function Cot (X, Cycle: Item) return Item is
  begin
    return (Inner => Cot (X.Inner, Cycle.Inner));
  end Cot;

  function Arcsin (X: Item) return Item is
  begin
    return (Inner => Arcsin (X.Inner));
  end Arcsin;

  function Arcsin (X, Cycle: Item) return Item is
  begin
    return (Inner => Arcsin (X.Inner, Cycle.Inner));
  end Arcsin;

  function Arccos (X: Item) return Item is
  begin
    return (Inner => Arccos (X.Inner));
  end Arccos;

  function Arccos (X, Cycle: Item) return Item is
  begin
    return (Inner => Arccos (X.Inner, Cycle.Inner));
  end Arccos;

  function Arctan (Y: Item;
                   X: Item := One) return Item is
  begin
    return (Inner => Arctan (Y.Inner, X.Inner));
  end Arctan;

  function Arctan (Y    : Item;
                   X    : Item := One;
                   Cycle: Item) return Item is
  begin
    return (Inner => Arctan (Y.Inner, X.Inner, Cycle.Inner));
  end Arctan;

  function Arccot (X: Item;
                   Y: Item := One) return Item is
  begin
    return (Inner => Arccot (X.Inner, Y.Inner));
  end Arccot;

  function Arccot (X    : Item;
                   Y    : Item := One;
                   Cycle: Item) return Item is
  begin
    return (Inner => Arccot (X.Inner, Y.Inner, Cycle.Inner));
  end Arccot;

  function Sinh (X: Item) return Item is
  begin
    return (Inner => Sinh (X.Inner));
  end Sinh;

  function Cosh (X: Item) return Item is
  begin
    return (Inner => Cosh (X.Inner));
  end Cosh;

  function Tanh (X: Item) return Item is
  begin
    return (Inner => Tanh (X.Inner));
  end Tanh;

  function Coth (X: Item) return Item is
  begin
    return (Inner => Coth (X.Inner));
  end Coth;

  function Arcsinh (X: Item) return Item is
  begin
    return (Inner => Arcsinh (X.Inner));
  end Arcsinh;

  function Arccosh (X: Item) return Item is
  begin
    return (Inner => Arccosh (X.Inner));
  end Arccosh;

  function Arctanh (X: Item) return Item is
  begin
    return (Inner => Arctanh (X.Inner));
  end Arctanh;

  function Arccoth (X: Item) return Item is
  begin
    return (Inner => Arccoth (X.Inner));
  end Arccoth;

end Constrained_Unchecked_SI;
