------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2003 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

with Ada.Characters.Latin_1, Ada.Strings.Maps;
use  Ada.Characters.Latin_1, Ada.Strings.Maps;

package IO_Customization is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      28 February 2003
  --====================================================================
  -- This package can be used to customize the way unit symbols are
  -- read and written.
  -- For both the product sign and the degree symbol, a set of
  -- characters are given that are all recognized upon input.
  -- For output, the default character is chosen.
  --
  -- Reason: The PC character set does not match the Latin 1 set,
  -- PC's degree symbol '' is at the position 248, i.e. Latin 1's ''.
  --
  -- Examples:
  -- 42.0*N*m and 42.0Nn will both be read. With the default specified
  -- below, the second form will be written.
  --
  -- [Should also Superscript_Two/Three   and Fraction_One_Quarter/
  -- One_Half/Three_Quarters    be allowed to produce output like
  -- s, kg/m and cm^ instead of s^2, kg/m^3, cm^(1/2)?]
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  28.02.2003
  --====================================================================

  Multiply: constant Character_Set := To_Set (Asterisk  &   --  42  *
                                              Middle_Dot);  -- 183  
  Default_Multiply: constant Character := Middle_Dot;

  Degree: constant Character_Set := To_Set (Degree_Sign        &   -- 176  
                                            LC_O_Oblique_Stroke);  -- 248  
  Default_Degree: constant Character := Degree_Sign;

end IO_Customization;
