------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception, if other files instantiate generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License. This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

generic
package Unconstrained_Checked_SI.Generic_Natural_Constants is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   0.0
  -- Date      3 September 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    0.0  03.09.2002 Only sketched
  --====================================================================

  -- Speed of Light in Vacuum

  c: constant Item := 2.997_924_58E+8*Meter/Second;  -- exact (by definition)

  -- Permittivity and Permeability of Vacuum

  Mu_0 : constant Item := 4.0E-7*Pi*Kilogram*Meter/(Second*Ampere)**2;  -- exact
  Eps_0: constant Item := 1.0/(Mu_0*c**2);  -- c = 1/Sqrt(Eps_0*Mu_0)

end Unconstrained_Checked_SI.Generic_Natural_Constants;
