------------------------------------------------------------------------------
-- Checked and Unchecked Computation with SI Units
-- Copyright (C) 2002 Christoph Karl Walter Grein
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
-- As a special exception,  if other files  instantiate  generics from this
-- unit, or you link this unit with other files to produce an executable,
-- this unit does not by itself cause the resulting executable to be
-- covered by the GNU General Public License.  This exception does not
-- however invalidate any other reasons why the executable file might be
-- covered by the GNU Public License.
--
-- Author's homepage and email address:
--   http://home.T-Online.de/home/Christ-Usch.Grein/
--   Christ-Usch.Grein@T-Online.de
------------------------------------------------------------------------------

package body Unconstrained_Unchecked_SI.Generic_Vector_Space is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.2
  -- Date      17 November 2002
  --====================================================================
  --
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  17.09.2002
  --  C.G.    1.1  08.11.2002 Added Det, Inv, Tran
  --  C.G.    1.2  17.11.2002 Use Real'Base
  --====================================================================

  function "abs" (Right: Vector) return Item is
  begin
    return Sqrt (Inner (Right, Right));
  end "abs";

  function "+" (Right: Vector) return Vector is
  begin
    return Right;
  end "+";

  function "-" (Right: Vector) return Vector is
  begin
    return (-Right (1), -Right (2), -Right (3));
  end "-";

  function "+" (Left, Right: Vector) return Vector is
  begin
    return (Left (1) + Right (1), Left (2) + Right (2), Left (3) + Right (3));
  end "+";

  function "-" (Left, Right: Vector) return Vector is
  begin
    return (Left (1) - Right (1), Left (2) - Right (2), Left (3) - Right (3));
  end "-";

  function "*" (Left: Real'Base; Right: Vector) return Vector is
  begin
    return (Left * Right (1), Left * Right (2), Left * Right (3));
  end "*";

  function "*" (Left: Item; Right: Vector) return Vector is
  begin
    return (Left * Right (1), Left * Right (2), Left * Right (3));
  end "*";

  function "*" (Left: Vector; Right: Real'Base) return Vector is
  begin
    return Right * Left;
  end "*";

  function "*" (Left: Vector; Right: Item) return Vector is
  begin
    return Right * Left;
  end "*";

  function "/" (Left: Vector; Right: Real'Base) return Vector is
  begin
    return (Left (1) / Right, Left (2) / Right, Left (3) / Right);
  end "/";

  function "/" (Left: Vector; Right: Item) return Vector is
  begin
    return (Left (1) / Right, Left (2) / Right, Left (3) / Right);
  end "/";

  function Inner (Left, Right: Vector) return Item is
  begin
    return Left (1) * Right (1) + Left (2) * Right (2) + Left (3) * Right (3);
  end Inner;

  function Cross (Left, Right: Vector) return Vector is
  begin
    return (Left (2) * Right (3) - Left (3) * Right (2),
            Left (3) * Right (1) - Left (1) * Right (3),
            Left (1) * Right (2) - Left (2) * Right (1));
  end Cross;

  function "+" (Right: Matrix) return Matrix is
  begin
    return Right;
  end "+";

  function "-" (Right: Matrix) return Matrix is
  begin
    return ((-Right (1, 1), -Right (1, 2), -Right (1, 3)),
            (-Right (2, 1), -Right (2, 2), -Right (2, 3)),
            (-Right (3, 1), -Right (3, 2), -Right (3, 3)));
  end "-";

  function "+" (Left, Right: Matrix) return Matrix is
  begin
    return ((Left (1, 1) + Right (1, 1), Left (1, 2) + Right (1, 2), Left (1, 3) + Right (1, 3)),
            (Left (2, 1) + Right (2, 1), Left (2, 2) + Right (2, 2), Left (2, 3) + Right (2, 3)),
            (Left (3, 1) + Right (3, 1), Left (3, 2) + Right (3, 2), Left (3, 3) + Right (3, 3)));
  end "+";

  function "-" (Left, Right: Matrix) return Matrix is
  begin
    return ((Left (1, 1) - Right (1, 1), Left (1, 2) - Right (1, 2), Left (1, 3) - Right (1, 3)),
            (Left (2, 1) - Right (2, 1), Left (2, 2) - Right (2, 2), Left (2, 3) - Right (2, 3)),
            (Left (3, 1) - Right (3, 1), Left (3, 2) - Right (3, 2), Left (3, 3) - Right (3, 3)));
  end "-";

  function "*" (Left: Real'Base; Right: Matrix) return Matrix is
  begin
    return ((Left * Right (1, 1), Left * Right (1, 2), Left * Right (1, 3)),
            (Left * Right (2, 1), Left * Right (2, 2), Left * Right (2, 3)),
            (Left * Right (3, 1), Left * Right (3, 2), Left * Right (3, 3)));
  end "*";

  function "*" (Left: Matrix; Right: Real'Base) return Matrix is
  begin
    return ((Left (1, 1) * Right, Left (1, 2) * Right, Left (1, 3) * Right),
            (Left (2, 1) * Right, Left (2, 2) * Right, Left (2, 3) * Right),
            (Left (3, 1) * Right, Left (3, 2) * Right, Left (3, 3) * Right));
  end "*";

  function "/" (Left: Matrix; Right: Real'Base) return Matrix is
  begin
    return ((Left (1, 1) / Right, Left (1, 2) / Right, Left (1, 3) / Right),
            (Left (2, 1) / Right, Left (2, 2) / Right, Left (2, 3) / Right),
            (Left (3, 1) / Right, Left (3, 2) / Right, Left (3, 3) / Right));
  end "/";

  function "*" (Left: Item; Right: Matrix) return Matrix is
  begin
    return ((Left * Right (1, 1), Left * Right (1, 2), Left * Right (1, 3)),
            (Left * Right (2, 1), Left * Right (2, 2), Left * Right (2, 3)),
            (Left * Right (3, 1), Left * Right (3, 2), Left * Right (3, 3)));
  end "*";

  function "*" (Left: Matrix; Right: Item) return Matrix is
  begin
    return ((Left (1, 1) * Right, Left (1, 2) * Right, Left (1, 3) * Right),
            (Left (2, 1) * Right, Left (2, 2) * Right, Left (2, 3) * Right),
            (Left (3, 1) * Right, Left (3, 2) * Right, Left (3, 3) * Right));
  end "*";

  function "/" (Left: Matrix; Right: Item) return Matrix is
  begin
    return ((Left (1, 1) / Right, Left (1, 2) / Right, Left (1, 3) / Right),
            (Left (2, 1) / Right, Left (2, 2) / Right, Left (2, 3) / Right),
            (Left (3, 1) / Right, Left (3, 2) / Right, Left (3, 3) / Right));
  end "/";

  function "*" (Left, Right: Matrix) return Matrix is
  begin
    return ((Left (1, 1) * Right (1, 1) + Left (1, 2) * Right (2, 1) + Left (1, 3) * Right (3, 1),
             Left (1, 1) * Right (1, 2) + Left (1, 2) * Right (2, 2) + Left (1, 3) * Right (3, 2),
             Left (1, 1) * Right (1, 3) + Left (1, 2) * Right (2, 3) + Left (1, 3) * Right (3, 3)),
            (Left (2, 1) * Right (1, 1) + Left (2, 2) * Right (2, 1) + Left (2, 3) * Right (3, 1),
             Left (2, 1) * Right (1, 2) + Left (2, 2) * Right (2, 2) + Left (2, 3) * Right (3, 2),
             Left (2, 1) * Right (1, 3) + Left (2, 2) * Right (2, 3) + Left (2, 3) * Right (3, 3)),
            (Left (3, 1) * Right (1, 1) + Left (3, 2) * Right (2, 1) + Left (3, 3) * Right (3, 1),
             Left (3, 1) * Right (1, 2) + Left (3, 2) * Right (2, 2) + Left (3, 3) * Right (3, 2),
             Left (3, 1) * Right (1, 3) + Left (3, 2) * Right (2, 3) + Left (3, 3) * Right (3, 3)));
  end "*";

  function Det (M: Matrix) return Item is
    -- Sarrus' rule
  begin
    return + M (1, 1) * M (2, 2) * M (3, 3)
           + M (1, 2) * M (2, 3) * M (3, 1)
           + M (1, 3) * M (2, 1) * M (3, 2)
           - M (1, 3) * M (2, 2) * M (3, 1)
           - M (2, 3) * M (3, 2) * M (1, 1)
           - M (3, 3) * M (1, 2) * M (2, 1);
  end Det;

  function Inv (M: Matrix) return Matrix is
    function Det (A11, A12,
                  A21, A22: Item) return Item is
    begin
      return A11*A22 - A12*A21;
    end Det;
    pragma Inline (Det);
    -- adjoints
    M11: constant Item := + Det (M (2, 2), M (2, 3),
                                 M (3, 2), M (3, 3));
    M12: constant Item := - Det (M (2, 1), M (2, 3),
                                 M (3, 1), M (3, 3));
    M13: constant Item := + Det (M (2, 1), M (2, 2),
                                 M (3, 1), M (3, 2));
    M21: constant Item := - Det (M (1, 2), M (1, 3),
                                 M (3, 2), M (3, 3));
    M22: constant Item := + Det (M (1, 1), M (1, 3),
                                 M (3, 1), M (3, 3));
    M23: constant Item := - Det (M (1, 1), M (1, 2),
                                 M (3, 1), M (3, 2));
    M31: constant Item := + Det (M (1, 2), M (1, 3),
                                 M (2, 2), M (2, 3));
    M32: constant Item := - Det (M (1, 1), M (1, 3),
                                 M (2, 1), M (2, 3));
    M33: constant Item := + Det (M (1, 1), M (1, 2),
                                 M (2, 1), M (2, 2));
    AdjT: constant Matrix := ((M11, M21, M31),
                              (M12, M22, M32),
                              (M13, M23, M33));
  begin
    return AdjT / Det (M);
  end Inv;

  function Tran (M: Matrix) return Matrix is
  begin
    return ((M (1, 1), M (2, 1), M (3, 1)),
            (M (1, 2), M (2, 2), M (3, 2)),
            (M (1, 3), M (2, 3), M (3, 3)));
  end Tran;

  function "*" (Left: Matrix; Right: Vector) return Vector is
  begin
    return (Left (1, 1) * Right (1) + Left (1, 2) * Right (2) + Left (1, 3) * Right (3),
            Left (2, 1) * Right (1) + Left (2, 2) * Right (2) + Left (2, 3) * Right (3),
            Left (3, 1) * Right (1) + Left (3, 2) * Right (2) + Left (3, 3) * Right (3));
  end "*";

  function "*" (Left: Vector; Right: Matrix) return Vector is
  begin
    return (Left (1) * Right (1, 1) + Left (2) * Right (2, 1) + Left (3) * Right (3, 1),
            Left (1) * Right (1, 2) + Left (2) * Right (2, 2) + Left (3) * Right (3, 2),
            Left (1) * Right (1, 3) + Left (2) * Right (2, 3) + Left (3) * Right (3, 3));
  end "*";

end Unconstrained_Unchecked_SI.Generic_Vector_Space;
