--                              -*- Mode: Ada -*-
-- Filename        : math-generics-arrays-matrix_vector_operations.adb
-- Description     : Mixed matrix vector operations
-- Author          : J. Schrer
-- Created On      : Wed May 21 19:06:05 1997
-- Last Modified By: 
-- Last Modified On: Wed May 21 19:07:27 1997
-- Update Count    : 1
-- Status          :

package body Math.Generics.Arrays.Matrix_Vector_Operations is
  -----------------------------------------------------------------------------
  function "*" (Left : in Matrix; Right : in Vector) return Vector is
    
    R     : Vector (Left'range(1));
    E     : Element;
    Offset: Integer;
  begin
    if    Left'Length(2) /= Right'Length then
      raise Matrix_Vector_Dimension_Error;
    elsif Left'First(2) = Right'First then
      for I in R'range loop
        E := Zero;
        for J in Right'range loop
          E := E + Left(I,J) * Right(J);
        end loop;
        R(I) := E;
      end loop;
    else
      Offset := Integer(Right'First) - Integer(Left'First(2));
      for I in R'range loop
        E := Zero;
        for J in Left'range(2) loop
          E := E + Left(I,J) * Right(Index(Integer(J) + Offset));
        end loop;
        R(I) := E;
      end loop;
    end if;
    return R;
  end "*";
  -----------------------------------------------------------------------------
  function "*" (Left : in Row_Vector; Right : in Matrix) return Row_Vector is
    
    R     : Row_Vector (Right'range(2));
    E     : Element;
    Offset: Integer;
  begin
    if    Left'Length /= Right'Length(1) then
      raise Matrix_Vector_Dimension_Error;
    elsif Left'First = Right'First(1) then
      for I in R'range loop
        E := Zero;
        for J in Left'range loop
          E := E + Left(J) * Right(J,I);
        end loop;
        R(I) := E;
      end loop;
    else
      Offset := Integer(Left'First) - Integer(Right'First(1));
      for I in R'range loop
        E := Zero;
        for J in Right'range(1) loop
          E := E + Left(Index(Integer(J) + Offset)) * Right(J,I);
        end loop;
        R(I) := E;
      end loop;
    end if;
    return R;
  end "*";
  -----------------------------------------------------------------------------
  function "*" (Left : in Vector; Right : in Row_Vector) return Matrix is 
    -- dyadic_product    
    R : Matrix (Left'range, Right'range);
  begin
    for I in R'range(1) loop
      for J in R'range(2) loop
        R(I,J) := Left(I) * Right(J);
      end loop;
    end loop;
    return R;
  end "*";
  -----------------------------------------------------------------------------
  function Copy_Row (Outoff        : in Matrix;
                     From_First,
                     To_Last,
                     Of_Row        : in Index;
                     Multiply_With : in Element := One) return Row_Vector is

    Result : Row_Vector (From_First .. To_Last);
  begin
    if Multiply_With = One then
      for I in Result'range loop
        Result(I) := Outoff(Of_Row,I);
      end loop;
    else
      for I in Result'range loop
        Result(I) := Multiply_With * Outoff(Of_Row,I);
      end loop;
    end if;
    return Result;
  end Copy_Row;
  -----------------------------------------------------------------------------
  function Copy_Col (Outoff        : in Matrix;
                     From_First,
                     To_Last,
                     Of_Col        : in Index;
                     Multiply_With : in Element := One) return Vector is

    Result : Vector (From_First .. To_Last);
  begin
    if Multiply_With = One then
      for I in Result'range loop
        Result(I) := Outoff(I,Of_Col);
      end loop;
    else
      for I in Result'range loop
        Result(I) := Multiply_With * Outoff(I,Of_Col);
      end loop;
    end if;
    return Result;
  end Copy_Col;
  -----------------------------------------------------------------------------
  function Copy_Row (Outoff        : in Matrix;
                     Of_Row        : in Index;
                     Multiply_With : in Element := One) return Row_Vector is

    Result : Row_Vector (Outoff'range(2));
  begin
    if Multiply_With = One then
      for I in Result'range loop
        Result(I) := Outoff(Of_Row,I);
      end loop;
    else
      for I in Result'range loop
        Result(I) := Multiply_With * Outoff(Of_Row,I);
      end loop;
    end if;
    return Result;
  end Copy_Row;
  -----------------------------------------------------------------------------
  function Copy_Col (Outoff        : in Matrix;
                     Of_Col        : in Index;
                     Multiply_With : in Element := One) return Vector is

    Result : Vector (Outoff'range(1));
  begin
    if Multiply_With = One then
      for I in Result'range loop
        Result(I) := Outoff(I,Of_Col);
      end loop;
    else
      for I in Result'range loop
        Result(I) := Multiply_With * Outoff(I,Of_Col);
      end loop;
    end if;
    return Result;
  end Copy_Col;
  -----------------------------------------------------------------------------
end Math.Generics.Arrays.Matrix_Vector_Operations;
