--                              -*- Mode: Ada -*-
-- Filename        : math-generics-arrays-types.ads
-- Description     : Declaration of array types
-- Author          : J. Schrer
-- Created On      : Wed May 21 19:09:10 1997
-- Last Modified By:
-- Last Modified On: Wed May 28 10:39:08 1997
-- Update Count    : 7
-- Status          :

generic
  type Element is private;
  type Index   is range <>;

  Zero : in Element;

package Math.Generics.Arrays.Types is

  type Vector          is array (Index range <>) of aliased Element;
  
  type    Row_Vector   is new Vector;
  subtype Col_Vector   is Vector;

  type Matrix          is array (Index range <>,
                                 Index range <>) of aliased Element;

  type Diagonal_Matrix is array (Index range <>) of aliased Element;

  type Vector_Access           is access all Vector;
  type Matrix_Access           is access all Matrix;
  type Diagonal_Matrix_Access  is access all Diagonal_Matrix;


  Symmetric_Matrix_Allocation_Error : exception;

  type Symmetric_Matrix is array (Index range <>) of Vector_Access;

  type Symmetric_Matrix_Access is access all Symmetric_Matrix;

  --  | A(1,1)   A(1,2)   A(1,3)   ...   A(1,N-1)     A(1,N)   |
  --  |          A(2,2)   A(2,3)   ...   A(2,N-1)     A(2,N)   |
  --  |            ...             ...                         |
  --  |            ...             ...   A(N-1,N-1)   A(N-1,N) |
  --  |            ...             ...                A(N,N)   |
  --
  --  A(I)(J),	 I = 1 .. N, J = I .. N

  function  Allocate_Symmetric_Matrix(Order : in Index)
            return Symmetric_Matrix;

  procedure Allocate (The_Array      : in out Symmetric_Matrix);

  procedure Allocate (The_Array      : in out Vector_Access;
                      Last           : in     Index;
                      First          : in     Index := 1);

  procedure Allocate (The_Array      : in out Matrix_Access;
                      Last1,  Last2  : in     Index;
                      First1, First2 : in     Index := 1);

  procedure Allocate (The_Array      : in out Diagonal_Matrix_Access;
                      Last           : in     Index;
                      First          : in     Index := 1);

  procedure Allocate (The_Array      : in out Symmetric_Matrix_Access;
                      Last           : in     Index;
                      First          : in     Index := 1);


  procedure Deallocate (The_Array : in out Vector_Access);
  procedure Deallocate (The_Array : in out Matrix_Access);
  procedure Deallocate (The_Array : in out Diagonal_Matrix_Access);
  procedure Deallocate (The_Array : in out Symmetric_Matrix);
  procedure Deallocate (The_Array : in out Symmetric_Matrix_Access);

end Math.Generics.Arrays.Types;

