--                              -*- Mode: Ada -*-
-- Filename        : math-generics-arrays-vector_operations.adb
-- Description     : generic vector operations
-- Author          : J. Schrer
-- Created On      : Wed May 21 19:12:23 1997
-- Last Modified By: 
-- Last Modified On: Wed May 21 19:13:46 1997
-- Update Count    : 1
-- Status          :

package body Math.Generics.Arrays.Vector_Operations is
  ---------------------------------------------------------------------------
  function "+" (Left, Right : in Vector) return Vector is
    
    R : Vector (Left'range);
  begin
    if    Left'Length /= Right'Length then
      raise Vector_Length_Error;
    elsif Left'First = Right'First then
      for I in R'range loop
        R(I) := Left(I) + Right(I);
      end loop;
    else
      R := Right;
      for I in R'range loop
        R(I) := Left(I) + R(I);
      end loop;
    end if;
    return R;
  end "+";
  ---------------------------------------------------------------------------
  function "-" (Right : in Vector) return Vector is
    
    R : Vector(Right'range);
  begin
    for I in R'range loop
      R(I) := - Right(I);
    end loop;
    return R;
  end "-";
  ---------------------------------------------------------------------------
  function "-" (Left, Right : in Vector) return Vector is
    
    R : Vector (Left'range);
  begin
    if    Left'Length /= Right'Length then
      raise Vector_Length_Error;
    elsif Left'First = Right'First then
      for I in R'range loop
        R(I) := Left(I) - Right(I);
      end loop;
    else
      R := Right;
      for I in R'range loop
        R(I) := Left(I) - R(I);
      end loop;
    end if;
    return R;
  end "-";
  ---------------------------------------------------------------------------
  function "*" (Left : in Element; Right : in Vector) return Vector is
    
    R : Vector (Right'range);
  begin
    for I in R'range loop
      R(I) := Left * Right(I);
    end loop;
    return R;
  end "*";
  ---------------------------------------------------------------------------
  function "*" (Left : in Vector; Right : in Element) return Vector is
    
    R : Vector (Left'range);
  begin
    for I in R'range loop
      R(I) := Left(I) * Right;
    end loop;
    return R;
  end "*";
  ---------------------------------------------------------------------------
  function "/" (Left : in Vector; Right : in Element) return Vector is
    
    R : Vector (Left'range);
  begin
    for I in R'range loop
      R(I) := Left(I) / Right;
    end loop;
    return R;
  end "/";
  ---------------------------------------------------------------------------
  function "*" (Left : in Row_Vector; Right : in Vector) return Element is
    
    R     : Element := Zero;
    Offset: Integer;
  begin
    if    Left'Length /= Right'Length then
      raise Vector_Length_Error;
    elsif Left'First = Right'First then
      for I in Left'range loop
        R := R + Left(I) * Right(I);
      end loop;
    else
      Offset := Integer(Right'First) - Integer(Left'First);
      for I in Left'range loop
        R := R + Left(I) * Right(Index(Integer(I) + Offset));
      end loop;
    end if;
    return R;
  end "*";
  ---------------------------------------------------------------------------
  function "*" (Left, Right : in Vector) return Vector is
    
    R : Vector (Left'range);
  begin
    if Left'Length /= 3 or else Right'Length /= 3 then
      raise Vector_Length_Error;
    end if;
    R(R'First)     := Left(Left'First + 1) * Right(Right'Last)      -
                      Left(Left'Last)      * Right(Right'First + 1);
    R(R'First + 1) := Left(Left'Last)      * Right(Right'First)     -
                      Left(Left'First)     * Right(Right'Last);
    R(R'Last)      := Left(Left'First)     * Right(Right'First + 1) -
                      Left(Left'First + 1) * Right(Right'First);
    return R;
  end "*";
  ---------------------------------------------------------------------------
  function "abs" (Right : in Vector) return Element is
    
    R : Element := Zero;
  begin
    for I in Right'range loop
      R := R + Right(I) * Right(I);
    end loop;
    return Sqrt(R);
  end "abs";
  ---------------------------------------------------------------------------
  function Elementwise_Multiplication
          (Left, Right : in Vector) return Vector is
    
    R : Vector (Left'range);
  begin
    if    Left'Length /= Right'Length then
      raise Vector_Length_Error;
    elsif Left'First = Right'First then
      for I in R'range loop
        R(I) := Left(I) * Right(I);
      end loop;
    else
      R := Right;
      for I in R'range loop
        R(I) := Left(I) * R(I);
      end loop;
    end if;
    return R;
  end Elementwise_Multiplication;
  ---------------------------------------------------------------------------
  procedure Zero_Vector (The_Vector : out Vector) is
  begin
    The_Vector := Vector'(The_Vector'range => Zero);
  end Zero_Vector;
  ---------------------------------------------------------------------------
  function  Zero_Vector (Of_Order : in Index) return Vector is
    
    The_Vector : constant Vector (1 .. Of_Order) := (others => Zero);
  begin
    return The_Vector;
  end Zero_Vector;
  ---------------------------------------------------------------------------
end Math.Generics.Arrays.Vector_Operations;
