--                              -*- Mode: Ada -*-
-- Filename        : math-generics-arrays-vector_operations.ads
-- Description     : generic vector operations
-- Author          : J. Schrer
-- Created On      : Wed May 21 19:13:10 1997
-- Last Modified By:
-- Last Modified On: Thu May 22 10:35:38 1997
-- Update Count    : 2
-- Status          :

generic
  type Element    is private;
  type Index      is range <>;
  type Vector     is array (Index range <>) of Element;
  type Row_Vector is new Vector;

  Zero : in Element;

  with function "+" (Left, Right : in Element) return Element is <>;
  with function "-" (Right       : in Element) return Element is <>;
  with function "-" (Left, Right : in Element) return Element is <>;
  with function "*" (Left, Right : in Element) return Element is <>;
  with function "/" (Left, Right : in Element) return Element is <>;
  with function Sqrt(X           : in Element) return Element is <>; 
                -- For "Abs"

package Math.Generics.Arrays.Vector_Operations is

  Vector_Length_Error : exception;
  
  -- Normal operators; raise Vector_Lenght_Error if left and right
  -- are incompatible for the operation.
  
  function "+" (Left, Right : in Vector) return Vector;
  function "-" (Right       : in Vector) return Vector;
  function "-" (Left, Right : in Vector) return Vector;

  function "*" (Left : in Element; Right : in Vector)  return Vector;
  function "*" (Left : in Vector;  Right : in Element) return Vector;
  function "/" (Left : in Vector;  Right : in Element) return Vector;

  function "*" (Left : in Row_Vector; Right : in Vector) return Element;
  
  -- Scalar product: R = sum(left(i) * right(i))
  
  function "*" (Left, Right : in Vector) return Vector;
  
  -- Cross product: R = left x rigth.
  -- Vector_Length_Error is raised if left'length /= 3 or right'length /= 3.

  function "abs"(Right : in Vector) return Element;
  
  -- Length of the vector = sqrt(sum(right(i)))

  function Elementwise_Multiplication(Left, Right : in Vector) return Vector;
  
  -- R(i) = left(i) * right(i)

  procedure Zero_Vector (The_Vector : out Vector);
  
  -- The_Vector := (others => Zero);

  function  Zero_Vector (Of_Order : in Index) return Vector;

  -- The_Vector := (1 .. Of_Order => Zero);
  
end Math.Generics.Arrays.Vector_Operations;
