--                    -*- Mode: Ada -*-
-- Filename        : opengl-coordinate_system.adb
-- Description     : Package for the definition of a coordinate_system. 
-- Author          : J. Schrer
-- Created On      : 24. Mar. 1998
-- Last Modified By:
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Ada.Strings.Fixed;
with Opengl.Types;
with Opengl.Fonts;
--with Opengl.Models.Strings;
with Opengl.Float_32.Arrays.Vector_Operations;
with Opengl.Float_32.Arrays.Types;
--with Opengl.Glu;

package body Opengl.Coordinate_System is

  use Opengl.Float_32.Arrays.Vector_Operations;
      -- to make vector-operators directly visible.
  use type Opengl.Float_32.Arrays.Types.Vector;

  The_Font : Fonts.Font_Type;
  ----------------------------------------------------------------------------
  procedure Rotate_Back(Attitude : in Types.Vector_3d) is
    Axis : Types.Vector_3d := (X .. Z => 0.0);
  begin  
    Axis(Models.Rotation_3) := 1.0;
    Models.Rotate(The_Angle => - Attitude(Models.Rotation_3),
                  The_Axis  => Axis);
    Axis := (X .. Z => 0.0);
    Axis(Models.Rotation_2) := 1.0;
    Models.Rotate(The_Angle => - Attitude(Models.Rotation_2),
                  The_Axis  => Axis);
    Axis := (X .. Z => 0.0);
    Axis(Models.Rotation_1) := 1.0;
    Models.Rotate(The_Angle => - Attitude(Models.Rotation_1),
                  The_Axis  => Axis);
  end Rotate_Back;    
  ----------------------------------------------------------------------------
  procedure Render(The : in out Model) is
    -------------------------------------------
    procedure Render(Axis  : in Types.Vector_3d;
                     Name  : in Character) is

      Font_Size : constant Gl.Glfloat := 5.0;
      Position  : constant Types.Vector_3d :=
        Axis * (1.0 + Font_Size / 2.0 / (The.Size_Scaling + The.Axis_Width));
      Text      : constant String := Name & The.Name;
    begin
      Gl.Glbegin(Gl.Gl_Lines);
      Gl.Glvertex3f(X => 0.0, Y => 0.0, Z => 0.0);
      Gl.Glvertex3f(X => Axis(X), Y => Axis(Y), Z => Axis(Z));
      Gl.Glend;
      
      Gl.Glpushmatrix;
      Models.Translate(Position);
      Rotate_Back(The.Attitude);
      
      Gl.Gllistbase(Gl.Gluint(Fonts.List_Index_Of(The_Font)));
      Gl.Glscalef(X => Font_Size,
                  Y => Font_Size,
                  Z => 1.0);
	    Gl.Glcalllists(N      => Text'Length,
                     C_Type => Gl.Gl_Unsigned_Byte,
                     Lists  => Text'Address);
      Gl.Glpopmatrix;
    end Render;
    --------------------------------------------------------------------------
    Names  : constant String(X .. Z) := "XYZ";                    
    Length : constant Gl.Glfloat := The.Size_Scaling + The.Axis_Width;
    Axes   : constant array(X .. Z) of Types.Vector_3d :=
							        (X => (X => Length, Y => 0.0,    Z => 0.0),
     					 	       Y => (X => 0.0,    Y => Length, Z => 0.0),
					     	       Z => (X => 0.0,    Y => 0.0,    Z => Length));
    Width  : aliased Gl.Glfloat := 1.0;                    
  begin
    if not Fonts.Is_Initialized(The_Font) then
      The_Font := Fonts.Font;
    end if;

    Models.Initialize_Render(The);
        
    Gl.Glgetfloatv(Pname  => Gl.Gl_Line_Width, 
                   Params => Width'Unchecked_Access);
    Gl.Gllinewidth(The.Axis_Width);

    Render(Axis => Axes(Models.Rotation_1), Name => Names(Models.Rotation_1));
    Render(Axis => Axes(Models.Rotation_2), Name => Names(Models.Rotation_2));
    Render(Axis => Axes(Models.Rotation_3), Name => Names(Models.Rotation_3));
    
    Gl.Gllinewidth(Width);
    Models.Finalize_Render(The);
  end Render;
  ----------------------------------------------------------------------------                                                                             
  procedure Initialize(The : in out Transformation_Sequence_Model) is
  begin
    for I in The.Coordinate_Systems'range loop
      The.Coordinate_Systems(I).Transformation := I;
    end loop;    
  end Initialize;
  ----------------------------------------------------------------------------                                                                               
  procedure Render    (The : in out Transformation_Sequence_Model) is
    -----------------------------------------------------------------------
    procedure Render_Without_Initialize_And_Finalize
      (The_Cs   : in out Model;
       Cs_Index : in     Positive) is 
      --------------------------------------------------------------------------    
      procedure Render(Axis  : in Types.Vector_3d;
                       Name  : in Character;
                       Index : in String) is
  
        Font_Size : constant Gl.Glfloat := 5.0;
        Position  : constant Types.Vector_3d :=
          Axis * (1.0 + Font_Size / 2.0 / (The_Cs.Size_Scaling + The_Cs.Axis_Width));
        Text      : constant String := Name & Index & The_Cs.Name;
      begin
        Gl.Glbegin(Gl.Gl_Lines);
        Gl.Glvertex3f(X => 0.0, Y => 0.0, Z => 0.0);
        Gl.Glvertex3f(X => Axis(X), Y => Axis(Y), Z => Axis(Z));
        Gl.Glend;
        
        Gl.Glpushmatrix;
        Models.Translate(Position);
        for I in reverse 1 .. Cs_Index loop
          Rotate_Back(The.Coordinate_Systems(I).Attitude);
        end loop;  
        Gl.Gllistbase(Gl.Gluint(Fonts.List_Index_Of(The_Font)));
        Gl.Glscalef(X => Font_Size,
                    Y => Font_Size,
                    Z => 1.0);
  	    Gl.Glcalllists(N      => Text'Length,
                       C_Type => Gl.Gl_Unsigned_Byte,
                       Lists  => Text'Address);
        Gl.Glpopmatrix;
      end Render;
      --------------------------------------------------------------------------
      function Index_Image return String is
      begin
        if The_Cs.Transformation = 0 then
          return "";
        else
          return Ada.Strings.Fixed.Trim
            (Natural'Image(The_Cs.Transformation), Ada.Strings.Both);
        end if;    
      end Index_Image;
      --------------------------------------------------------------------------
      Index  : constant String     := Index_Image;
      Names  : constant String(X .. Z) := "XYZ";                    
      Length : constant Gl.Glfloat := The_Cs.Size_Scaling + The_Cs.Axis_Width;
      Axes   : constant array(X .. Z) of Types.Vector_3d :=
		  					        (X => (X => Length, Y => 0.0,    Z => 0.0),
       					 	       Y => (X => 0.0,    Y => Length, Z => 0.0),
					      	       Z => (X => 0.0,    Y => 0.0,    Z => Length));
    begin
      if not Fonts.Is_Initialized(The_Font) then
        The_Font := Fonts.Font;
      end if;
  
      Gl.Gllinewidth(The_Cs.Axis_Width);
  
      Render(Axis  => Axes(Models.Rotation_1), 
             Name  => Names(Models.Rotation_1),
             Index => Index);
      Render(Axis  => Axes(Models.Rotation_2), 
             Name  => Names(Models.Rotation_2),
             Index => Index);
      Render(Axis  => Axes(Models.Rotation_3), 
             Name  => Names(Models.Rotation_3),
             Index => Index);
    end Render_Without_Initialize_And_Finalize;
    -------------------------------------------
  begin
    Models.Initialize_Render(The);
    for I in The.Coordinate_Systems'range loop
      Models.Initialize_Render(The.Coordinate_Systems(I));
      Render_Without_Initialize_And_Finalize(The.Coordinate_Systems(I), I);      
    end loop;    
    for I in reverse The.Coordinate_Systems'range loop
      Models.Finalize_Render(The.Coordinate_Systems(I));
    end loop;    
    Models.Finalize_Render(The);
  end Render;
  ----------------------------------------------------------------------------                                                                                 
end Opengl.Coordinate_System;
