--                    -*- Mode: Ada -*-
-- Filename        : opengl-coordinate_system.ads
-- Description     : Package for the definition of a coordinate_system. 
-- Author          : J. Schrer
-- Created On      : 24. Mar. 1998
-- Last Modified By:
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Opengl.Screen;
with Opengl.Models;
with Opengl.Colors;
with Opengl.Gl;

package Opengl.Coordinate_System is
  
  type Model is new Models.Model_3d with record
    Name           : String(1 .. 16) := (others => ' ');
    Axis_Width     : Gl.Glfloat := 1.0;
    Transformation : Natural    := 0; 
    -- If Transformation = n = 0, the texts of the coordinate axes is x,y,z.
    -- If Transformation = n > 0, the texts of the coordinate axes is xn,yn,zn.
    -- Is used to display a list of transformed coordinate-systems.
  end record;
  
  procedure Render(The : in out Model);

  
  use type Gl.Glfloat;
  
  Max_Length : constant Gl.Glfloat := 
    0.4 * Gl.Glfloat'Min(Screen.Width, Screen.Height);

  Standard : constant Model := 
    (Models.Root_Model with
     Size_Scaling   => Max_Length,  
     Color          => Colors.White,
     Position       => (X => Screen.Width  / 2.0, 
                        Y => Screen.Height / 2.0, 
                        Z => 0.0),
     Attitude       => (X => 0.0, Y => 0.0, Z => 0.0),
     Name           => (others => ' '),
     Axis_Width     => 1.0,
     Transformation => 0);
                                                           
  ----------------------------------------------------------------------------                                                             
  -- To display a coordinate-system transformation-sequence   
      
  type Model_Array is array(Positive range <>) of Model;
  
  type Transformation_Sequence_Model(Transformation_Number : Natural) is new 
       Models.Model_3d with record   
    Coordinate_Systems : Model_Array(1 .. Transformation_Number) :=
                         (others => Model'(Models.Root_Model with
                                    Size_Scaling   => Max_Length,  
                                    Color          => Colors.White,
                                    Position       => (X => 0.0, 
                                                       Y => 0.0, 
                                                       Z => 0.0),
                                    Attitude       => (X => 0.0, 
                                                       Y => 0.0, 
                                                       Z => 0.0),
                                    Name           => (others => ' '),
                                    Axis_Width     => 1.0,
                                    Transformation => 0));
  end record;
       
  procedure Initialize(The : in out Transformation_Sequence_Model);
  
  -- Assigns The.Coordinate_Systems(i).Transformation := i; 
  -- in order to number the axis-texts.
  
  procedure Render    (The : in out Transformation_Sequence_Model);            
            
end Opengl.Coordinate_System;
    
