--                    -*- Mode: Ada -*-
-- Filename        : opengl-display_lists.adb
-- Description     : Package for managing OpenGL display-list indices.
--
--                   All parts of an application using display-lists should
--                   aquire their indices by the below given function 
--                   next_list_id, in order to avoid list-id collisions.
--
-- Author          : J. Schrer
-- Created On      : 31. Mar. 1998
-- Last Modified By:
-- Last Modified On: ... Apr. 1998
-- Update Count    : 
-- Status          :

package body Opengl.Display_Lists is
  
  use type Gl.Gluint;
  use type Gl.Glboolean;
  
  Next_Free_Id : Gl.Gluint := 1000;
  ----------------------------------------------------------------------------
  function Next_List_Id(Required_Number : in Positive := 1) return Gl.Gluint is
    The : constant Gl.Gluint := Next_Free_Id;
  begin
    Next_Free_Id := Next_Free_Id + Gl.Gluint(Required_Number);
    return The;
  end Next_List_Id;
  ----------------------------------------------------------------------------
  function Lists_Used return Gl.Gluint is
  begin
    return Next_Free_Id - 1;
  end Lists_Used;
  ----------------------------------------------------------------------------
  procedure Generate(List : in out Gl.Gluint) is
  begin
    if Gl.Glislist(List) = Gl.Gl_True then
      Gl.Gldeletelists(List, C_Range => 1);
    else
      List := Next_List_Id;  
    end if;  
  end Generate;
  ----------------------------------------------------------------------------    
end Opengl.Display_Lists;
