--                    -*- Mode: Ada -*-
-- Filename        : opengl-display_lists.ads
-- Description     : Package for managing OpenGL display-list indices.
--
--                   All parts of an application using display-lists should
--                   aquire their indices by the below given function 
--                   next_list_id, in order to avoid list-id collisions.
--
-- Author          : J. Schrer
-- Created On      : 31. Mar. 1998
-- Last Modified By:
-- Last Modified On: ... Apr. 1998
-- Update Count    : 
-- Status          :

with Opengl.Gl;

package Opengl.Display_Lists is
  
  function Next_List_Id(Required_Number : in Positive := 1) return Gl.Gluint;

  -- The function returns the next free list-id and increments the internal
  -- id-counter by Required_Number for the next call to Next_List_Id.
    
  function Lists_Used return Gl.Gluint;
  
  -- Returns the number of used display-lists (1000 are reserved for OpenGL).
  
  procedure Generate(List : in out Gl.Gluint);
  
  -- If List is in use deletes it, else returns next free id.
    
end Opengl.Display_Lists;

