--                    -*- Mode: Ada -*-
-- Filename        : opengl-earth.ads
-- Description     : 
-- Author          : J. Schrer
-- Created On      : 
-- Last Modified By: 
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Ada.Calendar;

with Opengl.Models;
with Opengl.Models.Forms;
with Opengl.Models.Lighting;
with Opengl.Clip_Planes;
with Opengl.Types;
with Opengl.Screen;
with Opengl.Gl;
with Opengl.Bitmaps;
with Opengl.Coordinate_System;

package Opengl.Earth is

  type Model is new Models.Model_3d with private;

  function Time_Offset_Of  (The_Model : in Model) return Duration;
  
  procedure Set_Time_Offset(The_Model : in out Model;
                            To        : in     Duration);
                              
  -- To make the timezone-offset usable for an outer clock.

  procedure Set_Location(The_Model : in out Model;
                         Latitude, 
                         Longitude : in     Gl.Glfloat);                            
                         
  -- Set the location for the view on the earth.                       
                            
  procedure Render(The_Model : in out Model);
  
  use type Gl.Glfloat;

  function Model_With
    (Bitmap_File     : in String;
     Position        : in Types.Vector_3d :=
                         (X => Screen.Width  / 2.0,
          	              Y => Screen.Height / 2.0,
                          Z => 0.0);
     Attitude        : in Types.Vector_3d :=
                         (X => 0.0,
          	              Y => 0.0,
                          Z => 0.0);
     Scale           : in Gl.Glfloat := Screen.Height / 2.0;
     Longitude_Lines : in Natural    := 360;
     Latitude_Lines  : in Natural    := 180;
     Time_Factor     : in gl.glfloat := 1.0;
     Time_Offset     : in Duration   := 0.0;
     Sunlight        : in Boolean    := False;
     Latitude       : in Gl.Glfloat  := 0.0;
     Longitude      : in Gl.Glfloat  := 0.0) return Models.Root_Model_Access;

private

  -- Auxiliar objects --------------------------------------------------------

  type Object_Type is record
    Selected : Boolean    := False;
    Latitude,
    Longitude: Gl.Glfloat := 0.0;
    Symbol   : Models.Display_List_Model_3d;
    Name     : Models.Display_List_Model_3d;
  end record;

  type Object_Array is array(Positive range <>) of Object_Type;
  type Object_Array_Access is access Object_Array;


  Invalid_List : constant Gl.Gluint := Gl.Gluint'Last;
    
  type Model is new Models.Lighting.Real_Model with record 
    Solid_Sphere    : Models.Forms.Sphere_3d;
    Net_Sphere      : Models.Forms.Sphere_3d;
    Earth_Bitmap    : Bitmaps.Bitmap_Type;

    solid_List      : Gl.Gluint         := Invalid_List;
    net_List        : Gl.Gluint         := Invalid_List;
    
    Latitude        : Gl.Glfloat        := 0.0;
    Longitude       : Gl.Glfloat        := 0.0;                            

    Lighting_Model  : Models.Lighting.Lighting_Model;
    Sun             : Models.Lighting.Light_Source;
    Sun_Distance    : Gl.Glfloat        := Screen.Width;

    Z_Plane         : Clip_Planes.Plane := Clip_Planes.Z_Clip_Plane;

    Time            : Ada.Calendar.Time := Ada.Calendar.Clock;
    Render_Time     : Ada.Calendar.Time := Ada.Calendar.Clock;
    Time_Factor     : gl.glfloat        := 1.0;
    Time_Offset     : Duration          := 0.0;

    Sunlight        : Boolean           := False;
    Animated_Latlon : Boolean           := False;
    Animated_Time   : Boolean           := False;
    Great_Change    : Boolean           := False;
    Render_Net      : Boolean           := False;
    Render_Bitmap   : Boolean           := True;
    Render_C_sys    : Boolean           := False;

    Coord_System    : Coordinate_System.Model;
    Objects         : Object_Array_Access;
  end record;
  
end Opengl.Earth;
