--                              -*- Mode: Ada -*-
-- Filename        : opengl-logging.ads
-- Description     : Package with utilities for logging of opengl programs
-- Author          : J. Schrer
-- Created On      : 05. Jan 2000
-- Last Modified By: J. Schrer
-- Last Modified On: 
-- Update Count    : 1
-- Status          :

with Ada.Exceptions;
with Opengl.Models;

package Opengl.Logging is
  
  procedure Initialize_Log_File(Os_Info : in String := "");
  procedure Finalize_Log_File;
  
  -- The procedures are called at the beginning and the end of the main-program.
  -- Initialize_Log_File creates a file called <Ada.Command_Line.Command_Name>.log,
  -- where a call to Ada.Command_Line.Command_Name returns the name of the 
  -- main-executable.
  
  procedure put(text : in string);
  
  -- If the logging file is open writes text to it and flushes the file.
    
  procedure Write_Current_Task_Id;
  
  -- Write current task-id of main program.
  
  procedure Write_Timing_Info(The_Models   : in Models.Window_Models_Access;
                              Window_Index : in Positive := 1);
  
  -- Write timing-info of The_Models for window with index Window_Index.
    
  procedure Write_Error_File(E : in Ada.Exceptions.Exception_Occurrence);
  
  -- Creates a file with a filename derived from the current date and time like
  -- yyyymmddhhmmss.err and writes the current date and time and the messages
  -- contained in Ada.Exceptions.Exception_Information and 
  -- Ada.Exceptions.Exception_Message to it.
    
end Opengl.Logging;

