--                    -*- Mode: Ada -*-
-- Filename        : opengl-models-lighting.ads
-- Description     : Package for the definition of lighting 
--                   and models with material properties. 
-- Author          : J. Schrer
-- Created On      : 17.3.2001
-- Last Modified By:
-- Last Modified On:
-- Update Count    : 1
-- Status          :

package Opengl.Models.Lighting is

  Default_Ambient_Light : constant Types.Color_Type :=
                                   (Red   => 0.2, 
                                    Green => 0.2, 
                                    Blue  => 0.2, 
                                    Alpha => 1.0);

  Default_Diffuse_Light : constant Types.Color_Type :=
                                   (Red   => 0.8, 
                                    Green => 0.8, 
                                    Blue  => 0.8, 
                                    Alpha => 1.0);

  -- Lighting model ----------------------------------------------------------

  type Lighting_Model is record
    Initialized   : Boolean          := False;
    Ambient_Color : Types.Color_Type := Default_Ambient_Light;
    Local_Viewer  : Gl.Glboolean     := Gl.Gl_False;                                   
    Two_Side      : Gl.Glboolean     := Gl.Gl_False;                                   
--    Local_Viewer  : Gl.Glfloat       := 0.0;                                   
--    Two_Side      : Gl.Glfloat       := 0.0;                                   
--    color_control : gl.glenum        := gl.gl_single_color;
  end record;

  procedure Init(The_Model : in Lighting_Model);
  
  procedure Enable_Lighting;
  procedure Disable_Lighting;

  -- Light source model ------------------------------------------------------

  type Light_Source is new Model_3d with record
    Id                    : Gl.Glenum        := Gl.Glenum'Last;
    Initialized           : Boolean          := False;
    Ambient_Color         : Types.Color_Type := Colors.Black;
    Diffuse_Color         : Types.Color_Type := Colors.White;
    Specular_Color        : Types.Color_Type := Colors.White;
    -- position(x..z) from parent
    Spot_Direction        : Types.Vector_3d  := (0.0, 0.0, -1.0);
    Position_W            : Gl.Glfloat := 0.0;
    Spot_Exponent         : Gl.Glfloat := 0.0;
    Spot_Cutoff           : Gl.Glfloat := 180.0;
    Constant_Attenuation  : Gl.Glfloat := 1.0;
    Linear_Attenuation    : Gl.Glfloat := 0.0;
    Quadratic_Attenuation : Gl.Glfloat := 0.0;
  end record;  
  
  procedure Init   (The_Model : in out Light_Source);
  procedure Enable (The_Model : in out Light_Source);
  procedure Disable(The_Model : in out Light_Source);
  
  procedure Render (The_Model : in out Light_Source);  
  
  -- Model with material properties ------------------------------------------
  
  type Color_Index is (Ambient, Diffuse, Specular);
  type Color_Index_Array is array(Color_Index) of Natural;
  
  type Real_Model is abstract new Model_3d with record
    Initialized    : Boolean          := False;
    Face           : Gl.Glenum        := Gl.Gl_Front;
    Ambient_Color  : Types.Color_Type := Default_Ambient_Light;
    Diffuse_Color  : Types.Color_Type := Default_Diffuse_Light;
    Specular_Color : Types.Color_Type := Colors.Black;
    Emission_Color : Types.Color_Type := Colors.Black;
    Shininess      : Gl.Glfloat := 0.0;
    Color_Indexes  : Color_Index_Array := (0, 1, 1);
  end record;  

  procedure Init  (The_Model : in out Real_Model'Class);

  procedure Render(The_Model : in out Real_Model) is abstract; 
       
  procedure Initialize_Render(The_Model : in out Real_Model'Class);        

end Opengl.Models.Lighting;
