--                    -*- Mode: Ada -*-
-- Filename        : opengl-test_screen_prototyping.adb
-- Description     : Package that creates models for an opengl test-screen 
--                   (clock, timer, or new models that shall be tested).
--                   Version for small main program opengl.win_main_prototyping.
-- Author          : J. Schrer
-- Created On      : 05. Jan. 2000
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Win32, Win32.Windef, Win32.Winuser;
with Opengl.Win;

--with Ada.Real_Time;
with Ada.Strings.Unbounded;

with Ada.Strings.Unbounded;
with Utilities.Command_Line;
with Utilities.String_Vectors;

with Opengl.Command_Line;
with Opengl.Types;

with Opengl.Colors;
with Opengl.Colors.Palette;
with Opengl.Timer;
with Opengl.Clock;
with Opengl.Screen;
with Opengl.Gl;
with Opengl.Messages;
with Opengl.Float_32.Elementary_Functions;
with Opengl.Float_32.Arrays.Vector_Operations;
with Opengl.Float_32.Arrays.Types;
with Opengl.Float_32.Text_Io;
--with Opengl.Models.Strings.Labels;
--with Opengl.Models.Strings.Vertical;
--with Opengl.Models.Forms;
with Opengl.Fonts;
--with Opengl.Maps;
with Opengl.Earth;
with Opengl.Bitmaps;
with Opengl.Display_Lists;
--with Opengl.Models.Linear_Scales;
--with Opengl.Models.Linear_Dialfaces;
with Opengl.Glu;
with Opengl.Coordinate_System;
--with Opengl.Fonts;
--with Opengl.Models.Forms;
--with Opengl.Models.Led;
--with Opengl.Models.Drum_Counters;
--with Opengl.Instruments.Navaid_Symbols;
--with Opengl.Instruments.Rmi;
--with Opengl.Instruments.Altimeter;
--with Opengl.Instruments.Dme;
--with Opengl.Instruments.Inputs;
--with Opengl.Float_32.Text_Io;
--with Opengl.Models.Earth;
--with Simulator.Instruments.Efis;

package body Opengl.Test_Screen_Prototyping is

  use Opengl.Float_32.Elementary_Functions;
      -- to make sin, cos, etc directly visible.
  use Opengl.Float_32.Arrays.Vector_Operations;
  use type Opengl.Float_32.Arrays.Types.Vector;
  
  use      Opengl;
  use type Gl.Glfloat;
  use type Bitmaps.Byte;
  subtype  Glfloat   is Gl.Glfloat;
  subtype  Vector_3d is Types.Vector_3d;
	  
  function Argument(Key : in String; Default : in Integer) return Integer 
    renames Utilities.Command_Line.Argument;
  function Argument(Key : in String) return String 
    renames Utilities.Command_Line.Argument;
  function Has_Argument(Key : in String) return Boolean 
    renames Utilities.Command_Line.Has_Argument;
	----------------------------------------------------------------------------    
  function Argument(Key : in String; Default : in Gl.Glfloat) return Gl.Glfloat is
	begin
	  return Glfloat(Utilities.Command_Line.Argument(Key, Float(Default)));
	end Argument;  
	----------------------------------------------------------------------------
  function Argument(Key : in String; Default : in String) return String is
    Arg : constant String := Argument(Key);
  begin
    if Arg = "" then
      return Default;
    else
      return Arg;
    end if;
	end Argument;  
	----------------------------------------------------------------------------
  The_Filter : Bitmaps.Filter_Type := 
    (Bitmaps.Red   => (First => 0, Last => 25, Bias => 0),
     Bitmaps.Green => (First => 0, Last => 40, Bias => 0),
     Bitmaps.Blue  => (First => 0, Last => 80, Bias => 0));
     
  type Filter_Parameter_Type is (First, Last, Bias, Factor);
  
  The_Parameter : Filter_Parameter_Type    := Last;       
  The_Color     : Bitmaps.Color_Index_Type := Bitmaps.Blue;
  The_Factor    : array(Bitmaps.Color_Index_Type) of Float := (others => 1.0);
  ----------------------------------------------------------------------------        
  function Keypressed
          (Window_Handle : Win32.Windef.Hwnd;
           Message       : Win32.Uint;
           Wparam        : Win32.Wparam;
           Lparam        : Win32.Lparam) return Win32.Lresult is
  begin
    case Wparam is
      when  70 => The_Parameter := First;          -- f
      when  76 => The_Parameter := Last;           -- l
      when  65 => The_Parameter := Bias;           -- a
      when  77 => The_Parameter := Factor;         -- m
      when  82 => The_Color     := Bitmaps.Red;    -- r
      when  71 => The_Color     := Bitmaps.Green;  -- g
      when  66 => The_Color     := Bitmaps.Blue;   -- b
      when  48 => The_Filter(Bitmaps.Red).Bias   := 0; -- 0
                  The_Filter(Bitmaps.Green).Bias := 0;
                  The_Filter(Bitmaps.Blue).Bias  := 0;
                  The_Factor := (others => 1.0);
      when 187 =>                              -- +
        case The_Parameter is                      
          when First  => The_Filter(The_Color).First := The_Filter(The_Color).First + 1;
          when Last   => The_Filter(The_Color).Last  := The_Filter(The_Color).Last  + 1;
          when Bias   => The_Filter(The_Color).Bias  := The_Filter(The_Color).Bias  + 1;
          when Factor => The_Factor(The_Color)       := The_Factor(The_Color)       + 0.01;
        end case;            
      when 189 =>                              -- -
        case The_Parameter is
          when First  => The_Filter(The_Color).First := The_Filter(The_Color).First - 1;
          when Last   => The_Filter(The_Color).Last  := The_Filter(The_Color).Last  - 1;
          when Bias   => The_Filter(The_Color).Bias  := The_Filter(The_Color).Bias  - 1;
          when Factor => The_Factor(The_Color)       := The_Factor(The_Color)       - 0.01;
        end case;            
      when others => null;  
    end case;
    
    Timer.Set(Win32.Wparam'Image(Wparam) & ", '" & Character'Val(Wparam) & "'", 3);    
    Timer.Set("Red:" & Bitmaps.Byte'Image(The_Filter(Bitmaps.Red).First) & " .." & 
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Red).Last)  &  ", Green:" &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Green).First) & " .." &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Green).Last) & ", Blue:" &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Blue).First) & " .." &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Blue).Last) & ", Red*" &
              Float'Image(The_Factor(Bitmaps.Red)) & ", Green*" &              
              Float'Image(The_Factor(Bitmaps.Green)) & ", Blue*" &              
              Float'Image(The_Factor(Bitmaps.Blue)) & ", Red+" &              
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Red).Bias) & ", Green+" &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Green).Bias) & ", Blue+" &
              Bitmaps.Byte'Image(The_Filter(Bitmaps.Blue).Bias), 1);
    Timer.Set(Filter_Parameter_Type'Image(The_Parameter) & " " &
              Bitmaps.Color_Index_Type'Image(The_Color), 2);              
              
    return Win32.Winuser.Defwindowproc
	          (Hwnd   => Window_Handle,
	           Msg    => Message,
	           Wparam => Wparam,
	           Lparam => Lparam);
  end Keypressed;
  ----------------------------------------------------------------------------    
  procedure Filter(The_Pixel : in out Bitmaps.Pixel) is
    use Bitmaps;
  begin
    if The_Pixel(Red)   in The_Filter(Red).  First .. The_Filter(Red).  Last and then
       The_Pixel(Green) in The_Filter(Green).First .. The_Filter(Green).Last and then
       The_Pixel(Blue)  in The_Filter(Blue). First .. The_Filter(Blue). Last then

      The_Pixel(Red)   := Byte(Float(The_Pixel(Red))   * The_Factor(Red))   + The_Filter(Red).Bias;
      The_Pixel(Green) := Byte(Float(The_Pixel(Green)) * The_Factor(Green)) + The_Filter(Green).Bias;
      The_Pixel(Blue)  := Byte(Float(The_Pixel(Blue))  * The_Factor(Blue))  + The_Filter(Blue).Bias;          
    end if;                                                          
  end Filter;  
  ----------------------------------------------------------------------------
  function Models_Of(Window_Index : in Positive := 1) 
           return Models.Model_Group_Access is
    
	  Number               : constant := 20;	  
	  The_Models           : Models.Root_Model_Array      (1 .. Number);
	  The_Input_Calculation: Models.Model_Input_Procedures(1 .. Number);
	  Model_Count          : Natural := 0;               
	  ----------------------------------------------------------------------------
	  Default_X        : constant Glfloat := Screen.Width  / 2.0;         
	  Default_Y        : constant Glfloat := Screen.Height / 2.0;         
	  Default_Z        : constant Glfloat := 0.0;          
    Default_Position : constant Types.Vector_3d := (Default_X, Default_Y, Default_Z);
    Default_Scale    : constant Glfloat := Default_Y;         
    Scale            : constant Glfloat := Argument("-scale", Default_Scale);
    Attitude         : constant Glfloat := Argument("-att", 0.0);
    ----------------------------------------------------------------------------   
    procedure Create_Bitmap is
      Filename : constant String := Argument("-bmpfile", "world.bmp"); 
    begin                   
      if Has_Argument("-bmp") then 
        Win.Standard_Window.Callbacks(Win32.Winuser.Wm_Keydown) := Keypressed'access;
        Model_Count := Model_Count + 1;             
        
        if Filename = "world.bmp" then
          The_Models(Model_Count) := new Bitmaps.Model_With_Filter'
                    (Bitmaps.Model_With
                             (Filename => Filename,
                              Scale    => Argument("-scale",1.0),
                              Position => (Screen.Width / 2.0, 
                                           Screen.Height / 2.0, 
                                           0.0),
                              Attitude => Argument("-att", 0.0),
                              Filter   => Filter'access));
        else                      
          The_Models(Model_Count) := new Bitmaps.Model'
                    (Bitmaps.Model_With
                             (Filename => Filename,
                              Scale    => Argument("-scale",1.0),
                              Position => (Screen.Width / 2.0, 
                                           Screen.Height / 2.0, 
                                           0.0),
                              Attitude => Argument("-att", 0.0))); 
        end if;   
      end if;       
	  end Create_Bitmap;             
	  ----------------------------------------------------------------------------   
	  procedure Create_Earth is
	  begin                   
      if Command_Line.Earth then     
		    Model_Count := Model_Count + 1;     
        The_Models(Model_Count) := Earth.Model_With
          (Bitmap_File     => Command_Line.Earth_File,
           Longitude_Lines => 36, 
           Latitude_Lines  => 18,
           Time_Offset     => Command_Line.Time_Offset,
           Time_Factor     => gl.glfloat(Command_Line.Time_Factor),
           Latitude        => Gl.Glfloat(Command_Line.Latitude),
           Longitude       => Gl.Glfloat(Command_Line.Longitude)); 
      end if;    
    end Create_Earth;      
	  ----------------------------------------------------------------------------   
    procedure Create_Clock is
      Position   : Types.Vector_3d := Default_Position;
      Scale      : Gl.Glfloat      := Default_Scale;      
      Clock_12h  : constant Boolean := Has_Argument("-clock")    and 
                                       Command_Line.Clock = 0;
      Clocks_12h : constant Boolean := Command_Line.Clock >= 1;
      Clock_24h  : constant Boolean := Has_Argument("-clock24h") and 
                                       Command_Line.Clock_24_H = 0;
      Clocks_24h : constant Boolean := Command_Line.Clock >= 1;
      Clocks_Type: Clock.Dialface_Type := Clock.Hours_12;
    begin
      if Clock_12h then -- Create 12 h clock model
        if Clock_24h then
          Position(X) := Screen.Width / 4.0 + 5.0;
          Scale       := Default_Scale * 2.0 / 3.0;             
        end if;
        Model_Count := Model_Count + 1;
        The_Models(Model_Count) := new 
          Clock.Model'(Clock.Model_With
               (Position       => Position,
						    Attitude       => (0.0, 0.0, 0.0),
						    Scale          => Scale,
                Dialface       => Clock.Hours_12,
                Earth_Scale    => Gl.Glfloat(Command_Line.Earth_Scale),
                Time_Offset    => Command_Line.Time_Offset * 3600.0,
                Earth_Location => Command_Line.Earth_Location,
                Latitude       => Gl.Glfloat(Command_Line.Latitude),
                Longitude      => Gl.Glfloat(Command_Line.Longitude)));
      end if;
      if Clock_24h then -- Create 24 h clock model
        if Clock_12h then
          Position(X) := Screen.Width * 3.0 / 4.0 + 15.0;
        end if;
        Model_Count := Model_Count + 1;
        The_Models(Model_Count) := new 
          Clock.Model'(Clock.Model_With
               (Position       => Position,
						    Attitude       => (0.0, 0.0, 0.0),
						    Scale          => Scale,
                Dialface       => Clock.Hours_24,
                Earth_Scale    => Gl.Glfloat(Command_Line.Earth_Scale),
                Time_Offset    => Command_Line.Time_Offset * 3600.0,
                Earth_Location => Command_Line.Earth_Location,
                Latitude       => Gl.Glfloat(Command_Line.Latitude),
                Longitude      => Gl.Glfloat(Command_Line.Longitude)));
      end if;
      
      if    Clocks_12h then
        Model_Count := Model_Count + 1;
        The_Models(Model_Count) := new 
          Clock.Timezone_Models'(Clock.Model_With
               (Position       => Position,
						    Attitude       => (0.0, 0.0, 0.0),
						    Scale          => Scale,
                Dialface       => Clock.Hours_12,
                Earth_Scale    => Gl.Glfloat(Command_Line.Earth_Scale),
                Number         => Command_Line.Clock));
      elsif Clocks_24h then  
        Model_Count := Model_Count + 1;
        The_Models(Model_Count) := new 
          Clock.Timezone_Models'(Clock.Model_With
               (Position       => Position,
						    Attitude       => (0.0, 0.0, 0.0),
						    Scale          => Scale,
                Dialface       => Clock.Hours_24,
                Earth_Scale    => Gl.Glfloat(Command_Line.Earth_Scale),
                Number         => Command_Line.Clock));
      end if;          
    end Create_Clock;             
	  ----------------------------------------------------------------------------
	  procedure Create_Timer is
      Line_Index, Char_Index : Natural;
    begin    
  	  if Command_Line.Timer then -- Create timer model
        Model_Count := Model_Count + 1;
        The_Models(Model_Count) := new 
          Opengl.Timer.Model(Increment            => 1,
											       Screen_Update_Dt_Sec => 1,
											       Max_Iterations       => Positive'Last,
											       Include_Render_Time  => False,
											       Text                 => null);
        if Has_Argument("-font") then     
          for I in 0 .. 255 loop
            Line_Index := I  /  (256 / 4) + 1;
            Char_Index := I rem (256 / 4) + 1;
            Timer.Line(Line_Index)(Char_Index) := Character'Val(I);
          end loop;                     
        end if;
        if Has_Argument("-extensions") then  
          declare
            Extensions : constant String := Messages.Glgetstring(Gl.Gl_Extensions);
          begin  
            Timer.Line(3)(1 .. Extensions'Length) := Extensions;
          end;
        end if;
      end if;                               
    end Create_Timer;                 
	  ----------------------------------------------------------------------------   
--	  procedure Create_Sphere is
-- 		  Position : constant Vector_3d := (1 => Argument("-sx", Default_X),
--				                                2 => Argument("-sy", Default_Y),
--				                                3 => Argument("-sz", Default_Z));
-- 		  Attitude : constant Vector_3d := (1 => Argument("-ax", 0.0),
--				                                2 => Argument("-ay", 0.0),
--				                                3 => Argument("-az", 0.0));
--      Lat      : constant Positive := Argument("-lat", 36);                                  
--      Lon      : constant Positive := Argument("-lon", 36);                                  
--	  begin                   
--      if Has_Argument("-sphere") then     
--		    Model_Count := Model_Count + 1;     
--		    The_Models(Model_Count) := new Models.Forms.Sphere_3d'
--			      (Models.Root_Model with
--             Position           => Position,
--             Attitude           => Attitude,
--             Size_Scaling       => Scale,
--             Color              => Colors.Blue,
--             Radius             => Scale,
--             Slices             => Lon,
--             Stacks             => Lat,
--             Glu_Quadric_Object => Glu.Glunewquadric);  
--        declare
--          Sphere : Glu.Pgluquadricobj renames Models.Forms.Sphere_3d
--                   (The_Models(Model_Count).all).Glu_Quadric_Object;     
--        begin          
--	        if    Has_Argument("-fill")  then
--	          Glu.Gluquadricdrawstyle(Sphere, Glu.Glu_Fill);     
--	        elsif Has_Argument("-line")  then
--	          Glu.Gluquadricdrawstyle(Sphere, Glu.Glu_Line);     
--	        elsif Has_Argument("-sil")   then
--	          Glu.Gluquadricdrawstyle(Sphere, Glu.Glu_Silhouette);     
--	        elsif Has_Argument("-point") then
--	          Glu.Gluquadricdrawstyle(Sphere, Glu.Glu_Point);     
--          end if;  
--        end;  
--        declare
--          Coord_System : Models.Coordinate_System.Model := 
--                         Models.Coordinate_System.Standard;
--        begin
--          Coord_System.Attitude := Attitude;                 
--		      Model_Count := Model_Count + 1;     
--		      The_Models(Model_Count) := new Models.Coordinate_System.Model'(Coord_System);
--        end;    
--      end if;       
--	  end Create_Sphere;             
--	  ----------------------------------------------------------------------------
    procedure Create_Coordinate_Systems is
      Transformation_Sequence : Coordinate_System.Transformation_Sequence_Model
        (Transformation_Number => Argument("-cs", 0));
    begin           
      Coordinate_System.Initialize(Transformation_Sequence);
      Timer.Set("duration'last:" & Duration'Image(Duration'Last));
      if Transformation_Sequence.Transformation_Number > 0 then 
        Transformation_Sequence.Position := Default_Position;
        -- The first cs is not rotated, then rotate above x,y,z-axis in that sequence.                    
        for I in Transformation_Sequence.Coordinate_Systems'range loop
          Transformation_Sequence.Coordinate_Systems(I).Size_Scaling := Screen.Height / 3.0;
          Transformation_Sequence.Coordinate_Systems(I).Color := 
            Colors.Palette.Color_Array(Colors.Palette.Color_Type'Val
              (Colors.Palette.Color_Type'Pos(Colors.Palette.Red) + I - 1));
        end loop;  
        for I in X + 1 .. Z + 1 loop
          exit when I > Transformation_Sequence.Coordinate_Systems'Last;
          Transformation_Sequence.Coordinate_Systems(I).Attitude((I - 1) mod 3) := 10.0;
        end loop;  
	      Model_Count := Model_Count + 1;     
	      The_Models(Model_Count) := new 
          Coordinate_System.Transformation_Sequence_Model'(Transformation_Sequence);
      end if;       
	  end Create_Coordinate_Systems;             
	  ---------------------------------------------------------------------------    
--    procedure Create_Square is
--    begin
--      Model_Count := Model_Count + 1;     
--      The_Models(Model_Count) := new Models.Forms.Square_2d_Z'
--        (Models.Root_Model with 
--          Size_Scaling => 1.0,
--          Position     => Default_Position,
--          Attitude     => Attitude,
--          Color        => Colors.Blue,
--          Width        => Screen.Height / 3.0);      
--    end Create_Square;
	  ---------------------------------------------------------------------------    
--    procedure Create_Dashed_Arc is
--    begin
--      if Has_Argument("-dashedarc") then
--        Model_Count := Model_Count + 1;     
--        The_Models(Model_Count) := new Models.Forms.Dashed_Arc_2d_Z'
--          (Models.Root_Model with 
--            Size_Scaling => 1.0,
--            Position     => Default_Position,
--            Attitude     => Attitude,
--            Color        => Colors.Green,
--            Inner_Radius => Screen.Height / 2.0 - 1.0,
--            Outer_Radius => Screen.Height / 2.0,
--            Slices       => 2,
--            Loops        => 1,
--            Start_Angle  => -40.0,
--            End_Angle    => 220.0,
--            Dash_Angle   => 1.0,
--            Glu_Quadric_Object => Glu.Glunewquadric);
--      end if;            
--    end Create_Dashed_Arc;
--    --------------------------------------------------------------------------
--    procedure Create_Striped_Rectangle is
--    begin
--      if Has_Argument("-sr") then 
--        Model_Count := Model_Count + 1;     
--        The_Models(Model_Count) := new Models.Forms.Striped_Rectangle_2d_Z' 
--    	   	(Models.Root_Model with 
--           Size_Scaling   => 0.5 * Screen.Width,
--           Position       => (0.0,10.0,0.0),
--           Attitude       => 0.0,
--           Color          => Colors.Blue,
--           Height         => 0.4,
--           Stripe_Color   => Colors.Green,
--           Stripe_Width   => 0.2,
--           Stripe_Spacing => 0.2,
--           First_Stripe   => 0.0,
--           Stripe_Angle   => 45.0,
--           Hidden_Data    => Models.Forms.Default_Striped_Rectangle_Parts);
--        Models.Forms.Initialize(Models.Forms.Striped_Rectangle_2d_Z(The_Models(Model_Count).all));   
--      end if;
--    end Create_Striped_Rectangle;
--    --------------------------------------------------------------------------      
--	  procedure Create_Logo is
--      Logo : Models.Display_List_Model_2d_Z;
--      Bmp  : Bitmaps.Model;
--      Count: Natural := 1;
--      -------------------------------------      
--      procedure Render(Text : in String) is
--        Line : Models.Strings.String_2d := 
--    	        (Models.Root_Model with
--      	       Size_Scaling => 1.0,
--      	       Color        => Colors.White,
--      	       Position     => (Bmp.Position(X), 0.0, 0.0),        
--      	       Attitude     => 0.0,
--      	       Length       => Text'Length,
--      	       Text         => Text,
--               Alignment    => (Horizontal => Models.Strings.Center,
--                                Vertical   => Models.Strings.Top),
--      	       Font         => Fonts.Font,
--      	       Font_Size    => 3.0,
--               Hidden_Data  => Models.Strings.Default_Hidden_Data); 
--      begin
--        Count := Count + 1;
--        Models.Strings.Initialize(Line);
--        Line.Position(Y) := Bmp.Position(Y) - 
--          1.25 * Gl.Glfloat(Count) * Line.Font_Size;
--        Models.Strings.Render(Line);
--      end Render;    
--      ----------------------------------                    
--	  begin                   
--      if Has_Argument("-amstlogo") then 
--        Bmp := Bitmaps.Model_With
--              (Filename => "amst-logo.bmp",
--               Scale    => 1.0,
--               Position => (0.75 * Screen.Width, 30.0, 0.0),
--               Attitude => 0.0);
--
--        Logo.Display_List := Display_Lists.Next_List_Id;               
--        Gl.Glnewlist(Logo.Display_List, Mode => Gl.Gl_Compile);    
--        Bitmaps.Render(Bmp);
--        Render("AMST - Systemtechnik Ges.m.b.H");
--        Render("P.O.Box 3, A-5282 Ranshofen Austria");
--        Render("Phone: ++43 7722 892 0");
--        Render("Fax:   ++43 7722 892 99");
--        Render("E-Mail: office@amst.co.at");
--        Render("http:\\www.amst.co.at");
--        Gl.Glendlist;
--        
--        Model_Count := Model_Count + 1;     
--		    The_Models(Model_Count) := new Models.Display_List_Model_2d_Z'(Logo);
--      end if;       
--	  end Create_Logo;             
--	  ----------------------------------------------------------------------------   
  begin
--    Create_Logo;
    if Command_Line.Test then
  --    case Window_Index is
  --      when 1 =>
      Create_Clock;
      Create_Earth;
  --    Create_Sphere;        
  --    Create_Coordinate_Systems;            
      Create_Bitmap;
  --    Create_Square;   
  --     Create_Dashed_Arc; 
  --    Create_Striped_Rectangle;
      -- Timer has to be created and rendered as the last model. 
      -- Then the timer output contains the correct texts set by the models
      -- rendered before, otherwise the output is from the last render step.
  --      when others => null;
  --    end case;
    end if;  
    
    Create_Timer;
    
    if Model_Count = 0 then
      return null;
    else
      return new Models.Model_Group'                               
                (Model_Number      => Model_Count,
                 Render            => null,
                 Models            => The_Models           (1 .. Model_Count),
                 Input_Calculation => The_Input_Calculation(1 .. Model_Count),
                 Render_Time       => (others => 0.0),
                 Render_Counter    => 0);
    end if;                 
  end Models_Of;  
  ----------------------------------------------------------------------------
end Opengl.Test_Screen_Prototyping;

