--                    -*- Mode: Ada -*-
-- Filename        : opengl-types.adb
-- Description     : Package for general Types used for opengl-models.
-- Author          : J. Schrer
-- Created On      : 12. Feb. 1998
-- Last Modified By:
-- Last Modified On: ... Feb. 1998
-- Update Count    :
-- Status          :

with Ada.Unchecked_Deallocation;
with Opengl.Float_32.Arrays.Vector_Operations;

package body Opengl.Types is

  use Opengl.Float_32.Arrays.Vector_Operations;
  ----------------------------------------------------------------------------
  function "-" (Right : in integer_array) return integer_array is
    T : integer_array(Right'range);
  begin
    for I in Right'range loop
      T(I) := - Right(I);
    end loop;
    return T;
  end "-";
  ----------------------------------------------------------------------------
  function "*" (Left : in Gl.Glfloat; Right : in Points_2d ) return Points_2d is
    T : Points_2d (Right'range);
  begin
  	for I in Right'range loop
      T(I) := Right(I) * Left;
    end loop;
		return T;
  end "*";
  ----------------------------------------------------------------------------
  function "*" (Left : in Points_2d;  Right : in Gl.Glfloat) return Points_2d is
  begin  
    return Right * Left;
  end "*";
  ----------------------------------------------------------------------------
  function "/" (Left : in Points_2d;  Right : in Gl.Glfloat) return Points_2d is
    T : Points_2d (Left'range);
  begin
  	for I in Left'range loop
      T(I) := Left(I) / Right;
    end loop;
		return T;
  end "/";
  ----------------------------------------------------------------------------
  function "-" (Right : in Points_2d) return Points_2d is
    T : Points_2d(Right'range);
  begin
  	for I in Right'range loop
      T(I) := - Right(I);
    end loop;
    return T;
  end "-";
  ----------------------------------------------------------------------------  
  function "+" (Left : in Vector_2d; Right : in Points_2d) return Points_2d is
    T : Points_2d (Right'range);
  begin
  	for I in Right'range loop
      T(I) := Right(I) + Left;
    end loop;
    return T;
  end "+";
  ----------------------------------------------------------------------------  
  function "+" (Left : in Points_2d; Right : in Vector_2d) return Points_2d is
  begin  
    return Right + Left;
  end "+";
  ----------------------------------------------------------------------------  
  function "-" (Left  : in Points_2d; Right : in Vector_2d) return Points_2d is
  begin
    return (- Right) + Left;
  end "-";
  ----------------------------------------------------------------------------  
  function "*" (Left : in Gl.Glfloat; Right : in Points_3d ) return Points_3d is
    T : Points_3d (Right'range);
  begin
  	for I in Right'range loop
      T(I) := Right(I) * Left;
    end loop;
		return T;
  end "*";
  ----------------------------------------------------------------------------
  function "*" (Left : in Points_3d;  Right : in Gl.Glfloat) return Points_3d is
  begin  
    return Right * Left;
  end "*";
  ----------------------------------------------------------------------------
  function "/" (Left : in Points_3d;  Right : in Gl.Glfloat) return Points_3d is
    T : Points_3d (Left'range);
  begin
  	for I in Left'range loop
      T(I) := Left(I) / Right;
    end loop;
		return T;
  end "/";
  ----------------------------------------------------------------------------
  function "-" (Right : in Points_3d) return Points_3d is
    T : Points_3d(Right'range);
  begin
  	for I in Right'range loop
      T(I) := - Right(I);
    end loop;
    return T;
  end "-";
  ----------------------------------------------------------------------------  
  function "+" (Left : in Vector_3d; Right : in Points_3d) return Points_3d is
    T : Points_3d (Right'range);
  begin
  	for I in Right'range loop
      T(I) := Right(I) + Left;
    end loop;
    return T;
  end "+";
  ----------------------------------------------------------------------------  
  function "+" (Left : in Points_3d; Right : in Vector_3d) return Points_3d is
  begin  
    return Right + Left;
  end "+";
  ----------------------------------------------------------------------------
  function "-" (Left  : in Points_3d; Right : in Vector_3d) return Points_3d is
  begin
    return (- Right) + Left;
  end "-";
  ----------------------------------------------------------------------------  
  function Rotate(The_Points : in Points_2d) return Points_2d is
    The : Points_2d(The_Points'range);
  begin
    for I in The'range loop
      The(I)(X) := The_Points(I)(Y);
      The(I)(Y) := The_Points(I)(X);
    end loop;
    return The;  
  end Rotate;
  ----------------------------------------------------------------------------           
  procedure Free(Item : in out Points_2d_Access) is

    procedure Deallocate is new Ada.Unchecked_Deallocation
      (Object => Points_2d, Name => Points_2d_Access);

    pragma Inline(Deallocate);
  begin
    Deallocate(Item);
  end Free;
  ----------------------------------------------------------------------------
  procedure Free(Item : in out Points_3d_Access) is

    procedure Deallocate is new Ada.Unchecked_Deallocation
      (Object => Points_3d, Name => Points_3d_Access);

    pragma Inline(Deallocate);
  begin
    Deallocate(Item);
  end Free;
  ----------------------------------------------------------------------------
end Opengl.Types;
