--                    -*- Mode: Ada -*-
-- Filename        : opengl-types.ads
-- Description     : Package for general types used for opengl-models.
-- Author          : J. Schrer
-- Created On      : 12. Feb. 1998
-- Last Modified By:
-- Last Modified On:
-- Update Count    :
-- Status          :

with Opengl.Gl;
with Opengl.Float_32.Arrays.Types;

package Opengl.Types is

  -- Auxiliary declarations --------------------------------------------------

  subtype Axis_Type    is Positive  range X .. Z;
  subtype Xy_Axis_Type is Axis_Type range X .. Y;

  type Gluint_Array  is array(Integer range <>) of aliased Gl.Gluint;
  
  type Integer_Array is array(Integer range <>) of aliased Integer;
  
  subtype Float_Vector        is Opengl.Float_32.Arrays.Types.Vector;
  subtype Float_Vector_Access is Opengl.Float_32.Arrays.Types.Vector_Access;
  subtype Float_Matrix        is Opengl.Float_32.Arrays.Types.Matrix;
  subtype Float_Matrix_Access is Opengl.Float_32.Arrays.Types.Matrix_Access;
  
  subtype Vector_2d is Float_Vector(X .. Y);
  subtype Vector_3d is Float_Vector(X .. Z);

  subtype Transformation_Matrix_2d is Float_Matrix(X .. Y, X .. Y);
  subtype Transformation_Matrix_3d is Float_Matrix(X .. Z, X .. Z);

  -- For coordinate transformation


  type Points_2d is array (Positive range <>) of Vector_2d;
  type Points_3d is array (Positive range <>) of Vector_3d;

  type Points_2d_Access is access all Points_2d;
  type Points_3d_Access is access all Points_3d;

  function "-" (Right : in integer_array) return integer_array;

  function "*" (Left  : in Gl.Glfloat; Right : in Points_2d ) return Points_2d;
  function "*" (Left  : in Points_2d;  Right : in Gl.Glfloat) return Points_2d;
  
  function "/" (Left  : in Points_2d;  Right : in Gl.Glfloat) return Points_2d;
  
  function "-" (Right : in Points_2d) return Points_2d;
  function "+" (Left  : in Vector_2d; Right : in Points_2d) return Points_2d;
  function "+" (Left  : in Points_2d; Right : in Vector_2d) return Points_2d;
  function "-" (Left  : in Points_2d; Right : in Vector_2d) return Points_2d;
  
  function "*" (Left  : in Gl.Glfloat; Right : in Points_3d ) return Points_3d;
  function "*" (Left  : in Points_3d;  Right : in Gl.Glfloat) return Points_3d;
  
  function "/" (Left  : in Points_3d;  Right : in Gl.Glfloat) return Points_3d;
  
  function "-" (Right : in Points_3d) return Points_3d;
  function "+" (Left  : in Vector_3d; Right : in Points_3d) return Points_3d;
  function "+" (Left  : in Points_3d; Right : in Vector_3d) return Points_3d;
  function "-" (Left  : in Points_3d; Right : in Vector_3d) return Points_3d;

  function Rotate(The_Points : in Points_2d) return Points_2d;
    
  -- For polygons

  procedure Free(Item : in out Points_2d_Access);
  procedure Free(Item : in out Points_3d_Access);

  type Rectangle is new Vector_2d;

  Default_Rectangle : constant Rectangle :=
    (X => 320.0, Y => 240.0);

  -- For rectangle dimension with width (x) and height (y).

  type Pixel_Point is array(X .. Y) of Natural;

  type Screen_Rectangle is record
    Lower_Left  : Vector_2d := (X => 0.0,   Y => 0.0);
    Upper_Right : Vector_2d := (X => 320.0, Y => 240.0);
  end record;

  Default_Screen_Rectangle : Screen_Rectangle;

  type Pixel_Rectangle is record
    Lower_Left  : Pixel_Point := (X => 0, Y => 0);
    Upper_Right : Pixel_Point := (X => 0, Y => 0);
  end record;

  -- For rectangles defined by two points.

  type Color_Type is record
    Red   : aliased Gl.Glfloat range 0.0 .. 1.0 := 0.0;
    Green : aliased Gl.Glfloat range 0.0 .. 1.0 := 0.0;
    Blue  : aliased Gl.Glfloat range 0.0 .. 1.0 := 0.0;
    Alpha : aliased Gl.Glfloat range 0.0 .. 1.0 := 1.0;
  end record;

  -- The default color is black, default alpha is 1.0
  
  type Color_Array is array(Positive range <>) of Color_Type;
  
end Opengl.Types;
