--                              -*- Mode: Ada -*-
-- Filename        : opengl-utils.ads
-- Description     : Package for utilities to simplify the use of the
--                   thin Opengl-bindings
-- Author          : J. Schrer
-- Created On      : 17 Nov. 1997
-- Last Modified By: J. Schrer
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Opengl.Gl;
--with Opengl.Glaux;
with Opengl.Types;
with Opengl.Float_32.Text_Io;
--with System;

package Opengl.Utils is
  
  function Number_Array_Of(First           : in Integer; 
                           Increment       : in Integer; 
                           Number          : in Natural;
                           Index_First     : in Positive := 1;
                           Index_Increment : in Positive := 1)
    return Types.Integer_Array;
    
  -- Function to be used to generate digit_value-arrays in the ticks_with_digits
  -- subclasses of the packages Models.Dialfaces, models.linear_scales, 
  -- Models.Linear_Dialfaces.   
  --  
  -- First           : The first number to be generated.
  -- Increment       : The increment of the generated numbers.
  -- Number          : The lenght of the returned integer_array(1 .. Number).
  -- Index_First     : integer_array(Index_First) = First.
  -- Index_Increment : integer_array(Index_First + i * Index_Increment) = 
  --                   First + i * Increment; i = 0 .. n.
  --                   All other numbers in integer_array are set to 
  --                   Invalid_Number.  
  
        
	procedure Resize_Window(New_Width, New_Height : in Natural);

  -- Standard resize algorithm (see e.g. OpenGL superbible).

  function Center_Of(Rectangle : in Types.Screen_Rectangle)
           return Types.Vector_2d;

  function Center_Of(Rectangle : in Types.Pixel_Rectangle)
           return Types.Pixel_Point;

  -- Returns the center of a rectangle.

  function "+" (Left  : in Types.Screen_Rectangle;
                Right : in Types.Vector_2d)        return Types.Screen_Rectangle;

  function "+" (Left  : in Types.Vector_2d;
                Right : in Types.Screen_Rectangle) return Types.Screen_Rectangle;

  function "+" (Left  : in Types.Rectangle;
                Right : in Types.Vector_2d)        return Types.Screen_Rectangle;

  function "+" (Left  : in Types.Vector_2d;
                Right : in Types.Rectangle)        return Types.Screen_Rectangle;

  -- To translate a rectangle.                      

  function Screen_Point_Of(The_Point         : in Types.Pixel_Point)
           return Types.Vector_2d;


  type Rounding is (Down, Up);

  function Pixel_Point_Of(The_Point          : in Types.Vector_2d;
                          Round              : in Rounding := Up)
           return Types.Pixel_Point;

  function Screen_Rectangle_Of(The_Rectangle : in Types.Pixel_Rectangle)
           return Types.Screen_Rectangle;

  function Pixel_Rectangle_Of (The_Rectangle : in Types.Screen_Rectangle;
                               Next_Power_2  : in Boolean := False)
           return Types.Pixel_Rectangle;

  -- Some conversions.           
  ----------------------------------------------------------------------------
  -- Auxiliary Functions

  function To_Math_Angle          (Compass_Angle       : in Gl.Glfloat)
           return Gl.Glfloat;

  function To_Compass_Angle       (Math_Angle          : in Gl.Glfloat)
           return Gl.Glfloat;

  function To_Delta_Math_Angle    (Delta_Compass_Angle : in Gl.Glfloat)
           return Gl.Glfloat;

  function To_Delta_Compass_Angle (Delta_Math_Angle    : in Gl.Glfloat)
           return Gl.Glfloat;

  function Limitation (Input_Value : in Gl.Glfloat;
                       Lower_Limit : in Gl.Glfloat;
                       Upper_Limit : in Gl.Glfloat ) return Gl.Glfloat;

  pragma Inline(To_Math_Angle,
                To_Compass_Angle,
                To_Delta_Math_Angle,
                To_Delta_Compass_Angle,
                Limitation);


  procedure Set_Integer_Image(Image      : in out String;
                              Value      : in     Gl.Glfloat;
                              Right_Text : in     String := "");
 
  --  pragma Inline(Set_Integer_Image);                              
  
  -- Image is filled with integer'image(value). The right part of image is 
  -- filled with right_text. If image'length is to small for both parts
  -- (i.e. the value is too large), the left part of image is filled with a
  -- sequence of '-' characters.
                              
  procedure Set_Float_Image(Image      : in out String;
                            Value      : in     Gl.Glfloat;
                            Aft        : in     Ada.Text_Io.Field := 
                                                Opengl.Float_32.Text_Io.Default_Aft;
                            Exp        : in     Ada.Text_Io.Field := 0;
                            Right_Text : in     String := "");

  --  pragma Inline(Set_Float_Image);                              
  
  -- Uses Opengl.Float_32.Text_Io.Put with given parameters.
  -- When Ada.Text_Io.Layout_Error occurs, fills image with a string like
  -- "---.--"     when image'length =  6, aft = 2 and exp = 0 or                                                        
  -- "---.--E+00" when image'length = 10, aft = 2 and exp = 2.                                                        
                            
  function Position_Of(Value : in Gl.Glfloat;
                       Slope : in Gl.Glfloat;
                       Min   : in Gl.Glfloat;
                       Max   : in Gl.Glfloat) return Gl.Glfloat;

  pragma Inline(Position_Of);                              

  -- Returns value * slope if value in min .. max else min * slope or max * slope.       
  
  function Background_Color return Types.Color_Type;
  
  -- Reads the value -background from command_line or returns the default
  -- background color black.                                                                                                                                 
  ----------------------------------------------------------------------------           

  -- To make opengl-programming with callbacks more flexible.
  -- Callback functions are given to opengl e.g. by the procedure
  -- glaux.mainloop(p1), where p1 is of type Glaux.Auxmainproc.
  -- If the actual procedure p1 is declared in library level package
  -- (no package that is local to a procedure), a call
  --
  -- glaux.mainloop(p1'access) 
  --
  -- may be used. But if p1 is declared in a procedure, than the 
  -- access attribute is not valid, as the accessibility level of the
  -- accessibility level of the subprogram p1 is statically deeper than that 
  -- of the expected type Glaux.Auxmainproc. So the address shall be used
  --
  -- glaux.mainloop(opengl.utilities.convert(p1'address)) 


--  function To_Auxmainproc   (The_Procedure : in System.Address) 
--           return Glaux.Auxmainproc;
--  
--  function To_Auxexposeproc (The_Procedure : in System.Address)
--           return Glaux.Auxexposeproc;
--
--  function To_Auxreshapeproc(The_Procedure : in System.Address)
--           return Glaux.Auxreshapeproc;
--
--  function To_Auxidleproc   (The_Procedure : in System.Address)
--           return Glaux.Auxidleproc;
--  
--  function To_Auxkeyproc    (The_Procedure : in System.Address)
--           return Glaux.Auxkeyproc;
--  
--  function To_Auxmouseproc  (The_Procedure : in System.Address)
--           return Glaux.Auxmouseproc;

end Opengl.Utils;
