--                    -*- Mode: Ada -*-
-- Filename        : opengl-messages.adb
-- Description     : Package for messages generated by opengl-programs. 
-- Author          : J. Schrer
-- Created On      : 12. Feb. 1998
-- Last Modified By:
-- Last Modified On: ... Feb. 1998
-- Update Count    : 
-- Status          :

with Portable.Operating_System;
with Portable.Utils;
with Ada.Calendar;
with Ada.Command_Line;
with Interfaces.C;
with Win32.Winbase;
with Win32_Addons.Glu;
with Windows.Messages;
with Windows.Utils;
with Utilities.Calendar_Utilities;
with Opengl.Glu;
with Opengl.Timer;
with opengl.logging;

package body Opengl.Messages is
  
  use type Opengl.Gl.Glenum;
  
  package Calendar_Utilities renames Utilities.Calendar_Utilities;         
  
  Newline : constant String := Portable.Operating_System.New_Line;
  ----------------------------------------------------------------------------
  function Windows_Error_Message return String is
    Windows_Error_Code : constant Interfaces.C.Unsigned_Long :=
                                  Win32.Winbase.Getlasterror;
  begin                         
    return "Win-Error   : " & 
           Interfaces.C.Unsigned_Long'Image(Windows_Error_Code) & ": " & 
           Windows.Messages.Message_String(Message_Id => Windows_Error_Code);    
  end Windows_Error_Message;         
  ----------------------------------------------------------------------------
  function Opengl_Error_Message return String is
    Opengl_Error_Code : constant Gl.Glenum := Gl.Glgeterror;
  begin
    if Opengl_Error_Code = Gl.Gl_No_Error then
      return "";
    else
      return "Opengl-error: " & Gl.Glenum'Image(Opengl_Error_Code) & ": " & 
             Error_String(Opengl_Error_Code);
    end if;
  end Opengl_Error_Message;         
  ----------------------------------------------------------------------------
  function Error_Message(Subprogram, 
                         Message      : in String) return String is
  begin
    return "Executable  : " & Ada.Command_Line.Command_Name & Newline & 
           "Subprogram  : " & Subprogram                    & Newline &
           "Message     : " & Message                       & Newline &
           Opengl_Error_Message                             & Newline &
           Windows_Error_Message;
  end Error_Message;
  ----------------------------------------------------------------------------   
  procedure Log(The_Exception : in Ada.Exceptions.Exception_Occurrence) is
    
    Time_Text: constant String := Calendar_Utilities.Image_Of(Ada.Calendar.Clock) & 
                           ", " & Ada.Exceptions.Exception_Name(The_Exception);
    Ada_Text : constant String := Portable.Utils.Exception_Information
                                  (The_Exception);
    Ogl_Text : constant String := Opengl_Error_Message;
    Win_Text : constant String := Windows_Error_Message;
  begin
    Timer.Set(Time_Text, 4);
    if Ada_Text'Length > Timer.Info_Text'Length then
      Timer.Set(Ada_Text(1 .. Timer.Info_Text'Length), 3);    
      if Ada_Text'Length > 2 * Timer.Info_Text'Length then
        Timer.Set(Ada_Text(Timer.Info_Text'Length + 1 .. 
                       2 * Timer.Info_Text'Length), 2);
        Timer.Set(Ada_Text(2 * Timer.Info_Text'Length + 1 .. Ada_Text'Last), 1);            
      else
        Timer.Set(Ada_Text(Timer.Info_Text'Length + 1 .. Ada_Text'Last), 2);    
        Timer.Set(Ogl_Text, 1);        
      end if;        
    else      
      Timer.Set(Ada_Text, 3);    
      Timer.Set(Ogl_Text, 2);
      Timer.Set(Win_Text, 1);
    end if;
    Logging.put(Time_Text);
    Logging.put(Ada_Text);
    Logging.put(Ogl_Text);
    Logging.put(Win_Text);
  end Log;
  ----------------------------------------------------------------------------
  function Glgetstring(Name : Gl.Glenum) return String is
  begin
    return Windows.Utils.To_Ada(Gl.Glgetstring(Name));
  end Glgetstring;  
  ----------------------------------------------------------------------------
  function Glugetstring(Name : Gl.Glenum) return String is
  begin
    return Windows.Utils.To_Ada(Win32_Addons.Glu.Glugetstring(Name));
  end Glugetstring;  
  ----------------------------------------------------------------------------
  function Error_String(Error_Code : in Gl.Glenum := Gl.Glgeterror) 
           return String is
  begin
    return Windows.Utils.To_Ada(Glu.Gluerrorstring(Error_Code));
  end Error_String;            
  ----------------------------------------------------------------------------
end Opengl.Messages;

