--                    -*- Mode: Ada -*-
-- Filename        : opengl-win_stuff.ads
-- Description     : Package where resources are calculated that need the
--                   Win32Ada binding and are used in other packages than 
--                   opengl.win. The Win32Ada packages needed are only 
--                   "withed" in the package body, to isolate applications
--                   from them.
-- Author          : J. Schrer
-- Created On      : 10. Feb. 1998
-- Last Modified By: J. Schrer
-- Last Modified On:
-- Update Count    : 1
-- Status          :

with Opengl.Gl;
with Win32.Windef;

package Opengl.Win_Stuff is

  subtype Font_Weight_Type is Natural    range   0 .. 900;
  -- See Win32.WinGDI.

  function Font_Weight_Bold   return Font_Weight_Type; 
  function Font_Weight_Normal return Font_Weight_Type;
  function Font_Weight_Thin   return Font_Weight_Type; 

  function Default_Font_Name  return String;
  
  subtype Screen_Size_In_Mm is Gl.Glfloat range 0.0 .. 1000.0;
  
  function Screen_Width_In_Mm  return Screen_Size_In_Mm; -- default is 320.0
  function Screen_Height_In_Mm return Screen_Size_In_Mm; -- default is 240.0
  
  function Screen_Pixel_Width  return Natural;
  function Screen_Pixel_Height return Natural;
  
  function Max_Clip_Planes return Natural;
  
  function Os_Info return String;
  
  -- Returns operating-system name and version number.
  
  Invalid_Area : constant Win32.Windef.Rect := 
                         (Left   => Win32.Long'Last,
                          Top    => Win32.Long'Last,
                          Right  => Win32.Long'Last,
                          Bottom => Win32.Long'Last);
                          
  function Area_Of(The_Window : in Win32.Windef.Hwnd) return Win32.Windef.Rect;
  
  -- If the win32-function fails returns Invalid_Area.
    
end Opengl.Win_Stuff;

